@@ -29,31 +29,30 @@ namespace compiler {
 class t_program;
 
 /**
- *  class t_const
+ * Represents a const definition.
  *
- * A const is a constant value defined across languages that has a type and
- * a value. The trick here is that the declared type might not match the type
- * of the value object, since that is not determined until after parsing the
- * whole thing out.
+ * A const definition introduces a named constant in target languages that has
+ * a type and a value. The trick here is that the declared type might not match
+ * the type of the value object, since that is not determined until after
+ * parsing the whole thing out.
  */
 class t_const final : public t_named {
  public:
   /**
    * Constructor for t_const
    *
-   * @param program - An entire thrift program
-   * @param type    - A thrift type
-   * @param name    - The name of the constant variable
-   * @param value   - The constant value
+   * @param program  - An entire thrift program
+   * @param type_ref - A thrift type
+   * @param name     - The name of the constant variable
+   * @param value    - The constant value
    */
   t_const(
       const t_program* program,
-      t_type_ref type,
+      t_type_ref type_ref,
       std::string name,
       std::unique_ptr<t_const_value> value)
-      : t_named(std::move(name)),
-        program_(program),
-        type_(std::move(type)),
+      : t_named(program, std::move(name)),
+        type_ref_(std::move(type_ref)),
         value_(std::move(value)) {
     // value->get_owner() is set when rhs is referencing another constant.
     if (value_ && value_->get_owner() == nullptr) {
@@ -61,17 +60,21 @@ class t_const final : public t_named {
     }
   }
 
-  const t_program* program() const { return program_; }
-  const t_type_ref& type() const { return type_; }
+  const t_type* type() const { return type_ref_.get_type(); }
+
+  t_type_ref type_ref() const { return type_ref_; }
+  t_type_ref& type_ref() { return type_ref_; }
+
   const t_const_value* value() const { return value_.get(); }
+  t_const_value* value() { return value_.get(); }
+
   const t_const_value& get_value_from_structured_annotation(
       const char* key) const;
   const t_const_value* get_value_from_structured_annotation_or_null(
       const char* key) const;
 
  private:
-  const t_program* const program_;
-  t_type_ref type_;
+  t_type_ref type_ref_;
 
   std::unique_ptr<t_const_value> value_;
 
@@ -91,12 +94,8 @@ class t_const final : public t_named {
 
   std::unique_ptr<t_const> clone() const {
     return std::make_unique<t_const>(
-        get_program(), get_type(), name(), value_->clone());
+        program(), type(), name(), value_->clone());
   }
-
-  const t_program* get_program() const { return program_; }
-  const t_type* get_type() const { return type_.get_type(); }
-  t_const_value* get_value() const { return value_.get(); }
 };
 
 } // namespace compiler
