@@ -45,10 +45,9 @@ enum class t_error_safety {
 };
 
 /**
- * A thrift exception.
+ * Represents an exception definition.
  *
- * Exceptions are structured, like unions and structs, but can only
- * be used in error-specific contexts.
+ * Exceptions are similar to structs but can only be used in throws clauses.
  */
 // TODO(afuller): Inherit from t_structured instead.
 class t_exception : public t_struct {
@@ -64,6 +63,16 @@ class t_exception : public t_struct {
   t_error_safety safety() const { return safety_; }
   void set_safety(t_error_safety safety) { safety_ = safety; }
 
+  const t_field* get_message_field() const {
+    for (const auto* field : get_members()) {
+      if (field->find_structured_annotation_or_null(kExceptionMessageUri)) {
+        return field;
+      }
+    }
+    const auto* value = find_annotation_or_null("message");
+    return value ? get_field_by_name(*value) : nullptr;
+  }
+
  private:
   t_error_kind kind_{};
   t_error_blame blame_{};
@@ -72,7 +81,6 @@ class t_exception : public t_struct {
   // TODO(afuller): Remove everything below this comment. It is only provided
   // for backwards compatibility.
  public:
-  bool is_xception() const override { return is_exception(); }
   bool is_exception() const override { return true; }
 
  private:
