@@ -85,14 +85,13 @@ class t_scope {
         redefined_enum_values_.end();
   }
 
-  std::string get_fully_qualified_enum_value_names(const std::string& name);
+  std::string get_fully_qualified_enum_value_names(
+      const std::string& name) const;
 
   // Get a (poetically unresolved) reference to given type, declared in the
   // given program.
   t_type_ref ref_type(
-      const t_program& program,
-      const std::string& name,
-      const source_range& range);
+      t_program& program, const std::string& name, const source_range& range);
 
   node_list_view<t_placeholder_typedef> placeholder_typedefs() {
     return placeholder_typedefs_;
