@@ -14,6 +14,7 @@
  * limitations under the License.
  */
 
+#include <thrift/compiler/generate/common.h>
 #include <thrift/compiler/generate/mstch_objects.h>
 
 #include <fmt/core.h>
@@ -191,29 +192,13 @@ mstch::node mstch_const_map_element::element_value() {
       element_.second, context_, pos_, current_const_, expected_types_.second);
 }
 
-mstch::node mstch_const_value::value() {
-  switch (type_) {
-    case cv::CV_DOUBLE:
-      return fmt::format("{}", const_value_->get_double());
-    case cv::CV_BOOL:
-      return std::to_string(const_value_->get_bool());
-    case cv::CV_INTEGER:
-      return std::to_string(const_value_->get_integer());
-    case cv::CV_STRING:
-      return const_value_->get_string();
-    default:
-      return mstch::node();
-  }
-}
-
 mstch::node mstch_const_value::integer_value() {
   return type_ == cv::CV_INTEGER ? std::to_string(const_value_->get_integer())
                                  : mstch::node();
 }
 
 mstch::node mstch_const_value::double_value() {
-  return type_ == cv::CV_DOUBLE ? fmt::format("{}", const_value_->get_double())
-                                : mstch::node();
+  return type_ == cv::CV_DOUBLE ? const_value_->get_double() : mstch::node();
 }
 
 mstch::node mstch_const_value::bool_value() {
@@ -252,38 +237,15 @@ mstch::node mstch_const_value::string_value() {
   if (type_ != cv::CV_STRING) {
     return {};
   }
+  return get_escaped_string(const_value_->get_string());
+}
 
-  const std::string& str = const_value_->get_string();
-  std::string escaped;
-  escaped.reserve(str.size());
-  for (unsigned char c : str) {
-    switch (c) {
-      case '\\':
-        escaped.append("\\\\");
-        break;
-      case '"':
-        escaped.append("\\\"");
-        break;
-      case '\r':
-        escaped.append("\\r");
-        break;
-      case '\n':
-        escaped.append("\\n");
-        break;
-      default:
-        if (c < 0x20 || c >= 0x80) {
-          // Use octal escape sequences because they are the most portable
-          // across languages. Hexadecimal ones have a problem of consuming
-          // all hex digits after \x in C++, e.g. \xcafefe is a single escape
-          // sequence.
-          escaped.append(fmt::format("\\{:03o}", c));
-        } else {
-          escaped.push_back(c);
-        }
-        break;
-    }
+mstch::node mstch_const_value::string_length() {
+  if (type_ != cv::CV_STRING) {
+    return {};
   }
-  return escaped;
+
+  return const_value_->get_string().length();
 }
 
 mstch::node mstch_const_value::list_elems() {
@@ -460,6 +422,8 @@ mstch::node mstch_field::idl_type() {
       case t_type::type::t_program:
         return std::nullopt;
     }
+    // unneccessary, but prevents a GCC warning
+    return std::nullopt;
   });
 
   if (idl_type == std::nullopt) {
@@ -722,21 +686,6 @@ mstch::node mstch_program::constants() {
   return a;
 }
 
-mstch::node mstch_program::interned_values() {
-  mstch::array a;
-  const auto& container = program_->intern_list();
-  for (size_t i = 0, size = container.size(); i < size; ++i) {
-    a.push_back(context_.const_factory->make_mstch_object(
-        container[i].get(),
-        context_,
-        mstch_element_position(i, size),
-        container[i].get(),
-        container[i]->type(),
-        nullptr));
-  }
-  return a;
-}
-
 } // namespace compiler
 } // namespace thrift
 } // namespace apache
