@@ -429,17 +429,10 @@ class mstch_program : public mstch_base {
             {"program:typedefs?", &mstch_program::has_typedefs},
             {"program:constants?", &mstch_program::has_constants},
             {"program:thrift_uris?", &mstch_program::has_thrift_uris},
-            {"program:interned_values?", &mstch_program::has_interned_values},
-            {"program:num_interned_values",
-             &mstch_program::num_interned_values},
-            {"program:interned_values", &mstch_program::interned_values},
         });
     register_has_option("program:frozen?", "frozen");
     register_has_option("program:json?", "json");
     register_has_option("program:any?", "any");
-    register_has_option(
-        "program:unstructured_annotations_in_metadata?",
-        "deprecated_unstructured_annotations_in_metadata");
   }
 
   virtual std::string get_program_namespace(const t_program*) { return {}; }
@@ -456,13 +449,11 @@ class mstch_program : public mstch_base {
   mstch::node has_enums() { return !program_->enums().empty(); }
   mstch::node has_structs() {
     return !program_->structs_and_unions().empty() ||
-        !program_->xceptions().empty();
+        !program_->exceptions().empty();
   }
   mstch::node has_services() { return !program_->services().empty(); }
   mstch::node has_typedefs() { return !program_->typedefs().empty(); }
-  mstch::node has_constants() {
-    return !program_->consts().empty() || context_.options.count("interning");
-  }
+  mstch::node has_constants() { return !program_->consts().empty(); }
   mstch::node has_unions() {
     auto& structs = program_->structs_and_unions();
     return std::any_of(
@@ -476,12 +467,6 @@ class mstch_program : public mstch_base {
   mstch::node typedefs();
   mstch::node constants();
 
-  mstch::node has_interned_values() { return !program_->intern_list().empty(); }
-  mstch::node num_interned_values() {
-    return static_cast<int>(program_->intern_list().size());
-  }
-  mstch::node interned_values();
-
  protected:
   const t_program* program_;
 };
@@ -533,10 +518,14 @@ class mstch_service : public mstch_base {
     }
 
     // Collect performed interactions and cache them.
+    std::set<const t_interaction*> seen;
     for (const auto* function : get_functions()) {
       if (const auto& interaction = function->interaction()) {
-        interactions_.insert(
-            dynamic_cast<const t_interaction*>(interaction.get_type()));
+        auto* ptr = dynamic_cast<const t_interaction*>(interaction.get_type());
+        if (!seen.insert(ptr).second) {
+          continue; // Already seen this interaction.
+        }
+        interactions_.push_back(ptr);
       }
     }
   }
@@ -590,7 +579,7 @@ class mstch_service : public mstch_base {
 
  protected:
   const t_service* service_;
-  std::set<const t_interaction*> interactions_;
+  std::vector<const t_interaction*> interactions_;
 
   // If `service_` is really an interaction, `containing_service_` is the
   // service it belongs to.
@@ -1055,7 +1044,7 @@ class mstch_field : public mstch_base {
         });
   }
   mstch::node name() { return field_->get_name(); }
-  mstch::node key() { return std::to_string(field_->get_key()); }
+  mstch::node key() { return field_->get_key(); }
   mstch::node value();
   mstch::node type();
   /**
@@ -1067,7 +1056,7 @@ class mstch_field : public mstch_base {
    * element types, etc.).
    */
   mstch::node idl_type();
-  mstch::node index() { return std::to_string(pos_.index); }
+  mstch::node index() { return pos_.index; }
   mstch::node is_terse() {
     return field_->qualifier() == t_field_qualifier::terse;
   }
@@ -1276,7 +1265,7 @@ class mstch_enum_value : public mstch_base {
         });
   }
   mstch::node name() { return enum_value_->get_name(); }
-  mstch::node value() { return std::to_string(enum_value_->get_value()); }
+  mstch::node value() { return enum_value_->get_value(); }
 
  protected:
   const t_enum_value* enum_value_;
@@ -1353,7 +1342,6 @@ class mstch_const_value : public mstch_base {
             {"value:list?", &mstch_const_value::is_list},
             {"value:container?", &mstch_const_value::is_container},
             {"value:empty_container?", &mstch_const_value::is_empty_container},
-            {"value:value", &mstch_const_value::value},
             {"value:integer_value", &mstch_const_value::integer_value},
             {"value:double_value", &mstch_const_value::double_value},
             {"value:bool_value", &mstch_const_value::bool_value},
@@ -1361,6 +1349,7 @@ class mstch_const_value : public mstch_base {
             {"value:nonzero?", &mstch_const_value::is_non_zero},
             {"value:enum_name", &mstch_const_value::enum_name},
             {"value:string_value", &mstch_const_value::string_value},
+            {"value:string_length", &mstch_const_value::string_length},
             {"value:list_elements", &mstch_const_value::list_elems},
             {"value:map_elements", &mstch_const_value::map_elems},
             {"value:const_struct", &mstch_const_value::const_struct},
@@ -1398,7 +1387,6 @@ class mstch_const_value : public mstch_base {
     return (type_ == cv::CV_MAP && const_value_->get_map().empty()) ||
         (type_ == cv::CV_LIST && const_value_->get_list().empty());
   }
-  mstch::node value();
   mstch::node integer_value();
   mstch::node double_value();
   mstch::node bool_value();
@@ -1406,6 +1394,7 @@ class mstch_const_value : public mstch_base {
   mstch::node is_non_zero();
   mstch::node enum_name();
   mstch::node string_value();
+  mstch::node string_length();
   mstch::node list_elems();
   mstch::node map_elems();
   mstch::node const_struct();
