@@ -145,6 +145,8 @@ class py3_mstch_program : public mstch_program {
              &py3_mstch_program::includeNamespaces},
             {"program:cppIncludes", &py3_mstch_program::getCppIncludes},
             {"program:containerTypes", &py3_mstch_program::getContainerTypes},
+            {"program:hasPyContainerTypes",
+             &py3_mstch_program::hasPyContainerTypes},
             {"program:customTemplates", &py3_mstch_program::getCustomTemplates},
             {"program:customTypes", &py3_mstch_program::getCustomTypes},
             {"program:moveContainerTypes",
@@ -185,6 +187,15 @@ class py3_mstch_program : public mstch_program {
 
   mstch::node getContainerTypes() { return make_mstch_types(containers_); }
 
+  mstch::node hasPyContainerTypes() {
+    for (const auto* ttype : containers_) {
+      if (ttype->is_list()) {
+        return true;
+      }
+    }
+    return false;
+  }
+
   mstch::node getCppIncludes() {
     mstch::array a;
     if (program_->language_includes().count("cpp")) {
@@ -427,7 +438,17 @@ class py3_mstch_service : public mstch_service {
              &py3_mstch_service::get_lifecycle_functions},
             {"service:supportedFunctionsWithLifecycle",
              &py3_mstch_service::get_supported_functions_with_lifecycle},
+            {"service:supportedInteractions",
+             &py3_mstch_service::get_supported_interactions},
         });
+
+    // Collect supported interactions
+    for (const auto* function : get_functions()) {
+      if (function->is_interaction_constructor()) {
+        supported_interactions_.insert(dynamic_cast<const t_interaction*>(
+            function->interaction().get_type()));
+      }
+    }
   }
 
   mstch::node isExternalProgram() { return prog_ != service_->program(); }
@@ -479,8 +500,13 @@ class py3_mstch_service : public mstch_service {
     return make_mstch_functions(funcs, service_);
   }
 
+  mstch::node get_supported_interactions() {
+    return make_mstch_interactions(supported_interactions_, service_);
+  }
+
  protected:
   const t_program* prog_;
+  std::set<const t_interaction*> supported_interactions_;
 };
 
 class py3_mstch_interaction : public py3_mstch_service {
@@ -541,11 +567,7 @@ class py3_mstch_function : public mstch_function {
 
 class py3_mstch_type : public mstch_type {
  public:
-  struct CachedProperties {
-    const std::string cppTemplate;
-    std::string cppType;
-    std::string flatName;
-  };
+  using CachedProperties = apache::thrift::compiler::py3::CachedProperties;
 
   struct data {
     const t_program* program;
@@ -562,7 +584,6 @@ class py3_mstch_type : public mstch_type {
       : mstch_type(type->get_true_type(), ctx, pos),
         prog_(d.program),
         cached_props_(get_cached_props(type, d)) {
-    strip_cpp_comments_and_newlines(cached_props_.cppType);
     register_cached_methods(
         this,
         {
@@ -613,19 +634,19 @@ class py3_mstch_type : public mstch_type {
     return fmt::format("_{}", fmt::join(get_type_py3_namespace(), "_"));
   }
 
-  mstch::node flatName() { return cached_props_.flatName; }
+  mstch::node flatName() { return cached_props_.flatName(); }
 
   mstch::node cppNamespaces() {
     return create_string_array(get_type_cpp2_namespace());
   }
 
-  mstch::node cppTemplate() { return cached_props_.cppTemplate; }
+  mstch::node cppTemplate() { return cached_props_.cppTemplate(); }
 
   mstch::node cythonTemplate() { return to_cython_template(); }
 
   mstch::node isDefaultTemplate() { return is_default_template(); }
 
-  mstch::node cppType() { return cached_props_.cppType; }
+  mstch::node cppType() { return cached_props_.cppType(); }
 
   mstch::node cythonType() { return to_cython_type(); }
 
@@ -653,8 +674,10 @@ class py3_mstch_type : public mstch_type {
 
   mstch::node hasCustomTypeBehavior() { return has_custom_type_behavior(); }
 
+  // as used, non-simple types live in shared_ptr in container conversions
   mstch::node isSimple() {
-    return (type_->is_primitive_type() || type_->is_enum()) &&
+    return (type_->is_primitive_type() || type_->is_enum() ||
+            type_->is_list()) &&
         !has_custom_type_behavior();
   }
 
@@ -664,33 +687,17 @@ class py3_mstch_type : public mstch_type {
         resolved_type_->is_exception();
   }
 
-  const std::string& get_flat_name() const { return cached_props_.flatName; }
+  const std::string& get_flat_name() const { return cached_props_.flatName(); }
 
   void set_flat_name(const std::string& extra) {
-    std::string custom_prefix;
-    if (!is_default_template()) {
-      custom_prefix = to_cython_template() + "__";
-    } else {
-      if (cached_props_.cppType != "") {
-        custom_prefix = to_cython_type() + "__";
-      }
-    }
-    const t_program* typeProgram = type_->program();
-    if (typeProgram && typeProgram != prog_) {
-      custom_prefix += typeProgram->name() + "_";
-    }
-    custom_prefix += extra;
-    cached_props_.flatName = std::move(custom_prefix);
+    cached_props_.set_flat_name(prog_, type_, extra);
   }
 
   bool is_default_template() const {
-    return (!type_->is_container() && cached_props_.cppTemplate == "") ||
-        (type_->is_list() && cached_props_.cppTemplate == "std::vector") ||
-        (type_->is_set() && cached_props_.cppTemplate == "std::set") ||
-        (type_->is_map() && cached_props_.cppTemplate == "std::map");
+    return cached_props_.is_default_template(type_);
   }
 
-  bool has_custom_cpp_type() const { return cached_props_.cppType != ""; }
+  bool has_custom_cpp_type() const { return cached_props_.cppType() != ""; }
 
  protected:
   const t_program* get_type_program() const {
@@ -711,32 +718,10 @@ class py3_mstch_type : public mstch_type {
   }
 
   std::string to_cython_template() const {
-    // handle special built-ins first:
-    if (cached_props_.cppTemplate == "std::vector") {
-      return "vector";
-    } else if (cached_props_.cppTemplate == "std::set") {
-      return "cset";
-    } else if (cached_props_.cppTemplate == "std::map") {
-      return "cmap";
-    }
-    // then default handling:
-    return boost::algorithm::replace_all_copy(
-        cached_props_.cppTemplate, "::", "_");
+    return cached_props_.to_cython_template();
   }
 
-  std::string to_cython_type() const {
-    if (cached_props_.cppType == "") {
-      return "";
-    }
-    std::string cython_type = cached_props_.cppType;
-    boost::algorithm::replace_all(cython_type, "::", "_");
-    boost::algorithm::replace_all(cython_type, "<", "_");
-    boost::algorithm::replace_all(cython_type, ">", "");
-    boost::algorithm::replace_all(cython_type, " ", "");
-    boost::algorithm::replace_all(cython_type, ", ", "_");
-    boost::algorithm::replace_all(cython_type, ",", "_");
-    return cython_type;
-  }
+  std::string to_cython_type() const { return cached_props_.to_cython_type(); }
 
   bool is_integer() const { return type_->is_any_int() || type_->is_byte(); }
 
@@ -758,10 +743,10 @@ class py3_mstch_type : public mstch_type {
 
   bool has_cython_type() const { return !type_->is_container(); }
 
-  bool is_iobuf() const { return cached_props_.cppType == "folly::IOBuf"; }
+  bool is_iobuf() const { return cached_props_.cppType() == "folly::IOBuf"; }
 
   bool is_iobuf_ref() const {
-    return cached_props_.cppType == "std::unique_ptr<folly::IOBuf>";
+    return cached_props_.cppType() == "std::unique_ptr<folly::IOBuf>";
   }
 
   bool is_flexible_binary() const {
@@ -769,8 +754,8 @@ class py3_mstch_type : public mstch_type {
         !is_iobuf_ref() &&
         // We know that folly::fbstring is completely substitutable for
         // std::string and it's a common-enough type to special-case:
-        cached_props_.cppType != "folly::fbstring" &&
-        cached_props_.cppType != "::folly::fbstring";
+        cached_props_.cppType() != "folly::fbstring" &&
+        cached_props_.cppType() != "::folly::fbstring";
   }
 
   bool has_custom_type_behavior() const {
@@ -820,7 +805,7 @@ class py3_mstch_struct : public mstch_struct {
         py3_fields_.end());
   }
 
-  mstch::node getSize() { return std::to_string(py3_fields_.size()); }
+  mstch::node getSize() { return py3_fields_.size(); }
 
   mstch::node isStructOrderable() {
     return cpp2::is_orderable(*struct_) &&
@@ -842,7 +827,7 @@ class py3_mstch_struct : public mstch_struct {
   mstch::node exceptionMessage() {
     const auto* message_field =
         dynamic_cast<const t_exception&>(*struct_).get_message_field();
-    return message_field ? message_field->name() : "";
+    return message_field ? py3::get_py3_name(*message_field) : "";
   }
 
   mstch::node py3_fields() { return make_mstch_fields(py3_fields_); }
@@ -1287,7 +1272,7 @@ class t_mstch_py3_generator : public t_mstch_generator {
         enum_member_union_field_names_validator::validate_union);
   }
 
-  enum TypesFile { IsTypesFile, NotTypesFile };
+  enum class TypesFile { IsTypesFile, NotTypesFile };
 
  private:
   bool should_resolve_typedefs() const override { return true; }
@@ -1362,7 +1347,7 @@ void t_mstch_py3_generator::generate_file(
     const std::filesystem::path& base = {}) {
   auto program = get_program();
   const auto& name = program->name();
-  if (is_types_file == IsTypesFile) {
+  if (is_types_file == TypesFile::IsTypesFile) {
     mstch_context_.options["is_types_file"] = "";
   } else {
     mstch_context_.options.erase("is_types_file");
@@ -1372,12 +1357,12 @@ void t_mstch_py3_generator::generate_file(
 }
 
 void t_mstch_py3_generator::generate_types() {
-  std::vector<std::string> autoMigrateFiles{
+  std::vector<std::string> autoMigrateFilesWithTypeContext{
       "types.py",
-      // without auto_migrate, .pxd contains just bindings of cpp thrift types
-      "types.pxd",
+  };
+
+  std::vector<std::string> autoMigrateFilesNoTypeContext{
       "metadata.py",
-      "builders.py", // TODO
   };
 
   std::vector<std::string> converterFiles{
@@ -1392,14 +1377,15 @@ void t_mstch_py3_generator::generate_types() {
   };
 
   std::vector<std::string> cythonFilesNoTypeContext{
-      "types_reflection.py",
-      "types_empty.pyx",
-      "types_fields.pxd",
-      "types_fields.pyx",
       "builders.py",
+      "containers_FBTHRIFT_ONLY_DO_NOT_USE.py",
       "metadata.pxd",
       "metadata.pyi",
       "metadata.pyx",
+      "types_empty.pyx",
+      "types_fields.pxd",
+      "types_fields.pyx",
+      "types_reflection.py",
   };
 
   std::vector<std::string> cppFilesWithTypeContext{
@@ -1411,28 +1397,31 @@ void t_mstch_py3_generator::generate_types() {
       "metadata.cpp",
   };
 
+  if (has_option("enable_container_pickling_DO_NOT_USE")) {
+    generate_file("__init__.py", TypesFile::IsTypesFile, generateRootPath_);
+  }
   for (const auto& file : converterFiles) {
-    generate_file(file, IsTypesFile, generateRootPath_);
+    generate_file(file, TypesFile::IsTypesFile, generateRootPath_);
   }
   // - if auto_migrate is present, generate types.pxd, and types.py
   // - else, just generate normal cython files
-  if (has_option("auto_migrate")) {
-    for (const auto& file : autoMigrateFiles) {
-      generate_file(file, IsTypesFile, generateRootPath_);
-    }
-  } else {
-    for (const auto& file : cythonFilesWithTypeContext) {
-      generate_file(file, IsTypesFile, generateRootPath_);
-    }
-    for (const auto& file : cppFilesWithTypeContext) {
-      generate_file(file, IsTypesFile);
-    }
-    for (const auto& file : cythonFilesNoTypeContext) {
-      generate_file(file, NotTypesFile, generateRootPath_);
-    }
-    for (const auto& file : cppFilesWithNoTypeContext) {
-      generate_file(file, NotTypesFile);
-    }
+  for (const auto& file : autoMigrateFilesWithTypeContext) {
+    generate_file(file, TypesFile::IsTypesFile, generateRootPath_);
+  }
+  for (const auto& file : autoMigrateFilesNoTypeContext) {
+    generate_file(file, TypesFile::NotTypesFile, generateRootPath_);
+  }
+  for (const auto& file : cythonFilesWithTypeContext) {
+    generate_file(file, TypesFile::IsTypesFile, generateRootPath_);
+  }
+  for (const auto& file : cppFilesWithTypeContext) {
+    generate_file(file, TypesFile::IsTypesFile);
+  }
+  for (const auto& file : cythonFilesNoTypeContext) {
+    generate_file(file, TypesFile::NotTypesFile, generateRootPath_);
+  }
+  for (const auto& file : cppFilesWithNoTypeContext) {
+    generate_file(file, TypesFile::NotTypesFile);
   }
 }
 
@@ -1463,7 +1452,7 @@ void t_mstch_py3_generator::generate_services() {
       "clients_wrapper.pxd",
       "services_wrapper.pxd",
       "services_reflection.pxd",
-      "services_reflection.pyx",
+      "services_reflection.py",
   };
 
   std::vector<std::string> cppFiles{
@@ -1478,20 +1467,17 @@ void t_mstch_py3_generator::generate_services() {
   // - if auto_migrate is present, generate py3_clients and clients.px
   // - if auto_migrate isn't present, just generate all the normal files
 
-  if (has_option("auto_migrate")) {
-    for (const auto& file : pythonFiles) {
-      generate_file(file, NotTypesFile, generateRootPath_);
-    }
-  } else {
-    for (const auto& file : normalCythonFiles) {
-      generate_file(file, NotTypesFile, generateRootPath_);
-    }
-    for (const auto& file : cppFiles) {
-      generate_file(file, NotTypesFile);
-    }
-    for (const auto& file : cythonFiles) {
-      generate_file(file, NotTypesFile, generateRootPath_);
-    }
+  for (const auto& file : pythonFiles) {
+    generate_file(file, TypesFile::NotTypesFile, generateRootPath_);
+  }
+  for (const auto& file : normalCythonFiles) {
+    generate_file(file, TypesFile::NotTypesFile, generateRootPath_);
+  }
+  for (const auto& file : cppFiles) {
+    generate_file(file, TypesFile::NotTypesFile);
+  }
+  for (const auto& file : cythonFiles) {
+    generate_file(file, TypesFile::NotTypesFile, generateRootPath_);
   }
 }
 
