@@ -153,6 +153,8 @@ class python_mstch_program : public mstch_program {
         this,
         {
             {"program:is_types_file?", &python_mstch_program::is_types_file},
+            {"program:generate_mutable_types",
+             &python_mstch_program::generate_mutable_types},
         });
     register_has_option("program:import_static?", "import_static");
     gather_included_program_namespaces();
@@ -223,6 +225,10 @@ class python_mstch_program : public mstch_program {
     return module_path_array(adapter_type_hint_modules_);
   }
 
+  mstch::node generate_mutable_types() {
+    return !get_option("generate_mutable_types").empty();
+  }
+
  protected:
   struct Namespace {
     std::string ns;
@@ -596,7 +602,6 @@ class python_mstch_type : public mstch_type {
     register_cached_methods(
         this,
         {
-            {"type:module_path", &python_mstch_type::module_path},
             {"type:program_name", &python_mstch_type::program_name},
             {"type:metadata_path", &python_mstch_type::metadata_path},
             {"type:py3_namespace", &python_mstch_type::py3_namespace},
@@ -608,14 +613,18 @@ class python_mstch_type : public mstch_type {
     register_volatile_methods(
         this,
         {
+            {"type:module_path", &python_mstch_type::module_path},
             {"type:need_module_path?", &python_mstch_type::need_module_path},
         });
   }
 
   mstch::node module_path() {
+    std::string_view types_import_path =
+        get_option("generate_mutable_types").empty() ? ".thrift_types"
+                                                     : ".thrift_mutable_types";
     return get_py3_namespace_with_name_and_prefix(
-               get_type_program(), get_option("root_module_prefix")) +
-        ".thrift_types";
+               get_type_program(), get_option("root_module_prefix"))
+        .append(types_import_path);
   }
 
   mstch::node program_name() { return get_type_program()->name(); }
@@ -733,7 +742,7 @@ class python_mstch_struct : public mstch_struct {
   mstch::node exception_message() {
     const auto* message_field =
         dynamic_cast<const t_exception&>(*struct_).get_message_field();
-    return message_field ? message_field->name() : "";
+    return message_field ? py3::get_py3_name(*message_field) : "";
   }
 
   mstch::node adapter() {
@@ -744,7 +753,7 @@ class python_mstch_struct : public mstch_struct {
     return ::apache::thrift::compiler::generate_legacy_api(*struct_);
   }
 
-  mstch::node fields_size() { return std::to_string(struct_->fields().size()); }
+  mstch::node fields_size() { return struct_->fields().size(); }
 
  private:
   const t_const* adapter_annotation_;
@@ -947,6 +956,7 @@ class t_mstch_python_generator : public t_mstch_generator {
   void generate_file(
       const std::string& file,
       TypesFile is_types_file,
+      bool generate_mutable_types,
       const std::filesystem::path& base);
   void set_types_file(bool val);
   void generate_types();
@@ -1184,6 +1194,7 @@ std::filesystem::path t_mstch_python_generator::package_to_path() {
 void t_mstch_python_generator::generate_file(
     const std::string& file,
     TypesFile is_types_file,
+    bool generate_mutable_types,
     const std::filesystem::path& base = {}) {
   auto program = get_program();
   const auto& name = program->name();
@@ -1192,21 +1203,44 @@ void t_mstch_python_generator::generate_file(
   } else {
     mstch_context_.options.erase("is_types_file");
   }
+  if (generate_mutable_types) {
+    mstch_context_.options["generate_mutable_types"] = "yes";
+  } else {
+    mstch_context_.options.erase("generate_mutable_types");
+  }
   auto mstch_program = make_mstch_program_cached(program, mstch_context_);
   render_to_file(mstch_program, file, base / name / file);
 }
 
 void t_mstch_python_generator::generate_types() {
-  generate_file("thrift_types.py", IsTypesFile, generate_root_path_);
-  generate_file("thrift_types.pyi", IsTypesFile, generate_root_path_);
+  bool generate_mutable_types = false;
+  generate_file(
+      "thrift_types.py",
+      IsTypesFile,
+      generate_mutable_types,
+      generate_root_path_);
+  generate_file(
+      "thrift_types.pyi",
+      IsTypesFile,
+      generate_mutable_types,
+      generate_root_path_);
   if (has_option("experimental_generate_mutable_types")) {
-    generate_file("thrift_mutable_types.py", IsTypesFile, generate_root_path_);
-    generate_file("thrift_mutable_types.pyi", IsTypesFile, generate_root_path_);
+    generate_mutable_types = true;
+    generate_file(
+        "thrift_mutable_types.py",
+        IsTypesFile,
+        generate_mutable_types,
+        generate_root_path_);
+    generate_file(
+        "thrift_mutable_types.pyi",
+        IsTypesFile,
+        generate_mutable_types,
+        generate_root_path_);
   }
 }
 
 void t_mstch_python_generator::generate_metadata() {
-  generate_file("thrift_metadata.py", IsTypesFile, generate_root_path_);
+  generate_file("thrift_metadata.py", IsTypesFile, false, generate_root_path_);
 }
 
 void t_mstch_python_generator::generate_clients() {
@@ -1216,7 +1250,18 @@ void t_mstch_python_generator::generate_clients() {
     return;
   }
 
-  generate_file("thrift_clients.py", NotTypesFile, generate_root_path_);
+  generate_file(
+      "thrift_clients.py",
+      NotTypesFile,
+      false, // generate_mutable_types
+      generate_root_path_);
+  if (has_option("experimental_generate_mutable_types")) {
+    generate_file(
+        "thrift_mutable_clients.py",
+        NotTypesFile,
+        true, // generate_mutable_types
+        generate_root_path_);
+  }
 }
 
 void t_mstch_python_generator::generate_services() {
@@ -1225,7 +1270,7 @@ void t_mstch_python_generator::generate_services() {
     // services.
     return;
   }
-  generate_file("thrift_services.py", NotTypesFile, generate_root_path_);
+  generate_file("thrift_services.py", NotTypesFile, false, generate_root_path_);
 }
 
 } // namespace
