@@ -30,6 +30,7 @@
 #include <fmt/core.h>
 
 #include <thrift/compiler/ast/t_struct.h>
+#include <thrift/compiler/gen/cpp/namespace_resolver.h>
 #include <thrift/compiler/generate/mstch_objects.h>
 #include <thrift/compiler/generate/t_mstch_generator.h>
 #include <thrift/compiler/lib/rust/uri.h>
@@ -37,7 +38,6 @@
 #include <thrift/compiler/lib/uri.h>
 #include <thrift/compiler/sema/ast_validator.h>
 #include <thrift/compiler/sema/diagnostic_context.h>
-#include <thrift/compiler/sema/explicit_include_validator.h>
 
 namespace apache {
 namespace thrift {
@@ -55,14 +55,17 @@ struct rust_codegen_options {
   // currently being generated.
   std::string label;
 
-  // Key: package name according to Thrift.
-  // Value: determines the path used by generated code to name the crate.
-  std::map<std::string, rust_crate> cratemap;
+  // Index that can resolve a Thrift t_program to a Rust crate name.
+  rust_crate_index crate_index;
 
   // Whether to emit derive(Serialize, Deserialize).
   // Enabled by `--gen rust:serde`.
   bool serde = false;
 
+  // Whether to emit derive(Valuable).
+  // Enabled by `--gen rust:valuable`
+  bool valuable = false;
+
   // Whether fields w/optional values of None should
   // be skipped during serialization. Enabled w/ `--gen
   // rust:skip_none_serialization` Note: `rust:serde` must also be set for this
@@ -117,182 +120,6 @@ std::string quoted_rust_doc(const t_named* named_node) {
   return quote(doc.substr(first, last - first + 1), true);
 }
 
-bool can_derive_ord(const t_type* type) {
-  type = type->get_true_type();
-  if (type->is_string() || type->is_binary() || type->is_bool() ||
-      type->is_byte() || type->is_i16() || type->is_i32() || type->is_i64() ||
-      type->is_enum() || type->is_void()) {
-    return true;
-  }
-  if (type->has_annotation("rust.ord") ||
-      type->find_structured_annotation_or_null(kRustOrdUri)) {
-    return true;
-  }
-  if (type->is_list()) {
-    auto elem_type = dynamic_cast<const t_list*>(type)->get_elem_type();
-    return elem_type && can_derive_ord(elem_type);
-  }
-  return false;
-}
-
-bool validate_rust_serde(const t_node& node) {
-  const std::string* ann = node.find_annotation_or_null("rust.serde");
-
-  return ann == nullptr || *ann == "true" || *ann == "false";
-}
-
-bool rust_serde_enabled(
-    const rust_codegen_options& options, const t_node& node) {
-  const std::string* ann = node.find_annotation_or_null("rust.serde");
-
-  if (ann == nullptr) {
-    return options.serde;
-  } else if (*ann == "true") {
-    return true;
-  } else if (*ann == "false") {
-    return false;
-  } else {
-    throw std::runtime_error("rust.serde should be `true` or `false`");
-  }
-}
-
-std::string get_types_import_name(
-    const t_program* program, const rust_codegen_options& options) {
-  if (program == options.current_program) {
-    return options.current_crate + "::types";
-  }
-
-  auto program_name = program->name();
-  auto crate = options.cratemap.find(program_name);
-  if (crate == options.cratemap.end()) {
-    return program_name + "__types";
-  } else if (crate->second.name == "crate") {
-    return crate->second.import_name() + "::types";
-  } else {
-    return crate->second.import_name();
-  }
-}
-
-std::string get_client_import_name(
-    const t_program* program, const rust_codegen_options& options) {
-  if (program == options.current_program) {
-    return options.current_crate + "::client";
-  }
-
-  auto program_name = program->name();
-  auto crate = options.cratemap.find(program_name);
-  if (crate == options.cratemap.end()) {
-    return program_name + "__clients";
-  } else if (crate->second.name == "crate") {
-    return crate->second.import_name() + "::client";
-  }
-
-  std::string absolute_crate_name = "::" + crate->second.name + "_clients";
-  if (crate->second.multifile_module) {
-    return absolute_crate_name + "::" + mangle(*crate->second.multifile_module);
-  } else {
-    return absolute_crate_name;
-  }
-}
-
-std::string get_server_import_name(
-    const t_program* program, const rust_codegen_options& options) {
-  if (program == options.current_program) {
-    return options.current_crate;
-  }
-
-  auto program_name = program->name();
-  auto crate = options.cratemap.find(program_name);
-  if (crate == options.cratemap.end()) {
-    return program_name + "__services";
-  } else if (crate->second.name == "crate") {
-    return crate->second.import_name();
-  }
-
-  std::string absolute_crate_name = "::" + crate->second.name + "_services";
-  if (crate->second.multifile_module) {
-    return absolute_crate_name + "::" + mangle(*crate->second.multifile_module);
-  } else {
-    return absolute_crate_name;
-  }
-}
-
-std::string get_mock_import_name(
-    const t_program* program, const rust_codegen_options& options) {
-  if (program == options.current_program) {
-    return options.current_crate;
-  }
-
-  auto program_name = program->name();
-  auto crate = options.cratemap.find(program_name);
-  if (crate == options.cratemap.end()) {
-    return program_name + "__mocks";
-  } else if (crate->second.name == "crate") {
-    return crate->second.import_name();
-  }
-
-  std::string absolute_crate_name = "::" + crate->second.name + "_mocks";
-  if (crate->second.multifile_module) {
-    return absolute_crate_name + "::" + mangle(*crate->second.multifile_module);
-  } else {
-    return absolute_crate_name;
-  }
-}
-
-// Path to the crate root of the given service's mocks crate. Unlike
-// `get_mock_import_name`, for multifile Thrift libraries the module name is not
-// included here.
-std::string get_mock_crate(
-    const t_program* program, const rust_codegen_options& options) {
-  if (program == options.current_program) {
-    return "crate";
-  }
-
-  auto program_name = program->name();
-  auto crate = options.cratemap.find(program_name);
-  if (crate == options.cratemap.end()) {
-    return program_name + "__mocks";
-  } else if (crate->second.name == "crate") {
-    return "crate";
-  } else {
-    return "::" + crate->second.name + "_mocks";
-  }
-}
-
-std::string multifile_module_name(const t_program* program) {
-  const std::string& namespace_rust = program->get_namespace("rust");
-
-  // If source file has `namespace rust cratename.modulename` then modulename.
-  auto separator = namespace_rust.find('.');
-  if (separator != std::string::npos) {
-    return namespace_rust.substr(separator + 1);
-  }
-
-  // Otherwise, the module is named after the source file, modulename.thrift.
-  return mangle(program->name());
-}
-
-bool node_is_boxed(const t_named& node) {
-  return node.has_annotation("rust.box") || node.has_annotation("thrift.box") ||
-      node.find_structured_annotation_or_null(kBoxUri) ||
-      node.find_structured_annotation_or_null(kRustBoxUri);
-}
-
-bool node_is_arced(const t_named& node) {
-  return node.has_annotation("rust.arc") ||
-      node.find_structured_annotation_or_null(kRustArcUri);
-}
-
-FieldKind field_kind(const t_named& node) {
-  if (node_is_arced(node)) {
-    return FieldKind::Arc;
-  }
-  if (node_is_boxed(node)) {
-    return FieldKind::Box;
-  }
-  return FieldKind::Inline;
-}
-
 std::string get_type_annotation(const t_named* node) {
   if (const t_const* annot =
           node->find_structured_annotation_or_null(kRustTypeUri)) {
@@ -327,6 +154,204 @@ bool has_newtype_annotation(const t_named* node) {
   return false;
 }
 
+bool can_derive_ord(const t_type* type) {
+  bool has_custom_type_annotation = has_type_annotation(type);
+
+  type = type->get_true_type();
+  if (type->is_string() || type->is_binary() || type->is_bool() ||
+      type->is_byte() || type->is_i16() || type->is_i32() || type->is_i64() ||
+      type->is_enum() || type->is_void()) {
+    return true;
+  }
+  if (type->has_annotation("rust.ord") ||
+      type->find_structured_annotation_or_null(kRustOrdUri)) {
+    return true;
+  }
+  if (type->is_list()) {
+    auto elem_type = dynamic_cast<const t_list*>(type)->get_elem_type();
+    return elem_type && can_derive_ord(elem_type);
+  }
+  // We can implement Ord on BTreeMap (the default map type) if both the key and
+  // value implement Eq.
+  if (type->is_map() && !has_custom_type_annotation) {
+    auto map_type = dynamic_cast<const t_map*>(type);
+    auto key_elem_type = map_type->get_key_type();
+    auto val_elem_type = map_type->get_val_type();
+
+    return key_elem_type && val_elem_type && can_derive_ord(key_elem_type) &&
+        can_derive_ord(val_elem_type);
+  }
+  return false;
+}
+
+bool validate_rust_serde(const t_named& node) {
+  if (const std::string* ann = node.find_annotation_or_null("rust.serde")) {
+    return ann == nullptr || *ann == "true" || *ann == "false";
+  }
+  // The structued form `@rust.Serde { enabled = ... }` if it exists, does not
+  // require further validation.
+  return true;
+}
+
+bool rust_serde_enabled(
+    const rust_codegen_options& options, const t_named& node) {
+  if (const t_const* annot =
+          node.find_structured_annotation_or_null(kRustSerdeUri)) {
+    return get_annotation_property_bool(annot, "enabled");
+  }
+
+  if (const std::string* ann = node.find_annotation_or_null("rust.serde")) {
+    if (*ann == "true") {
+      return true;
+    }
+    if (*ann == "false") {
+      return false;
+    }
+    throw std::runtime_error("rust.serde should be `true` or `false`");
+  }
+
+  return options.serde;
+}
+
+std::string get_types_import_name(
+    const t_program* program, const rust_codegen_options& options) {
+  if (program == options.current_program) {
+    return options.current_crate + "::types";
+  }
+
+  auto crate = options.crate_index.find(program);
+  if (!crate) {
+    return program->name() + "__types";
+  } else if (crate->dependency_path.empty()) {
+    return crate->import_name(program) + "::types";
+  } else {
+    return crate->import_name(program);
+  }
+}
+
+std::string get_client_import_name(
+    const t_program* program, const rust_codegen_options& options) {
+  if (program == options.current_program) {
+    return options.current_crate + "::client";
+  }
+
+  auto crate = options.crate_index.find(program);
+  if (!crate) {
+    return program->name() + "__clients";
+  } else if (crate->dependency_path.empty()) {
+    return crate->import_name(program) + "::client";
+  }
+
+  std::string path;
+  for (const auto& dep : crate->dependency_path) {
+    path += path.empty() ? "::" : "::__dependencies::";
+    path += dep + "_clients";
+  }
+
+  if (crate->multifile) {
+    path += "::" + multifile_module_name(program);
+  }
+
+  return path;
+}
+
+std::string get_server_import_name(
+    const t_program* program, const rust_codegen_options& options) {
+  if (program == options.current_program) {
+    return options.current_crate;
+  }
+
+  auto crate = options.crate_index.find(program);
+  if (!crate) {
+    return program->name() + "__services";
+  } else if (crate->dependency_path.empty()) {
+    return crate->import_name(program);
+  }
+
+  std::string path;
+  for (const auto& dep : crate->dependency_path) {
+    path += path.empty() ? "::" : "::__dependencies::";
+    path += dep + "_services";
+  }
+
+  if (crate->multifile) {
+    path += "::" + multifile_module_name(program);
+  }
+
+  return path;
+}
+
+std::string get_mock_import_name(
+    const t_program* program, const rust_codegen_options& options) {
+  if (program == options.current_program) {
+    return options.current_crate;
+  }
+
+  auto crate = options.crate_index.find(program);
+  if (!crate) {
+    return program->name() + "__mocks";
+  } else if (crate->dependency_path.empty()) {
+    return crate->import_name(program);
+  }
+
+  std::string path;
+  for (const auto& dep : crate->dependency_path) {
+    path += path.empty() ? "::" : "::__dependencies::";
+    path += dep + "_mocks";
+  }
+
+  if (crate->multifile) {
+    path += "::" + multifile_module_name(program);
+  }
+
+  return path;
+}
+
+// Path to the crate root of the given service's mocks crate. Unlike
+// `get_mock_import_name`, for multifile Thrift libraries the module name is not
+// included here.
+std::string get_mock_crate(
+    const t_program* program, const rust_codegen_options& options) {
+  if (program == options.current_program) {
+    return "crate";
+  }
+
+  auto crate = options.crate_index.find(program);
+  if (!crate) {
+    return program->name() + "__mocks";
+  } else if (crate->dependency_path.empty()) {
+    return "crate";
+  }
+
+  std::string path;
+  for (const auto& dep : crate->dependency_path) {
+    path += path.empty() ? "::" : "::__dependencies::";
+    path += dep + "_mocks";
+  }
+  return path;
+}
+
+bool node_is_boxed(const t_named& node) {
+  return node.has_annotation("rust.box") || node.has_annotation("thrift.box") ||
+      node.find_structured_annotation_or_null(kBoxUri) ||
+      node.find_structured_annotation_or_null(kRustBoxUri);
+}
+
+bool node_is_arced(const t_named& node) {
+  return node.has_annotation("rust.arc") ||
+      node.find_structured_annotation_or_null(kRustArcUri);
+}
+
+FieldKind field_kind(const t_named& node) {
+  if (node_is_arced(node)) {
+    return FieldKind::Arc;
+  }
+  if (node_is_boxed(node)) {
+    return FieldKind::Box;
+  }
+  return FieldKind::Inline;
+}
+
 void parse_include_srcs(
     mstch::array& elements, std::optional<std::string> const& include_srcs) {
   if (!include_srcs) {
@@ -620,11 +645,16 @@ class rust_mstch_program : public mstch_program {
     register_cached_methods(
         this,
         {
+            {"program:direct_dependencies?",
+             &rust_mstch_program::rust_has_direct_dependencies},
+            {"program:direct_dependencies",
+             &rust_mstch_program::rust_direct_dependencies},
             {"program:types", &rust_mstch_program::rust_types},
             {"program:clients", &rust_mstch_program::rust_clients},
             {"program:nonexhaustiveStructs?",
              &rust_mstch_program::rust_nonexhaustive_structs},
             {"program:serde?", &rust_mstch_program::rust_serde},
+            {"program:valuable?", &rust_mstch_program::rust_valuable},
             {"program:skip_none_serialization?",
              &rust_mstch_program::rust_skip_none_serialization},
             {"program:multifile?", &rust_mstch_program::rust_multifile},
@@ -633,7 +663,6 @@ class rust_mstch_program : public mstch_program {
              &rust_mstch_program::rust_client_package},
             {"program:includes", &rust_mstch_program::rust_includes},
             {"program:label", &rust_mstch_program::rust_label},
-            {"program:namespace", &rust_mstch_program::rust_namespace},
             {"program:nonstandardTypes",
              &rust_mstch_program::rust_nonstandard_types},
             {"program:nonstandardFields",
@@ -672,6 +701,23 @@ class rust_mstch_program : public mstch_program {
         "deprecated_default_enum_min_i32");
   }
 
+  mstch::node rust_has_direct_dependencies() {
+    return !options_.crate_index.direct_dependencies().empty();
+  }
+
+  mstch::node rust_direct_dependencies() {
+    mstch::array direct_dependencies;
+    for (auto crate : options_.crate_index.direct_dependencies()) {
+      mstch::map dependency;
+      dependency["dependency:name"] =
+          mangle_crate_name(crate->dependency_path[0]);
+      dependency["dependency:name_unmangled"] = crate->dependency_path[0];
+      dependency["dependency:label"] = crate->label;
+      direct_dependencies.push_back(std::move(dependency));
+    }
+    return direct_dependencies;
+  }
+
   mstch::node rust_types() {
     auto types = "::" + options_.types_crate;
     if (options_.multifile_mode) {
@@ -696,7 +742,7 @@ class rust_mstch_program : public mstch_program {
         return true;
       }
     }
-    for (t_exception* strct : program_->xceptions()) {
+    for (t_exception* strct : program_->exceptions()) {
       if (!strct->has_annotation("rust.exhaustive") &&
           !strct->find_structured_annotation_or_null(kRustExhaustiveUri)) {
         return true;
@@ -708,6 +754,7 @@ class rust_mstch_program : public mstch_program {
   mstch::node rust_skip_none_serialization() {
     return options_.skip_none_serialization;
   }
+  mstch::node rust_valuable() { return options_.valuable; }
   mstch::node rust_multifile() { return options_.multifile_mode; }
   mstch::node rust_crate() {
     if (options_.multifile_mode) {
@@ -730,20 +777,12 @@ class rust_mstch_program : public mstch_program {
     if (program_ == options_.current_program) {
       return options_.label;
     }
-    auto crate = options_.cratemap.find(program_->name());
-    if (crate != options_.cratemap.end()) {
-      return crate->second.label;
+    auto crate = options_.crate_index.find(program_);
+    if (crate) {
+      return crate->label;
     }
     return false;
   }
-  mstch::node rust_namespace() {
-    auto program_name = program_->name();
-    auto crate = options_.cratemap.find(program_name);
-    if (crate != options_.cratemap.end()) {
-      return crate->second.name;
-    }
-    return program_name;
-  }
   template <typename F>
   void foreach_field(F&& f) const {
     for (const t_structured* strct : program_->structs_and_unions()) {
@@ -980,8 +1019,6 @@ class rust_mstch_service : public mstch_service {
          {"service:requestContext?", &rust_mstch_service::rust_request_context},
          {"service:extendedClients",
           &rust_mstch_service::rust_extended_clients},
-         {"service:extendedServers",
-          &rust_mstch_service::rust_extended_servers},
          {"service:docs?", &rust_mstch_service::rust_has_doc},
          {"service:docs", &rust_mstch_service::rust_doc},
          {"service:program_name", &rust_mstch_service::program_name}});
@@ -1004,13 +1041,13 @@ class rust_mstch_service : public mstch_service {
     if (service_->has_annotation("rust.mod")) {
       return service_->get_annotation("rust.mod");
     } else if (
-        const t_const* annot =
+        const t_const* annot_mod =
             service_->find_structured_annotation_or_null(kRustModUri)) {
-      return get_annotation_property_string(annot, "name");
+      return get_annotation_property_string(annot_mod, "name");
     } else if (
-        const t_const* annot =
+        const t_const* annot_name =
             service_->find_structured_annotation_or_null(kRustNameUri)) {
-      return snakecase(get_annotation_property_string(annot, "name"));
+      return snakecase(get_annotation_property_string(annot_name, "name"));
     } else if (service_->has_annotation("rust.name")) {
       return snakecase(service_->get_annotation("rust.name"));
     } else {
@@ -1022,27 +1059,13 @@ class rust_mstch_service : public mstch_service {
         service_->find_structured_annotation_or_null(kRustRequestContextUri);
   }
   mstch::node rust_extended_clients() {
-    return rust_extended_services(get_client_import_name);
-  }
-  mstch::node rust_extended_servers() {
-    return rust_extended_services(get_server_import_name);
-  }
-  mstch::node rust_extended_services(std::string (*get_import_name)(
-      const t_program*, const rust_codegen_options&)) {
     mstch::array extended_services;
     const t_service* service = service_;
-    std::string type_prefix = get_import_name(service_->program(), options_);
     std::string as_ref_impl = "&self.parent";
-    while (true) {
-      const t_service* parent_service = service->get_extends();
-      if (parent_service == nullptr) {
-        break;
-      }
-      if (parent_service->program() != service->program()) {
-        type_prefix += "::dependencies::" + parent_service->program()->name();
-      }
+    while (const t_service* parent_service = service->get_extends()) {
       mstch::map node;
-      node["extendedService:packagePrefix"] = type_prefix;
+      node["extendedService:packagePrefix"] =
+          get_client_import_name(parent_service->program(), options_);
       node["extendedService:asRefImpl"] = as_ref_impl;
       node["extendedService:service"] =
           make_mstch_extended_service_cached(parent_service);
@@ -1331,6 +1354,10 @@ class rust_mstch_struct : public mstch_struct {
           type_has_transitive_adapter(field.get_type(), true)) {
         return false;
       }
+
+      if (field.find_structured_annotation_or_null(kRustTypeUri)) {
+        return false;
+      }
     }
     return true;
   }
@@ -1900,7 +1927,7 @@ class mstch_rust_struct_field : public mstch_base {
             {"field:has_adapter?", &mstch_rust_struct_field::has_adapter},
         });
   }
-  mstch::node key() { return std::to_string(field_->get_key()); }
+  mstch::node key() { return field_->get_key(); }
   mstch::node rust_name() { return named_rust_name(field_); }
   mstch::node is_optional() {
     return field_->get_req() == t_field::e_req::optional;
@@ -2169,7 +2196,7 @@ class rust_mstch_typedef : public mstch_typedef {
   mstch::node rust_ord() {
     return typedef_->has_annotation("rust.ord") ||
         typedef_->find_structured_annotation_or_null(kRustOrdUri) ||
-        (can_derive_ord(typedef_->get_type()) &&
+        (can_derive_ord(typedef_) &&
          !type_has_transitive_adapter(typedef_->get_type(), true));
   }
   mstch::node rust_copy() {
@@ -2316,7 +2343,8 @@ void t_mstch_rust_generator::generate_program() {
     auto cratemap = load_crate_map(*cratemap_flag);
     options_.multifile_mode = cratemap.multifile_mode;
     options_.label = std::move(cratemap.label);
-    options_.cratemap = std::move(cratemap.cratemap);
+    options_.crate_index =
+        rust_crate_index{program_, std::move(cratemap.cratemap)};
   }
 
   options_.serde = has_option("serde");
@@ -2325,6 +2353,8 @@ void t_mstch_rust_generator::generate_program() {
     assert(options_.serde);
   }
 
+  options_.valuable = has_option("valuable");
+
   options_.any_registry_initialization_enabled = has_option("any");
 
   parse_include_srcs(
@@ -2385,6 +2415,9 @@ void t_mstch_rust_generator::generate_program() {
     namespace_rust = program_->name();
   }
 
+  std::string namespace_cpp2 =
+      gen::cpp::namespace_resolver::gen_namespace(*program_);
+
   std::string service_names;
   for (const t_service* service : program_->services()) {
     service_names += named_rust_name(service);
@@ -2401,7 +2434,8 @@ void t_mstch_rust_generator::generate_program() {
   render_to_file(prog, "client.rs", "client.rs");
   render_to_file(prog, "server.rs", "server.rs");
   render_to_file(prog, "mock.rs", "mock.rs");
-  write_output("namespace", namespace_rust + '\n');
+  write_output("namespace-rust", namespace_rust + '\n');
+  write_output("namespace-cpp2", namespace_cpp2 + '\n');
   write_output("service-names", service_names);
 }
 
@@ -2528,13 +2562,13 @@ void t_mstch_rust_generator::fill_validator_visitors(
       options_));
   validator.add_enum_visitor(validate_enum_annotations);
   validator.add_program_visitor(validate_program_annotations);
-  add_explicit_include_validators(validator, diagnostic_level::error);
 }
 
 THRIFT_REGISTER_GENERATOR(
     mstch_rust,
     "Rust",
     "    serde:           Derive serde Serialize/Deserialize traits for types\n"
+    "    valuable:        Derive valuable Valuable trait for types\n"
     "    any:             Generate \"any registry\" initialization functions\n"
     "    deprecated_default_enum_min_i32:\n"
     "                     Default enum value is i32::MIN. Deprecated, to be removed in future versions\n"
