@@ -16,6 +16,7 @@
 
 #include <cctype>
 #include <boost/algorithm/string.hpp>
+#include <fmt/core.h>
 #include <thrift/compiler/lib/go/util.h>
 
 namespace apache {
@@ -244,6 +245,24 @@ bool codegen_data::is_current_program(const t_program* program) {
   return (program == current_program_);
 }
 
+std::string_view codegen_data::maybe_munge_ident_and_cache(
+    const t_named* named, bool exported, bool compact) {
+  assert(named);
+
+  if (auto name = get_go_name_annotation(named)) {
+    return *name;
+  }
+
+  go_munged_names_cache_key_ key{named->name(), exported, compact};
+  auto& item = go_munged_names_cache_[key];
+  if (item.view.data() != nullptr) {
+    return item.view;
+  }
+
+  return item.view = item.ownership =
+             munge_ident(named->name(), exported, compact);
+}
+
 std::string codegen_data::get_go_package_alias(const t_program* program) {
   if (is_current_program(program)) {
     return "";
@@ -254,7 +273,8 @@ std::string codegen_data::get_go_package_alias(const t_program* program) {
   if (iter != go_package_map_.end()) {
     return iter->second;
   }
-  throw std::runtime_error("unable to determine Go package alias");
+  throw std::runtime_error(
+      fmt::format("unable to determine Go package alias '{}'", package));
 }
 
 std::string codegen_data::go_package_alias_prefix(const t_program* program) {
@@ -290,7 +310,19 @@ std::string get_go_package_base_name(
     const t_program* program, std::string name_override) {
   auto go_package = get_go_package_name(program, name_override);
   std::vector<std::string> parts;
-  boost::split(parts, go_package, boost::is_any_of("."));
+  // The go package name can be seperated by slashes or dots.
+  // Slashes can only be used if it was quoted, for example `namespace go
+  // 'foo/bar'`. These quotes are already removed, when we get to this function.
+  // Either way the go package name is always the last part.
+  // e.g. 'foo/bar' -> bar
+  //      foo -> foo
+  //      foo.bar -> bar
+  //      foo.bar.baz -> baz
+  if (go_package.find('/') != std::string::npos) {
+    boost::split(parts, go_package, boost::is_any_of("/"));
+  } else {
+    boost::split(parts, go_package, boost::is_any_of("."));
+  }
   auto base_name = go_package;
   if (parts.size() > 0) {
     base_name = parts.back();
@@ -380,14 +412,12 @@ std::string munge_ident(const std::string& ident, bool exported, bool compat) {
 
   auto result = out.str();
 
-  // Compat: legacy generator adds underscores to names ending with
-  // Args/Result. Compat: legacy generator adds underscores to names startng
-  // with New. (to avoid name collisions with constructors and helper
-  // arg/result structs)
+  // We add underscores to names starting with New to avoid name collisions with
+  // constructors. For example, if we are given a NewFoo message and a Foo
+  // message, the Foo message will generate a NewFoo constructor function, which
+  // will conflict with the NewFoo struct.
   bool starts_with_new = boost::algorithm::starts_with(result, "New");
-  bool ends_with_args = boost::algorithm::ends_with(result, "Args");
-  bool ends_with_rslt = boost::algorithm::ends_with(result, "Result");
-  if (compat && (starts_with_new || ends_with_args || ends_with_rslt)) {
+  if (starts_with_new) {
     result += '_';
   }
 
@@ -598,10 +628,6 @@ const std::string* get_go_name_annotation(const t_named* node) {
   if (name_annotation != nullptr) {
     return &(name_annotation->get_value_from_structured_annotation("name")
                  .get_string());
-  } else if (node->has_annotation("go.name")) {
-    // TODO: remove this else-if clause once
-    // all non-structured annotations are migrated.
-    return &(node->get_annotation("go.name"));
   }
   return nullptr;
 }
@@ -611,10 +637,6 @@ const std::string* get_go_tag_annotation(const t_named* node) {
   if (tag_annotation != nullptr) {
     return &(tag_annotation->get_value_from_structured_annotation("tag")
                  .get_string());
-  } else if (node->has_annotation("go.tag")) {
-    // TODO: remove this else-if clause once
-    // all non-structured annotations are migrated.
-    return &(node->get_annotation("go.tag"));
   }
   return nullptr;
 }
