@@ -17,6 +17,9 @@
 #include <string>
 #include <string_view>
 #include <utility>
+#include <fmt/core.h>
+#include <openssl/evp.h>
+#include <openssl/md5.h>
 #include <openssl/sha.h>
 #include <thrift/compiler/ast/t_const.h>
 #include <thrift/compiler/ast/t_exception.h>
@@ -55,9 +58,8 @@ std::string uri_or_name(const t_named& node) {
 } // namespace
 
 t_type_ref schematizer::stdType(std::string_view uri) {
-  auto scope = bundle_->get_root_program()->scope();
   return t_type_ref::from_req_ptr(
-      static_cast<const t_type*>(scope->find_by_uri(uri)));
+      static_cast<const t_type*>(scope_.find_by_uri(uri)));
 }
 
 std::unique_ptr<t_const_value> schematizer::typeUri(const t_type& type) {
@@ -752,15 +754,50 @@ std::unique_ptr<t_const_value> wrap_with_protocol_value(
   return ret;
 }
 
+std::string_view schematizer::program_checksum(const t_program& program) {
+  if (auto it = program_checksums_.find(&program);
+      it != program_checksums_.end()) {
+    return it->second;
+  }
+  // @lint-ignore CLANGTIDY facebook-hte-CArray
+  unsigned char hash[MD5_DIGEST_LENGTH];
+  auto val = sm_.get_file(program.path())->text;
+  EVP_Digest(val.data(), val.size(), hash, nullptr, EVP_md5(), nullptr);
+  return (
+      program_checksums_[&program] =
+          std::string(reinterpret_cast<const char*>(hash), sizeof(hash)));
+}
+
 std::string schematizer::identify_definition(const t_named& node) {
   // @lint-ignore CLANGTIDY facebook-hte-CArray
   unsigned char hash[SHA256_DIGEST_LENGTH];
-  auto val = node.program()->path() + node.name();
+  auto val = fmt::format(
+      "{}{}{}",
+      program_checksum(*node.program()),
+      node.program()->path(),
+      node.name());
   SHA256(reinterpret_cast<const unsigned char*>(val.c_str()), val.size(), hash);
   constexpr size_t kBytes = 16;
   return std::string(reinterpret_cast<const char*>(hash), kBytes);
 }
 
+int64_t schematizer::identify_program(const t_program& node) {
+  // @lint-ignore CLANGTIDY facebook-hte-CArray
+  unsigned char hash[SHA256_DIGEST_LENGTH];
+  auto val = fmt::format("{}{}", program_checksum(node), node.path());
+  SHA256(reinterpret_cast<const unsigned char*>(val.c_str()), val.size(), hash);
+  int64_t ret;
+  memcpy(&ret, hash, sizeof(ret));
+  return ret;
+}
+
+std::string schematizer::name_schema(
+    source_manager& sm, const t_program& node) {
+  schematizer s(*node.scope(), sm, {});
+  return fmt::format(
+      "_fbthrift_schema_{:x}", static_cast<uint64_t>(s.identify_program(node)));
+}
+
 } // namespace compiler
 } // namespace thrift
 } // namespace apache
