@@ -279,10 +279,10 @@ class parser {
         : nullptr;
   }
 
-  boost::optional<comment> try_parse_inline_doc() {
+  std::optional<comment> try_parse_inline_doc() {
     return token_.kind == tok::inline_doc
         ? actions_.on_inline_doc(token_.range, consume_token().string_value())
-        : boost::optional<comment>();
+        : std::optional<comment>();
   }
 
   // structured_annotation: "@" (identifier | struct_initializer)
@@ -483,7 +483,7 @@ class parser {
     switch (token_.kind) {
       case tok::kw_void:
         is_void = true;
-        ret.type = &t_base_type::t_void();
+        ret.type = &t_primitive_type::t_void();
         consume_token();
         break;
       case tok::kw_sink:
@@ -669,7 +669,7 @@ class parser {
     auto attrs = parse_attributes();
 
     // Parse the field id.
-    auto field_id = boost::optional<int64_t>();
+    auto field_id = std::optional<int64_t>();
     if (auto integer = try_parse_integer()) {
       field_id = *integer;
       expect_and_consume(':');
@@ -740,7 +740,7 @@ class parser {
   // the token as return_clause.
   t_type_ref parse_type() {
     auto range = track_range();
-    if (const t_base_type* type = try_parse_base_type()) {
+    if (const t_primitive_type* type = try_parse_base_type()) {
       return actions_.on_type(range, *type, try_parse_deprecated_annotations());
     }
     switch (token_.kind) {
@@ -785,27 +785,27 @@ class parser {
 
   // base_type: "bool" | "byte" | "i16" | "i32" | "i64" | "float" | "double" |
   //            "string" | "binary"
-  const t_base_type* try_parse_base_type() {
-    auto get_base_type = [this]() -> const t_base_type* {
+  const t_primitive_type* try_parse_base_type() {
+    auto get_base_type = [this]() -> const t_primitive_type* {
       switch (token_.kind) {
         case tok::kw_bool:
-          return &t_base_type::t_bool();
+          return &t_primitive_type::t_bool();
         case tok::kw_byte:
-          return &t_base_type::t_byte();
+          return &t_primitive_type::t_byte();
         case tok::kw_i16:
-          return &t_base_type::t_i16();
+          return &t_primitive_type::t_i16();
         case tok::kw_i32:
-          return &t_base_type::t_i32();
+          return &t_primitive_type::t_i32();
         case tok::kw_i64:
-          return &t_base_type::t_i64();
+          return &t_primitive_type::t_i64();
         case tok::kw_float:
-          return &t_base_type::t_float();
+          return &t_primitive_type::t_float();
         case tok::kw_double:
-          return &t_base_type::t_double();
+          return &t_primitive_type::t_double();
         case tok::kw_string:
-          return &t_base_type::t_string();
+          return &t_primitive_type::t_string();
         case tok::kw_binary:
-          return &t_base_type::t_binary();
+          return &t_primitive_type::t_binary();
         default:
           return nullptr;
       }
@@ -841,8 +841,8 @@ class parser {
     auto attrs = parse_attributes();
     auto name = parse_identifier();
     auto value = try_consume_token('=')
-        ? boost::optional<int64_t>(parse_integer())
-        : boost::none;
+        ? std::optional<int64_t>(parse_integer())
+        : std::nullopt;
     try_parse_deprecated_annotations(attrs);
     if (token_.kind == ',') {
       consume_token();
@@ -874,7 +874,6 @@ class parser {
   //   integer | float | string_literal | bool_literal | identifier |
   //   list_initializer | map_initializer | struct_initializer
   std::unique_ptr<t_const_value> parse_initializer() {
-    auto range = track_range();
     auto s = sign::plus;
     switch (token_.kind) {
       case tok::bool_literal:
@@ -885,14 +884,14 @@ class parser {
       case to_tok('+'):
         consume_token();
         if (token_.kind == tok::int_literal) {
-          return actions_.on_integer(range, parse_integer(s));
+          return actions_.on_integer(parse_integer(s));
         } else if (token_.kind == tok::float_literal) {
           return actions_.on_float(parse_float(s));
         }
         report_expected("number");
         break;
       case tok::int_literal:
-        return actions_.on_integer(range, parse_integer());
+        return actions_.on_integer(parse_integer());
       case tok::float_literal:
         return actions_.on_float(parse_float());
       case tok::string_literal:
@@ -970,7 +969,7 @@ class parser {
   }
 
   // integer: ["+" | "-"] int_literal
-  boost::optional<int64_t> try_parse_integer(sign s = sign::plus) {
+  std::optional<int64_t> try_parse_integer(sign s = sign::plus) {
     auto range = track_range();
     switch (token_.kind) {
       case to_tok('-'):
@@ -1033,7 +1032,7 @@ class parser {
   //   | "permanent"
   //   | "server"
   //   | "client"
-  boost::optional<identifier> try_parse_identifier() {
+  std::optional<identifier> try_parse_identifier() {
     auto range = track_range();
     switch (token_.kind) {
       case tok::identifier:
