@@ -20,8 +20,8 @@
 #include <string>
 #include <vector>
 
-#include <thrift/compiler/ast/diagnostic_context.h>
 #include <thrift/compiler/ast/t_named.h>
+#include <thrift/compiler/sema/diagnostic_context.h>
 
 namespace apache {
 namespace thrift {
