@@ -19,10 +19,10 @@
 #include <functional>
 #include <type_traits>
 
+#include <thrift/compiler/detail/pluggable_functions.h>
 #include <thrift/compiler/lib/cpp2/util.h>
 #include <thrift/compiler/lib/schematizer.h>
 #include <thrift/compiler/lib/uri.h>
-#include <thrift/compiler/sema/patch_mutator.h>
 #include <thrift/compiler/sema/standard_mutator_stage.h>
 
 namespace apache {
@@ -340,8 +340,7 @@ void normalize_return_type(
 
   // Check the (first) response type.
   const t_type* true_type = type->get_true_type();
-  if (dynamic_cast<const t_service*>(true_type) ||
-      dynamic_cast<const t_exception*>(true_type)) {
+  if (dynamic_cast<const t_service*>(true_type)) {
     ctx.error("Invalid first response type: {}", type->get_full_name());
   }
 }
@@ -402,6 +401,36 @@ void lower_type_annotations(
   }
 }
 
+void inject_schema_const(
+    diagnostic_context& ctx, mutator_context&, t_program& prog) {
+  schematizer::options opts;
+  opts.only_root_program_ = true;
+  opts.use_hash = true;
+  opts.include.reset(schematizer::included_data::DoubleWrites);
+  opts.include.reset(schematizer::included_data::Docs);
+  opts.include.reset(schematizer::included_data::SourceRanges);
+
+  std::string serialized = detail::pluggable_functions().call<GetSchemaTag>(
+      opts, ctx.source_mgr(), prog);
+  if (serialized.empty()) {
+    throw std::runtime_error(
+        "Enabling --inject-schema-const requires using thrift2ast");
+  }
+
+  auto cnst = std::make_unique<t_const>(
+      &prog,
+      t_primitive_type::t_binary(),
+      schematizer::name_schema(ctx.source_mgr(), prog),
+      std::make_unique<t_const_value>(std::move(serialized)));
+  // Since schema injection happens after 'match_const_type_with_value' mutator
+  // we need to set the explicitly type to binary here.
+  cnst->value()->set_ttype(t_primitive_type::t_binary());
+  cnst->set_uri("");
+  cnst->set_src_range(prog.src_range());
+  cnst->set_generated();
+  prog.add_definition(std::move(cnst));
+}
+
 } // namespace
 
 ast_mutators standard_mutators(bool use_legacy_type_ref_resolution) {
@@ -425,10 +454,15 @@ ast_mutators standard_mutators(bool use_legacy_type_ref_resolution) {
     main.add_definition_visitor(&match_annotation_types_with_const_values);
   }
 
-  add_patch_mutators(mutators);
   return mutators;
 }
 
+ast_mutator schema_mutator() {
+  ast_mutator mutator;
+  mutator.add_program_visitor(&inject_schema_const);
+  return mutator;
+}
+
 } // namespace compiler
 } // namespace thrift
 } // namespace apache
