@@ -1798,26 +1798,6 @@ TEST(CompilerTest, py3_invalid_field_names) {
       {"--gen", "mstch_py3"});
 }
 
-TEST(CompilerTest, assign_only_patch) {
-  check_compile(R"(
-    include "thrift/lib/thrift/patch.thrift"
-
-    @patch.GeneratePatch
-    package "meta.com/thrift/test"
-
-    struct MyStruct {
-          # expected-error@-1: List patch can not be AssignOnly
-          # expected-error@-2: Set patch can not be AssignOnly
-        @patch.AssignOnlyPatch
-        1: list<i32> field1;
-        @patch.AssignOnlyPatch
-        2: set<i32> field2;
-        @patch.AssignOnlyPatch
-        3: map<i32, string> field3;
-    }
-  )");
-}
-
 TEST(CompilerTest, warn_on_non_explicit_includes) {
   std::map<std::string, std::string> name_contents_map;
   name_contents_map["path/to/upstream.thrift"] = R"(
@@ -2018,15 +1998,18 @@ TEST(CompilerTest, cursor_serialization_adapter) {
   )");
 }
 
-TEST(CompilerTest, no_generate_patch_in_cpp) {
+TEST(CompilerTest, exception_invalid_use) {
   check_compile(R"(
-    include "thrift/lib/thrift/patch.thrift"
-
-    @patch.GeneratePatch
-    package "meta.com/thrift/test"
-
-    struct MyStruct { # expected-error: @patch.GeneratePatch is disallowed in C++ [no-generate-patch]
-        1: i32 field;
+    exception E {}
+    struct S {
+      1: E field; # TODO (T191018859)
     }
+
+    service Svc {
+      E return(); # expected-error: Exceptions cannot be used as function return types
+      void param(1: E e); # expected-error: Exceptions cannot be used as function arguments
+    }
+
+    const E e = E{}; # expected-error: Exceptions cannot be used as const types
   )");
 }
