@@ -32,8 +32,8 @@ using apache::thrift::metadata::ThriftType;
 class MetadataCodegenTest : public testing::Test {
  protected:
   ThriftStruct getStruct(const std::string& name) {
-    ThriftStruct s = metadata_.structs_ref()->at(name);
-    EXPECT_EQ(*s.name_ref(), name);
+    ThriftStruct s = metadata_.structs()->at(name);
+    EXPECT_EQ(*s.name(), name);
     return s;
   }
   ThriftType getResolvedType(const ThriftType& type) {
@@ -48,9 +48,9 @@ class MetadataCodegenTest : public testing::Test {
   }
   ThriftType checkField(
       const ThriftField& field, int key, const std::string& name) {
-    EXPECT_EQ(*field.name_ref(), name);
-    EXPECT_EQ(*field.id_ref(), key);
-    return getResolvedType(*field.type_ref());
+    EXPECT_EQ(*field.name(), name);
+    EXPECT_EQ(*field.id(), key);
+    return getResolvedType(*field.type());
   }
   void checkFieldUnion(
       const ThriftField& field,
@@ -59,13 +59,13 @@ class MetadataCodegenTest : public testing::Test {
       const std::string& typeName,
       bool optional = true) {
     auto type = checkField(field, key, name);
-    EXPECT_EQ(*field.is_optional_ref(), optional);
-    EXPECT_EQ(*type.get_t_union().name_ref(), typeName);
+    EXPECT_EQ(*field.is_optional(), optional);
+    EXPECT_EQ(*type.get_t_union().name(), typeName);
   }
   void checkFieldString(
       const ThriftField& field, int key, const std::string& name) {
     auto type = checkField(field, key, name);
-    EXPECT_FALSE(*field.is_optional_ref());
+    EXPECT_FALSE(*field.is_optional());
     EXPECT_EQ(type.get_t_primitive(), ThriftPrimitiveType::THRIFT_STRING_TYPE);
   }
   ThriftMetadata metadata_;
@@ -74,49 +74,48 @@ class MetadataCodegenTest : public testing::Test {
 TEST_F(MetadataCodegenTest, structTest) {
   StructMetadata<ThriftField>::gen(metadata_);
 
-  EXPECT_EQ(metadata_.enums_ref()->size(), 1);
+  EXPECT_EQ(metadata_.enums()->size(), 1);
   std::string enumName{"metadata.ThriftPrimitiveType"};
-  ThriftEnum e = metadata_.enums_ref()->at(enumName);
-  EXPECT_EQ(*e.name_ref(), enumName);
-  EXPECT_EQ(e.elements_ref()->size(), 10);
-  EXPECT_EQ(e.elements_ref()->at(1), "THRIFT_BOOL_TYPE");
-  EXPECT_EQ(e.elements_ref()->at(2), "THRIFT_BYTE_TYPE");
-  EXPECT_EQ(e.elements_ref()->at(3), "THRIFT_I16_TYPE");
-  EXPECT_EQ(e.elements_ref()->at(4), "THRIFT_I32_TYPE");
-  EXPECT_EQ(e.elements_ref()->at(5), "THRIFT_I64_TYPE");
-  EXPECT_EQ(e.elements_ref()->at(6), "THRIFT_FLOAT_TYPE");
-  EXPECT_EQ(e.elements_ref()->at(8), "THRIFT_BINARY_TYPE");
-  EXPECT_EQ(e.elements_ref()->at(9), "THRIFT_STRING_TYPE");
-  EXPECT_EQ(e.elements_ref()->at(10), "THRIFT_VOID_TYPE");
+  ThriftEnum e = metadata_.enums()->at(enumName);
+  EXPECT_EQ(*e.name(), enumName);
+  EXPECT_EQ(e.elements()->size(), 10);
+  EXPECT_EQ(e.elements()->at(1), "THRIFT_BOOL_TYPE");
+  EXPECT_EQ(e.elements()->at(2), "THRIFT_BYTE_TYPE");
+  EXPECT_EQ(e.elements()->at(3), "THRIFT_I16_TYPE");
+  EXPECT_EQ(e.elements()->at(4), "THRIFT_I32_TYPE");
+  EXPECT_EQ(e.elements()->at(5), "THRIFT_I64_TYPE");
+  EXPECT_EQ(e.elements()->at(6), "THRIFT_FLOAT_TYPE");
+  EXPECT_EQ(e.elements()->at(8), "THRIFT_BINARY_TYPE");
+  EXPECT_EQ(e.elements()->at(9), "THRIFT_STRING_TYPE");
+  EXPECT_EQ(e.elements()->at(10), "THRIFT_VOID_TYPE");
 
   ThriftStruct s;
   s = getStruct("metadata.ThriftListType");
-  EXPECT_EQ(s.fields_ref()->size(), 1);
-  checkFieldUnion(s.fields_ref()[0], 1, "valueType", "metadata.ThriftType");
+  EXPECT_EQ(s.fields()->size(), 1);
+  checkFieldUnion(s.fields()[0], 1, "valueType", "metadata.ThriftType");
 
   s = getStruct("metadata.ThriftMapType");
-  EXPECT_EQ(s.fields_ref()->size(), 2);
-  checkFieldUnion(s.fields_ref()[0], 1, "keyType", "metadata.ThriftType");
-  checkFieldUnion(s.fields_ref()[1], 2, "valueType", "metadata.ThriftType");
+  EXPECT_EQ(s.fields()->size(), 2);
+  checkFieldUnion(s.fields()[0], 1, "keyType", "metadata.ThriftType");
+  checkFieldUnion(s.fields()[1], 2, "valueType", "metadata.ThriftType");
 
   s = getStruct("metadata.ThriftEnumType");
-  EXPECT_EQ(s.fields_ref()->size(), 1);
-  checkFieldString(s.fields_ref()[0], 1, "name");
+  EXPECT_EQ(s.fields()->size(), 1);
+  checkFieldString(s.fields()[0], 1, "name");
 
   s = getStruct("metadata.ThriftTypedefType");
-  EXPECT_EQ(s.fields_ref()->size(), 3);
-  checkFieldString(s.fields_ref()[0], 1, "name");
-  checkFieldUnion(
-      s.fields_ref()[1], 2, "underlyingType", "metadata.ThriftType");
+  EXPECT_EQ(s.fields()->size(), 3);
+  checkFieldString(s.fields()[0], 1, "name");
+  checkFieldUnion(s.fields()[1], 2, "underlyingType", "metadata.ThriftType");
 
-  auto td = s.fields_ref()[2];
-  EXPECT_EQ(td.name_ref(), "structured_annotations");
-  EXPECT_EQ(td.type_ref()->getType(), ThriftType::t_list);
+  auto td = s.fields()[2];
+  EXPECT_EQ(td.name(), "structured_annotations");
+  EXPECT_EQ(td.type()->getType(), ThriftType::Type::t_list);
   EXPECT_EQ(
-      td.type_ref()->t_list_ref()->valueType_ref()->getType(),
-      ThriftType::t_struct);
+      td.type()->t_list_ref()->valueType_ref()->getType(),
+      ThriftType::Type::t_struct);
   EXPECT_EQ(
-      td.type_ref()->t_list_ref()->valueType_ref()->t_struct_ref()->name_ref(),
+      td.type()->t_list_ref()->valueType_ref()->t_struct_ref()->name(),
       "metadata.ThriftConstStruct");
 }
 } // namespace
