@@ -29,7 +29,7 @@ std::shared_ptr<t_program> dedent_and_parse_to_program(
   auto temp_file = std::make_shared<const folly::test::TemporaryFile>();
   const auto path = temp_file->path().string();
   std::ofstream(path) << folly::stripLeftMargin(source);
-  auto ctx = diagnostic_context::ignore_all(sm);
+  auto ctx = diagnostics_engine::ignore_all(sm);
   auto bundle = folly::to_shared_ptr(
       apache::thrift::compiler::parse_and_mutate_program(sm, ctx, path, {}));
   return {bundle->root_program(), [bundle, temp_file](auto) {}};
