@@ -24,8 +24,8 @@
 #include <thrift/lib/cpp2/protocol/Protocol.h>
 #if __has_include(<thrift/lib/thrift/gen-cpp2/any_rep_types.h>)
 #include <thrift/lib/cpp2/type/Any.h>
-#include <thrift/lib/cpp2/type/AnyRef.h>
 #include <thrift/lib/cpp2/type/AnyValue.h>
+#include <thrift/lib/cpp2/type/Runtime.h>
 #include <thrift/lib/cpp2/type/Tag.h>
 #include <thrift/lib/cpp2/type/TypeRegistry.h>
 #define THRIFT_ANY_AVAILABLE
@@ -136,7 +136,7 @@ void helper<ProtocolReader, ProtocolWriter>::process_exn(
             req->includeEnvelope(), func, &oprot, protoSeqId, nullptr, x),
         ctx->getHeader()->getWriteTransforms());
     eb->runInEventBaseThread(
-        [payload = move(payload), request = move(req)]() mutable {
+        [payload = std::move(payload), request = std::move(req)]() mutable {
           if (request->isStream()) {
             request->sendStreamReply(
                 ResponsePayload::create(std::move(payload)),
