@@ -21,9 +21,10 @@
 #include <thrift/lib/cpp2/async/Interaction.h>
 #include <thrift/lib/cpp2/async/RequestChannel.h>
 
-namespace apache {
-namespace thrift {
+namespace apache::thrift {
+
 class InteractionHandle;
+class ClientInterceptorBase;
 
 class GeneratedAsyncClient : public TClientBase {
  public:
@@ -34,13 +35,13 @@ class GeneratedAsyncClient : public TClientBase {
    public:
     Options() {}
 
-    Options& includeGlobalEventHandlers(bool include) {
-      clientBaseOptions_.includeGlobalEventHandlers = include;
+    Options& includeGlobalLegacyEventHandlers(bool include) {
+      clientBaseOptions_.includeGlobalLegacyClientHandlers = include;
       return *this;
     }
 
     static Options zeroDependency() {
-      return Options().includeGlobalEventHandlers(false);
+      return Options().includeGlobalLegacyEventHandlers(false);
     }
 
    private:
@@ -52,6 +53,11 @@ class GeneratedAsyncClient : public TClientBase {
   GeneratedAsyncClient(std::shared_ptr<RequestChannel> channel);
   GeneratedAsyncClient(
       std::shared_ptr<RequestChannel> channel, Options options);
+  GeneratedAsyncClient(
+      std::shared_ptr<RequestChannel> channel,
+      std::shared_ptr<std::vector<std::shared_ptr<ClientInterceptorBase>>>
+          interceptors,
+      Options options = Options());
 
   virtual const char* getServiceName() const noexcept = 0;
 
@@ -69,7 +75,26 @@ class GeneratedAsyncClient : public TClientBase {
   static void setInteraction(
       const InteractionHandle& handle, RpcOptions& rpcOptions);
 
+  template <bool IsOneWay>
+  std::pair<RequestClientCallback::Ptr, ContextStack*>
+  prepareRequestClientCallback(
+      std::unique_ptr<RequestCallback> callback,
+      ContextStack::UniquePtr contextStack) {
+    RequestCallback::Context callbackContext;
+    callbackContext.oneWay = IsOneWay;
+    callbackContext.protocolId = this->getChannel()->getProtocolId();
+    auto* ctx = contextStack.get();
+    if (callback) {
+      callbackContext.ctx = std::move(contextStack);
+    }
+    auto wrappedCallback = apache::thrift::toRequestClientCallbackPtr(
+        std::move(callback), std::move(callbackContext));
+    return std::make_pair(std::move(wrappedCallback), ctx);
+  }
+
   std::shared_ptr<RequestChannel> channel_;
+  std::shared_ptr<std::vector<std::shared_ptr<ClientInterceptorBase>>>
+      interceptors_;
 };
 
 class InteractionHandle : public GeneratedAsyncClient {
@@ -95,5 +120,4 @@ class InteractionHandle : public GeneratedAsyncClient {
   friend class GeneratedAsyncClient;
 };
 
-} // namespace thrift
-} // namespace apache
+} // namespace apache::thrift
