@@ -325,7 +325,8 @@ void HeaderClientChannel::sendRequestResponse(
   try {
     setBaseReceivedCallback(); // Cpp2Channel->setReceiveCallback can throw
   } catch (const TTransportException&) {
-    twcb->messageSendError(folly::exception_wrapper(std::current_exception()));
+    twcb->messageSendError(
+        folly::exception_wrapper(folly::current_exception()));
     return;
   }
 
