@@ -214,15 +214,6 @@ const std::shared_ptr<void>& RpcOptions::getRoutingData() const {
   return routingData_;
 }
 
-RpcOptions& RpcOptions::setRoutingHint(uint64_t hint) {
-  routingHint_ = hint;
-  return *this;
-}
-
-uint64_t RpcOptions::getRoutingHint() const {
-  return routingHint_;
-}
-
 RpcOptions& RpcOptions::setContextPropMask(uint8_t mask) {
   contextPropComponentEnabledMask_ = mask;
   return *this;
@@ -282,5 +273,14 @@ folly::SocketFds RpcOptions::copySocketFdsToSend() const {
   return folly::SocketFds{fdsToSend_};
 }
 
+RpcOptions& RpcOptions::setConnectionKey(std::string key) {
+  connectionKey_ = std::move(key);
+  return *this;
+}
+
+std::string_view RpcOptions::getConnectionKey() const {
+  return connectionKey_;
+}
+
 } // namespace thrift
 } // namespace apache
