@@ -138,10 +138,6 @@ class RpcOptions {
   RpcOptions& setRoutingData(std::shared_ptr<void> data);
   const std::shared_ptr<void>& getRoutingData() const;
 
-  // Opaque 64-bit host hint to be used by the routing layer (best-effort)
-  RpcOptions& setRoutingHint(uint64_t hint);
-  uint64_t getRoutingHint() const;
-
   RpcOptions& setContextPropMask(uint8_t mask);
   uint8_t getContextPropMask() const;
 
@@ -161,6 +157,13 @@ class RpcOptions {
   RpcOptions& setFdsToSend(folly::SocketFds::ToSend);
   folly::SocketFds copySocketFdsToSend() const;
 
+  /**
+   * In the routing layer, this key can be used to create or select specific
+   * connections to a server.
+   */
+  RpcOptions& setConnectionKey(std::string key);
+  std::string_view getConnectionKey() const;
+
  private:
   using timeout_ms_t = uint32_t;
   timeout_ms_t timeout_{0};
@@ -173,11 +176,11 @@ class RpcOptions {
   bool enableChecksum_{false};
   BufferOptions bufferOptions_;
   int64_t interactionId_{0};
-  uint64_t routingHint_{0};
   uint8_t contextPropComponentEnabledMask_{0xff};
 
   std::string routingKey_;
   std::string shardId_;
+  std::string connectionKey_;
 
   // For sending and receiving headers.
   std::optional<transport::THeader::StringToStringMap> writeHeaders_;
