@@ -18,6 +18,7 @@
 
 #include <thrift/lib/cpp2/protocol/BinaryProtocol.h>
 #include <thrift/lib/cpp2/protocol/CompactProtocol.h>
+#include <thrift/lib/cpp2/protocol/JSONProtocol.h>
 #include <thrift/lib/cpp2/protocol/SimpleJSONProtocol.h>
 
 namespace apache {
@@ -96,6 +97,12 @@ template size_t write<SimpleJSONProtocolWriter>(
     SimpleJSONProtocolWriter* iprot,
     const StructInfo& structInfo,
     const void* object);
+template void read<JSONProtocolReader>(
+    JSONProtocolReader* iprot, const StructInfo& structInfo, void* object);
+template size_t write<JSONProtocolWriter>(
+    JSONProtocolWriter* iprot,
+    const StructInfo& structInfo,
+    const void* object);
 
 } // namespace detail
 } // namespace thrift
