@@ -22,7 +22,6 @@
 #include <type_traits>
 #include <unordered_set>
 
-#include <fatal/type/same_reference_as.h>
 #include <folly/CPortability.h>
 #include <folly/Conv.h>
 #include <folly/Utility.h>
@@ -56,7 +55,11 @@ namespace detail {
 
 template <typename C, typename T>
 decltype(auto) forward_elem(T& elem) {
-  return std::forward<typename fatal::same_reference_as<T, C>::type>(elem);
+  if constexpr (std::is_lvalue_reference_v<C>) {
+    return elem;
+  } else {
+    return static_cast<T&&>(elem);
+  }
 }
 
 template <typename TT, typename = void>
@@ -776,11 +779,13 @@ template <class Protocol>
 uint32_t serializeObject(Protocol& prot, const Object& obj) {
   uint32_t serializedSize = 0;
   serializedSize += prot.writeStructBegin("");
-  for (const auto& [fieldID, fieldVal] : *obj.members()) {
-    auto fieldType = getTType(fieldVal);
-    serializedSize += prot.writeFieldBegin("", fieldType, fieldID);
-    serializedSize += serializeValue(prot, fieldVal);
-    serializedSize += prot.writeFieldEnd();
+  if (obj.members()) {
+    for (const auto& [fieldID, fieldVal] : *obj.members()) {
+      auto fieldType = getTType(fieldVal);
+      serializedSize += prot.writeFieldBegin("", fieldType, fieldID);
+      serializedSize += serializeValue(prot, fieldVal);
+      serializedSize += prot.writeFieldEnd();
+    }
   }
   serializedSize += prot.writeFieldStop();
   serializedSize += prot.writeStructEnd();
