@@ -31,7 +31,6 @@
 #include <folly/io/Cursor.h>
 #include <thrift/lib/cpp2/Thrift.h>
 #include <thrift/lib/cpp2/reflection/reflection.h>
-#include <thrift/lib/cpp2/reflection/serializer.h>
 
 namespace apache {
 namespace thrift {
@@ -81,12 +80,79 @@ Int rand_in_range(Rng& rng, const populator_opts::range<Int>& range) {
   return static_cast<Int>(tmp);
 }
 
-// bring in an identifier from detail namespace in serializer.h
-using ::apache::thrift::detail::deref;
-using ::apache::thrift::detail::disable_if_smart_pointer;
-using ::apache::thrift::detail::enable_if_smart_pointer;
-using ::apache::thrift::detail::extract_descriptor_fid;
-using ::apache::thrift::detail::is_required_field;
+// pointer type for cpp2.ref fields, while discrimiminating against the
+// pointer corner case in Thrift (e.g., a unqiue_pointer<folly::IOBuf>)
+template <typename>
+struct is_smart_pointer : std::false_type {};
+template <typename D>
+struct is_smart_pointer<std::unique_ptr<folly::IOBuf, D>> : std::false_type {};
+
+// supported smart pointer types for cpp2.ref_type fields
+template <typename T, typename D>
+struct is_smart_pointer<std::unique_ptr<T, D>> : std::true_type {};
+template <typename T>
+struct is_smart_pointer<std::shared_ptr<T>> : std::true_type {};
+
+template <typename T>
+using enable_if_smart_pointer =
+    typename std::enable_if<is_smart_pointer<T>::value>::type;
+
+template <typename T>
+using disable_if_smart_pointer =
+    typename std::enable_if<!is_smart_pointer<T>::value>::type;
+
+// helper predicate for determining if a struct's MemberInfo is required
+// to be read out of the protocol
+struct is_required_field {
+  template <typename MemberInfo>
+  using apply = std::integral_constant<
+      bool,
+      MemberInfo::optional::value == optionality::required>;
+};
+
+struct extract_descriptor_fid {
+  template <typename T>
+  using apply = typename T::metadata::id;
+};
+
+template <typename T, typename Enable = void>
+struct deref;
+
+// General case: methods on deref are no-op, returning their input
+template <typename T>
+struct deref<T, disable_if_smart_pointer<T>> {
+  static T& clear_and_get(T& in) { return in; }
+};
+
+// Special case: We specifically *do not* dereference a unique pointer to
+// an IOBuf, because this is a type that the protocol can (de)serialize
+// directly
+template <>
+struct deref<std::unique_ptr<folly::IOBuf>> {
+  using T = std::unique_ptr<folly::IOBuf>;
+  static T& clear_and_get(T& in) { return in; }
+};
+
+// General case: deref returns a reference to what the
+// unique pointer contains
+template <typename PtrType>
+struct deref<PtrType, enable_if_smart_pointer<PtrType>> {
+  using T = typename std::remove_const<typename PtrType::element_type>::type;
+  static T& clear_and_get(std::shared_ptr<const T>& in) {
+    auto t = std::make_shared<T>();
+    auto ret = t.get();
+    in = std::move(t);
+    return *ret;
+  }
+  static T& clear_and_get(std::shared_ptr<T>& in) {
+    in = std::make_shared<T>();
+    return *in;
+  }
+  static T& clear_and_get(std::unique_ptr<T>& in) {
+    in = std::make_unique<T>();
+    return *in;
+  }
+};
 
 } // namespace detail
 
