@@ -146,10 +146,12 @@ Cpp2Connection::Cpp2Connection(
           worker_->getServer()->getEventBaseManager(),
           nullptr,
           worker_->getServer()->getClientIdentityHook(),
-          worker_.get()),
+          worker_.get(),
+          apache::thrift::detail::getServiceInterceptorsIfServerIsSetUp(
+              *worker_->getServer())
+              .size()),
       transport_(transport),
-      executor_(worker_->getServer()->getHandlerExecutor_deprecated().get()),
-      metricCollector_{worker_->getServer()->getMetricCollector()} {
+      executor_(worker_->getServer()->getHandlerExecutor_deprecated().get()) {
   processor_->coalesceWithServerScopedLegacyEventHandlers(
       *worker_->getServer());
   if (worker_->getServer()->resourcePoolSet().empty()) {
@@ -164,19 +166,39 @@ Cpp2Connection::Cpp2Connection(
   for (const auto& handler : worker_->getServer()->getEventHandlersUnsafe()) {
     handler->newConnection(&context_);
   }
+
+#if FOLLY_HAS_COROUTINES
+  const auto& serviceInterceptorsInfo =
+      apache::thrift::detail::getServiceInterceptorsIfServerIsSetUp(
+          *worker_->getServer());
+  for (std::size_t i = 0; i < serviceInterceptorsInfo.size(); ++i) {
+    ServiceInterceptorBase::ConnectionInfo connectionInfo{
+        &context_,
+        context_.getStorageForServiceInterceptorOnConnectionByIndex(i)};
+    serviceInterceptorsInfo[i].interceptor->internal_onConnection(
+        connectionInfo);
+  }
+#endif // FOLLY_HAS_COROUTINES
 }
 
 Cpp2Connection::~Cpp2Connection() {
+#if FOLLY_HAS_COROUTINES
+  const auto& serviceInterceptorsInfo =
+      apache::thrift::detail::getServiceInterceptorsIfServerIsSetUp(
+          *worker_->getServer());
+  for (std::size_t i = 0; i < serviceInterceptorsInfo.size(); ++i) {
+    ServiceInterceptorBase::ConnectionInfo connectionInfo{
+        &context_,
+        context_.getStorageForServiceInterceptorOnConnectionByIndex(i)};
+    serviceInterceptorsInfo[i].interceptor->internal_onConnectionClosed(
+        connectionInfo);
+  }
+#endif // FOLLY_HAS_COROUTINES
+
   for (const auto& handler : worker_->getServer()->getEventHandlersUnsafe()) {
     handler->connectionDestroyed(&context_);
   }
 
-  if (connectionAdded_) {
-    if (auto* observer = worker_->getServer()->getObserver()) {
-      observer->connClosed();
-    }
-  }
-
   channel_.reset();
 }
 
@@ -194,7 +216,6 @@ void Cpp2Connection::stop() {
       if (auto* observer = worker_->getServer()->getObserver()) {
         observer->taskKilled();
       }
-      metricCollector_.requestRejected({});
     }
   }
 
@@ -205,6 +226,15 @@ void Cpp2Connection::stop() {
     channel_->closeNow();
   }
 
+  if (connectionAdded_) {
+    if (auto* observer = worker_->getServer()->getObserver()) {
+      observer->connClosed(server::TServerObserver::ConnectionInfo(
+          reinterpret_cast<uint64_t>(transport_.get()),
+          context_.getSecurityProtocol()));
+      connectionAdded_ = false;
+    }
+  }
+
   transport_.reset();
 
   this_.reset();
@@ -300,7 +330,6 @@ void Cpp2Connection::killRequest(
   if (auto* observer = server->getObserver()) {
     observer->taskKilled();
   }
-  metricCollector_.requestRejected({});
 
   // Nothing to do for Thrift oneway request.
   if (req->isOneway()) {
@@ -319,10 +348,8 @@ void Cpp2Connection::killRequestServerOverloaded(
     OverloadResult&& overloadResult) {
   auto server = worker_->getServer();
   if (auto* observer = server->getObserver()) {
-    observer->serverOverloaded();
+    observer->serverOverloaded(overloadResult.loadShedder);
   }
-  metricCollector_.requestRejected(
-      {RequestRejectedScope::ServerOverloaded{overloadResult.loadShedder}});
 
   // Nothing to do for Thrift oneway request.
   if (req->isOneway()) {
@@ -496,7 +523,6 @@ void Cpp2Connection::requestReceived(
   if (observer) {
     observer->receivedRequest(&methodName);
   }
-  metricCollector_.requestReceived();
 
   auto injectedFailure = server->maybeInjectFailure();
   switch (injectedFailure) {
