@@ -48,7 +48,6 @@ namespace thrift {
 class Cpp2Connection;
 class ThriftServer;
 class ThriftQuicServer;
-class MetricCollector;
 
 /**
  * Cpp2Worker drives the actual I/O for ThriftServer connections.
@@ -273,11 +272,10 @@ class Cpp2Worker : public IOWorkerContext,
       DoNotUse /* ignored, never call constructor directly */)
       : Acceptor(
             server ? server->getServerSocketConfig()
-                   : wangle::ServerSocketConfig()),
+                   : std::make_shared<wangle::ServerSocketConfig>()),
         wangle::PeekingAcceptorHandshakeHelper::PeekCallback(kPeekCount),
         server_(server),
-        activeRequests_(0),
-        metricCollector_{server ? &server->getMetricCollector() : nullptr} {
+        activeRequests_(0) {
     if (server) {
       // Leave enough headroom to close connections ungracefully before the
       // worker join timeout expires.
@@ -412,7 +410,6 @@ class Cpp2Worker : public IOWorkerContext,
   std::unique_ptr<MemoryTracker> ingressMemoryTracker_;
   std::unique_ptr<MemoryTracker> egressMemoryTracker_;
   std::unique_ptr<folly::WorkerProvider> workerProvider_;
-  const MetricCollector* const metricCollector_;
 
   void initRequestsRegistry();
 
