@@ -46,9 +46,11 @@ void ThriftProcessor::onThriftRequest(
   DCHECK(channel);
 
   auto& processorFactory = server_.getDecoratedProcessorFactory();
-  if (processor_ == nullptr) {
-    processor_ = processorFactory.getProcessor();
-  }
+  const auto& processor = processor_.try_emplace_with([&] {
+    auto p = processorFactory.getProcessor();
+    p->coalesceWithServerScopedLegacyEventHandlers(server_);
+    return p;
+  });
 
   auto worker = connContext->getWorker();
   worker->getEventBase()->dcheckIsInEventBaseThread();
@@ -100,7 +102,8 @@ void ThriftProcessor::onThriftRequest(
   auto reqContext = request->getRequestContext();
 
   Cpp2Worker::dispatchRequest(
-      processor_.get(),
+      processorFactory,
+      processor.get(),
       std::move(request),
       SerializedCompressedRequest(std::move(payload)),
       methodMetadataResult,
