@@ -14,7 +14,6 @@
  * limitations under the License.
  */
 
-#include "ThriftServer.h"
 #include <fcntl.h>
 #include <signal.h>
 
@@ -109,10 +108,7 @@ THRIFT_FLAG_DEFINE_bool(fizz_server_enable_hybrid_kex, false);
 THRIFT_FLAG_DEFINE_bool(server_fizz_enable_aegis, false);
 THRIFT_FLAG_DEFINE_bool(server_fizz_prefer_psk_ke, false);
 THRIFT_FLAG_DEFINE_bool(server_fizz_enable_receiving_dc, false);
-
-THRIFT_FLAG_DEFINE_bool(fizz_deprecate_draft_versions, true);
 THRIFT_FLAG_DEFINE_bool(server_fizz_enable_presenting_dc, false);
-
 THRIFT_FLAG_DEFINE_bool(enable_rotation_for_in_memory_ticket_seeds, false);
 
 namespace apache::thrift::detail {
@@ -175,7 +171,7 @@ folly::Synchronized<std::vector<ThriftServer::IOObserverFactory>>
     ioObserverFactories{};
 
 /**
- * Multiplexes the user-service (set via setProcessorFactory) with the
+ * Multiplexes the user-service (set via setInterface) with the
  * monitoring interface (set via setMonitoringInterface).
  */
 std::unique_ptr<AsyncProcessorFactory> createDecoratedProcessorFactory(
@@ -282,24 +278,6 @@ ThriftServer::ThriftServer()
   initializeDefaults();
 }
 
-ThriftServer::ThriftServer(const ThriftServerInitialConfig& initialConfig)
-    : thriftConfig_(initialConfig),
-      adaptiveConcurrencyController_{
-          apache::thrift::detail::makeAdaptiveConcurrencyConfig(),
-          thriftConfig_.getMaxRequests().getObserver(),
-          detail::getThriftServerConfig(*this)},
-      cpuConcurrencyController_{
-          detail::makeCPUConcurrencyControllerConfig(this),
-          *this,
-          detail::getThriftServerConfig(*this)},
-      addresses_(1),
-      wShutdownSocketSet_(folly::tryGetShutdownSocketSet()),
-      lastRequestTime_(
-          std::chrono::steady_clock::now().time_since_epoch().count()) {
-  tracker_.emplace(instrumentation::kThriftServerTrackerKey, *this);
-  initializeDefaults();
-}
-
 void ThriftServer::initializeDefaults() {
   if (FLAGS_thrift_ssl_policy == "required") {
     sslPolicy_ = SSLPolicy::REQUIRED;
@@ -384,10 +362,9 @@ ThriftServer::~ThriftServer() {
   stopWorkers();
 }
 
-void ThriftServer::setProcessorFactory(
-    std::shared_ptr<AsyncProcessorFactory> pFac) {
+void ThriftServer::setInterface(std::shared_ptr<AsyncProcessorFactory> iface) {
   CHECK(configMutable());
-  cpp2Pfac_ = pFac;
+  cpp2Pfac_ = iface;
   applicationServerInterface_ = nullptr;
   for (auto* serviceHandler : cpp2Pfac_->getServiceHandlers()) {
     if (auto serverInterface = dynamic_cast<ServerInterface*>(serviceHandler)) {
@@ -949,13 +926,6 @@ void ThriftServer::setupThreadManager() {
       if (auto observer = getObserverShared()) {
         observer->resourcePoolsEnabled(explanation);
       }
-
-      XLOG_IF(INFO, infoLoggingEnabled_)
-          << "QPS limit will be enforced by "
-          << (FLAGS_thrift_server_enforces_qps_limit
-                  ? "the thrift server"
-                  : "the concurrency controller");
-
       ensureResourcePools();
 
       // During resource pools roll out we want to track services that get
@@ -1095,11 +1065,7 @@ bool ThriftServer::runtimeResourcePoolsChecks() {
   // This can be called multiple times - only run it to completion once
   // but note below that it can exit early.
   if (runtimeServerActions_.checkComplete) {
-    auto result = !runtimeDisableResourcePoolsSet();
-    XLOG_IF(INFO, infoLoggingEnabled_)
-        << "runtimeResourcePoolsChecks() is already completed and result is "
-        << result;
-    return result;
+    return !runtimeDisableResourcePoolsSet();
   }
   // If this is called too early we can't run our other checks.
   if (!getProcessorFactory()) {
@@ -1191,8 +1157,6 @@ bool ThriftServer::runtimeResourcePoolsChecks() {
         << "runtimeResourcePoolsChecks() returns false because of runtimeDisableResourcePoolsSet()";
     return false;
   }
-  XLOG_IF(INFO, infoLoggingEnabled_)
-      << "Resource pools check complete - allowed";
   return true;
 }
 
@@ -1204,7 +1168,6 @@ void ThriftServer::ensureResourcePools() {
   }
 
   if (!resourcePoolSet().hasResourcePool(ResourcePoolHandle::defaultSync())) {
-    XLOG_IF(INFO, infoLoggingEnabled_) << "Creating a default sync pool";
     // Ensure there is a sync resource pool.
     resourcePoolSet().setResourcePool(
         ResourcePoolHandle::defaultSync(),
@@ -1438,8 +1401,7 @@ void ThriftServer::lockResourcePoolSet() {
 
   if (!resourcePoolSet().empty()) {
     XLOG_IF(INFO, infoLoggingEnabled_)
-        << "Resource pools (" << resourcePoolSet().size()
-        << "): " << resourcePoolSet().describe();
+        << "Resource pools configured: " << resourcePoolSet().size();
 
     auto descriptions = resourcePoolSet().poolsDescriptions();
     if (auto observer = getObserverShared()) {
@@ -1448,17 +1410,9 @@ void ThriftServer::lockResourcePoolSet() {
 
     size_t count{0};
     for (auto description : descriptions) {
-      XLOG_IF(INFO, infoLoggingEnabled_)
-          << fmt::format("Resource pool [{}]: {}", count++, description);
+      VLOG(1) << fmt::format("Resource pool [{}]: {}", count++, description);
     }
   }
-  if (FLAGS_thrift_server_enforces_qps_limit) {
-    XLOG_IF(INFO, infoLoggingEnabled_)
-        << "QPS limit will be enforced by Thrift Server";
-  } else {
-    XLOG_IF(INFO, infoLoggingEnabled_)
-        << "QPS limit will be enforced by Resource Pool";
-  }
 }
 
 /**
@@ -1663,7 +1617,7 @@ void ThriftServer::stopAcceptingAndJoinOutstandingRequests() {
                 .get(dumpSnapshotResult.timeout);
           } catch (...) {
             XLOG(ERR) << "Failed to dump server snapshot on long shutdown: "
-                      << folly::exceptionStr(std::current_exception());
+                      << folly::exceptionStr(folly::current_exception());
           }
         }
 
@@ -1718,8 +1672,8 @@ void ThriftServer::callOnStartServing() {
   {
     ServiceInterceptorBase::InitParams initParams;
     std::vector<folly::coro::Task<void>> tasks;
-    for (auto& serviceInterceptor : getServiceInterceptors()) {
-      tasks.emplace_back(serviceInterceptor->co_onStartServing(initParams));
+    for (const auto& info : getServiceInterceptors()) {
+      tasks.emplace_back(info.interceptor->co_onStartServing(initParams));
     }
     folly::coro::blockingWait(folly::coro::collectAllRange(std::move(tasks)));
   }
@@ -2434,16 +2388,68 @@ std::string ThriftServer::RuntimeServerActions::explain() const {
 /* static */ ThriftServer::ProcessedModuleSet
 ThriftServer::processModulesSpecification(ModulesSpecification&& specs) {
   ProcessedModuleSet result;
+#if FOLLY_HAS_COROUTINES
+  class ServiceInterceptorsCollector {
+   public:
+    void addModule(const ModulesSpecification::Info& moduleSpec) {
+      std::vector<std::shared_ptr<ServiceInterceptorBase>> serviceInterceptors =
+          moduleSpec.module->getServiceInterceptors();
+
+      std::vector<ServiceInterceptorInfo> result;
+      for (auto& interceptor : serviceInterceptors) {
+        auto qualifiedName =
+            fmt::format("{}.{}", moduleSpec.name, interceptor->getName());
+        if (seenNames_.find(qualifiedName) != seenNames_.end()) {
+          throw std::logic_error(
+              fmt::format("Duplicate ServiceInterceptor: {}", qualifiedName));
+        }
+        seenNames_.insert(qualifiedName);
+        result.emplace_back(ServiceInterceptorInfo{
+            std::move(qualifiedName), std::move(interceptor)});
+      }
+      interceptorsByModule_.emplace_back(std::move(result));
+    }
+
+    std::vector<ServiceInterceptorInfo> coalesce() && {
+      if constexpr (folly::kIsDebug) {
+        // Our contract guarantees ordering of interceptors within a module, but
+        // not across modules. In debug mode, let's shuffle order across modules
+        // so that tests break if someone is relying on such ordering.
+        // The quality of the RNG is not important here.
+        std::shuffle(
+            interceptorsByModule_.begin(),
+            interceptorsByModule_.end(),
+            folly::Random::DefaultGenerator());
+      }
+
+      std::vector<ServiceInterceptorInfo> result;
+      for (auto& interceptors : interceptorsByModule_) {
+        result.insert(
+            result.end(),
+            std::make_move_iterator(interceptors.begin()),
+            std::make_move_iterator(interceptors.end()));
+      }
+      return result;
+    }
+
+   private:
+    std::vector<std::vector<ServiceInterceptorInfo>> interceptorsByModule_;
+    std::unordered_set<std::string> seenNames_;
+  };
+#else
+  class ServiceInterceptorsCollector {
+   public:
+    void addModule(const ModulesSpecification::Info&) {}
+    std::vector<ServiceInterceptorInfo> coalesce() && { return {}; }
+  };
+#endif // FOLLY_HAS_COROUTINES
+  ServiceInterceptorsCollector serviceInterceptorsCollector;
 
   for (auto& info : specs.infos) {
     std::vector<std::shared_ptr<TProcessorEventHandler>> legacyEventHandlers =
         info.module->getLegacyEventHandlers();
     std::vector<std::shared_ptr<server::TServerEventHandler>>
         legacyServerEventHandlers = info.module->getLegacyServerEventHandlers();
-    std::vector<std::shared_ptr<ServiceInterceptorBase>> serviceInterceptors =
-        info.module->getServiceInterceptors();
-
-    result.modules.emplace_back(std::move(info));
 
     result.coalescedLegacyEventHandlers.insert(
         result.coalescedLegacyEventHandlers.end(),
@@ -2453,11 +2459,13 @@ ThriftServer::processModulesSpecification(ModulesSpecification&& specs) {
         result.coalescedLegacyServerEventHandlers.end(),
         std::make_move_iterator(legacyServerEventHandlers.begin()),
         std::make_move_iterator(legacyServerEventHandlers.end()));
-    result.coalescedServiceInterceptors.insert(
-        result.coalescedServiceInterceptors.end(),
-        std::make_move_iterator(serviceInterceptors.begin()),
-        std::make_move_iterator(serviceInterceptors.end()));
+
+    serviceInterceptorsCollector.addModule(info);
+
+    result.modules.emplace_back(std::move(info));
   }
+  result.coalescedServiceInterceptors =
+      std::move(serviceInterceptorsCollector).coalesce();
 
   return result;
 }
