@@ -31,15 +31,12 @@ THRIFT_FLAG_DECLARE_int64(
     server_ingress_memory_limit_enforcement_payload_size_min_bytes);
 
 namespace apache::thrift {
-class ThriftServerInitialConfig;
 
 class ThriftServerConfig {
  public:
   ThriftServerConfig() = default;
   ~ThriftServerConfig() = default;
 
-  explicit ThriftServerConfig(const ThriftServerInitialConfig& initialConfig);
-
   /**
    * Get the prefix for naming the CPU (pool) threads.
    *
@@ -872,61 +869,4 @@ class ThriftServerConfig {
   // its setters.
   std::atomic<bool> frozen_{false};
 };
-
-class ThriftServerInitialConfig {
- public:
-  FOLLY_CONSTEVAL ThriftServerInitialConfig() = default;
-
-  // to fix oss for now we'll have this as a pair<T, bool> to mimick the
-  // behavior of optional
-#define THRIFT_SERVER_INITIAL_CONFIG_DEFINE(TYPE, NAME, INIT_VALUE) \
- private:                                                           \
-  std::pair<TYPE, bool> NAME##_ = {INIT_VALUE, false};              \
-                                                                    \
- public:                                                            \
-  FOLLY_CONSTEVAL ThriftServerInitialConfig NAME(TYPE value) {      \
-    auto initialConfig(*this);                                      \
-    initialConfig.NAME##_.first = value;                            \
-    initialConfig.NAME##_.second = true;                            \
-    return initialConfig;                                           \
-  }
-
-  // replace with thrift struct if/when it becomes constexpr friendly
-
-  THRIFT_SERVER_INITIAL_CONFIG_DEFINE(uint32_t, maxRequests, 0)
-  THRIFT_SERVER_INITIAL_CONFIG_DEFINE(uint32_t, maxConnections, 0)
-  THRIFT_SERVER_INITIAL_CONFIG_DEFINE(uint64_t, maxResponseSize, 0)
-  THRIFT_SERVER_INITIAL_CONFIG_DEFINE(bool, useClientTimeout, true)
-  THRIFT_SERVER_INITIAL_CONFIG_DEFINE(
-      std::chrono::milliseconds, taskExpireTimeout, {})
-  THRIFT_SERVER_INITIAL_CONFIG_DEFINE(
-      std::chrono::milliseconds, streamExpireTimeout, {})
-  THRIFT_SERVER_INITIAL_CONFIG_DEFINE(
-      std::chrono::milliseconds, queueTimeout, {})
-  THRIFT_SERVER_INITIAL_CONFIG_DEFINE(
-      std::chrono::nanoseconds, socketQueueTimeout, {})
-  THRIFT_SERVER_INITIAL_CONFIG_DEFINE(size_t, egressMemoryLimit, 0)
-  THRIFT_SERVER_INITIAL_CONFIG_DEFINE(
-      size_t, egressBufferBackpressureThreshold, 0)
-  THRIFT_SERVER_INITIAL_CONFIG_DEFINE(size_t, ingressMemoryLimit, 0)
-  THRIFT_SERVER_INITIAL_CONFIG_DEFINE(
-      size_t, minPayloadSizeToEnforceIngressMemoryLimit, 0)
-
-  /*
-   * Running UBSan causes some tests to FATAL. The current theory is that this
-   * is caused by the compiler not generating proper code for FOLLY_CONSTEVAL.
-   * For now, we're adding a dummy field at the end as a hack to workaround this
-   * issue, until its fixed. All new fields should be added before this last
-   * dummy field
-   */
-
-  THRIFT_SERVER_INITIAL_CONFIG_DEFINE(
-      std::chrono::milliseconds,
-      lastUnusedField,
-      {}) // DO NOT ADD FIELDS AFTER THIS
-#undef THRIFT_SERVER_INITIAL_CONFIG_DEFINE
-
- private:
-  friend class ThriftServerConfig;
-};
 } // namespace apache::thrift
