@@ -52,7 +52,6 @@ ThriftRequestCore::ThriftRequestCore(
     : serverConfigs_(serverConfigs),
       kind_(metadata.kind_ref().value_or(
           RpcKind::SINGLE_REQUEST_SINGLE_RESPONSE)),
-      metricCollector_{serverConfigs_.getMetricCollector()},
       checksumRequested_(metadata.crc32c_ref().has_value()),
       loadMetric_(
           metadata.loadMetric_ref()
@@ -125,7 +124,6 @@ ThriftRequestCore::ThriftRequestCore(
   if (auto* observer = serverConfigs_.getObserver()) {
     observer->receivedRequest(&reqContext_.getMethodName());
   }
-  metricCollector_.requestReceived();
 }
 
 bool ThriftRequestCore::includeInRecentRequestsCount(
