@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -151,6 +151,11 @@ class FakeThreadManager : public apache::thrift::concurrency::ThreadManager {
     LOG(FATAL) << "Method not implemented in this fake object";
   }
 
+  bool codelEnabled() const override {
+    LOG(FATAL) << "Method not implemented in this fake object";
+    return false;
+  }
+
   folly::Codel* getCodel() override {
     LOG(FATAL) << "Method not implemented in this fake object";
     return nullptr;
