@@ -55,6 +55,23 @@ THRIFT_FLAG_DECLARE_bool(raw_client_rocket_upgrade_enabled_v2);
 THRIFT_FLAG_DECLARE_bool(server_rocket_upgrade_enabled);
 THRIFT_FLAG_DECLARE_int64(thrift_client_checksum_sampling_rate);
 
+// Timeout used for polling callCompleted_ to make the test more robust.
+constexpr auto kPollTimeout = std::chrono::seconds(5);
+
+void pollSleep() {
+  std::this_thread::sleep_for(std::chrono::milliseconds(1)); // NOLINT
+}
+
+#define EXPECT_EQ_POLL(actual_ref, expected, timeout)                \
+  {                                                                  \
+    auto start = std::chrono::steady_clock::now();                   \
+    while ((actual_ref) != (expected) &&                             \
+           (std::chrono::steady_clock::now() - start) < (timeout)) { \
+      pollSleep();                                                   \
+    }                                                                \
+  }                                                                  \
+  EXPECT_EQ(actual_ref, expected)
+
 namespace apache {
 namespace thrift {
 
@@ -348,7 +365,7 @@ void TransportCompatibilityTest::TestObserverSendReceiveRequests() {
     EXPECT_EQ(5, server_->observer_->sentReply_);
     EXPECT_EQ(5, server_->observer_->receivedRequest_);
     if (FLAGS_transport != "http2") {
-      EXPECT_EQ(5, server_->observer_->callCompleted_);
+      EXPECT_EQ_POLL(server_->observer_->callCompleted_, 5, kPollTimeout);
     }
   });
 }
@@ -405,7 +422,7 @@ void TransportCompatibilityTest::TestRequestResponse_Simple() {
     EXPECT_EQ(3, client->future_sumTwoNumbers(1, 2).get());
     EXPECT_EQ(8, client->future_add(5).get());
     if (FLAGS_transport != "http2") {
-      EXPECT_EQ(5, server_->observer_->callCompleted_);
+      EXPECT_EQ_POLL(server_->observer_->callCompleted_, 5, kPollTimeout);
     }
   });
 }
@@ -571,7 +588,7 @@ void TransportCompatibilityTest::TestRequestResponse_Header_Load() {
   connectToServer([](std::unique_ptr<TestServiceAsyncClient> client) {
     RpcOptions rpcOptions;
     rpcOptions.setWriteHeader("header_from_client", "2");
-    rpcOptions.setWriteHeader(THeader::QUERY_LOAD_HEADER, {});
+    rpcOptions.setWriteHeader((std::string)THeader::QUERY_LOAD_HEADER, {});
     auto resultAndHeaders = client->header_future_headers(rpcOptions).get();
     const auto readHeaders =
         std::move(resultAndHeaders.second)->releaseHeaders();
@@ -879,7 +896,7 @@ void TransportCompatibilityTest::TestOneway_Simple() {
     EXPECT_EQ(2, server_->observer_->receivedRequest_);
     EXPECT_EQ(1, server_->observer_->sentReply_);
     if (FLAGS_transport != "http2") {
-      EXPECT_EQ(1, server_->observer_->callCompleted_);
+      EXPECT_EQ_POLL(server_->observer_->callCompleted_, 1, kPollTimeout);
     }
   });
 }
@@ -1271,33 +1288,17 @@ void TransportCompatibilityTest::TestCustomAsyncProcessor() {
     void processSerializedCompressedRequestWithMetadata(
         apache::thrift::ResponseChannelRequest::UniquePtr req,
         apache::thrift::SerializedCompressedRequest&& serializedRequest,
-        const apache::thrift::AsyncProcessorFactory::MethodMetadata&,
+        const apache::thrift::AsyncProcessorFactory::MethodMetadata&
+            methodMetadata,
         apache::thrift::protocol::PROTOCOL_TYPES protType,
         apache::thrift::Cpp2RequestContext* context,
         folly::EventBase* eb,
         apache::thrift::concurrency::ThreadManager* tm) override {
-      underlyingProcessor_->processSerializedRequest(
+      underlyingProcessor_->processSerializedCompressedRequestWithMetadata(
           apache::thrift::ResponseChannelRequest::UniquePtr(
               new TestResponseChannelRequest(std::move(req))),
-          std::move(serializedRequest).uncompress(),
-          protType,
-          context,
-          eb,
-          tm);
-    }
-
-    void processSerializedRequest(
-        apache::thrift::ResponseChannelRequest::UniquePtr req,
-        apache::thrift::SerializedRequest&& serializedRequest,
-        apache::thrift::protocol::PROTOCOL_TYPES protType,
-        apache::thrift::Cpp2RequestContext* context,
-        folly::EventBase* eb,
-        apache::thrift::concurrency::ThreadManager* tm) override {
-      processSerializedCompressedRequestWithMetadata(
-          std::move(req),
-          apache::thrift::SerializedCompressedRequest(
-              std::move(serializedRequest)),
-          apache::thrift::AsyncProcessorFactory::MethodMetadata(),
+          std::move(serializedRequest),
+          methodMetadata,
           protType,
           context,
           eb,
