@@ -29,7 +29,7 @@
 #include <folly/io/async/AsyncTransport.h>
 
 #include <thrift/lib/cpp2/server/Cpp2Worker.h>
-#include <thrift/lib/cpp2/server/metrics/MetricCollector.h>
+#include <thrift/lib/cpp2/server/ThriftServerInternals.h>
 #include <thrift/lib/cpp2/transport/rocket/framing/FrameType.h>
 #include <thrift/lib/cpp2/transport/rocket/server/RocketServerConnection.h>
 #include <thrift/lib/cpp2/transport/rocket/server/ThriftRocketServerHandler.h>
@@ -60,7 +60,8 @@ THRIFT_DETAIL_REGISTER_SERVER_EXTENSION_DEFAULT(
 } // namespace detail
 
 RocketRoutingHandler::RocketRoutingHandler(ThriftServer& server)
-    : metricCollector_{server.getMetricCollector()} {
+    : streamMetricCallback_(
+          detail::ThriftServerInternals(server).getStreamMetricCallback()) {
   auto addSetupFramehandler = [&](auto&& handlerFactory) {
     if (auto handler = handlerFactory(server)) {
       setupFrameHandlers_.push_back(std::move(handler));
@@ -140,6 +141,7 @@ void RocketRoutingHandler::handleConnection(
       server->getEgressBufferRecoveryFactor();
   cfg.socketOptions = &server->getPerConnectionSocketOptions();
   cfg.parserAllocator = server->getCustomAllocatorForParser();
+  const std::string& securotyProtocol = sock->getSecurityProtocol();
 
   auto* const sockPtr = sock.get();
   auto* const connection = new rocket::RocketServerConnection(
@@ -148,6 +150,7 @@ void RocketRoutingHandler::handleConnection(
           worker, *address, sockPtr, setupFrameHandlers_),
       worker->getIngressMemoryTracker(),
       worker->getEgressMemoryTracker(),
+      streamMetricCallback_,
       cfg);
   onConnection(*connection);
   connectionManager->addConnection(
@@ -156,7 +159,10 @@ void RocketRoutingHandler::handleConnection(
       /* connectionAgeTimeout */ true);
 
   if (auto* observer = server->getObserver()) {
-    observer->connAccepted(tinfo);
+    observer->connAccepted(
+        tinfo,
+        server::TServerObserver::ConnectionInfo(
+            reinterpret_cast<uint64_t>(sockPtr), securotyProtocol));
     observer->activeConnections(
         connectionManager->getNumConnections() *
         server->getNumIOWorkerThreads());
