@@ -53,18 +53,19 @@
 #include <thrift/lib/cpp2/transport/rocket/server/RocketStreamClientCallback.h>
 
 THRIFT_FLAG_DEFINE_bool(enable_rocket_connection_observers, false);
+THRIFT_FLAG_DEFINE_bool(enable_stream_graceful_shutdown, true);
+THRIFT_FLAG_DEFINE_bool(thrift_enable_stream_counters, true);
 
 namespace apache {
 namespace thrift {
 namespace rocket {
 
-constexpr std::chrono::seconds RocketServerConnection::SocketDrainer::kTimeout;
-
 RocketServerConnection::RocketServerConnection(
     folly::AsyncTransport::UniquePtr socket,
     std::unique_ptr<RocketServerHandler> frameHandler,
     MemoryTracker& ingressMemoryTracker,
     MemoryTracker& egressMemoryTracker,
+    StreamMetricCallback& streamMetricCallback,
     const Config& cfg)
     : evb_(*socket->getEventBase()),
       socket_(std::move(socket)),
@@ -86,6 +87,8 @@ RocketServerConnection::RocketServerConnection(
       socketDrainer_(*this),
       ingressMemoryTracker_(ingressMemoryTracker),
       egressMemoryTracker_(egressMemoryTracker),
+      streamMetricCallback_(streamMetricCallback),
+      enableObservers_(THRIFT_FLAG(enable_rocket_connection_observers)),
       observerContainer_(this) {
   CHECK(socket_);
   CHECK(frameHandler_);
@@ -115,6 +118,14 @@ RocketServerConnection::RocketServerConnection(
   }
 }
 
+namespace {
+StreamMetricCallback& getNoopStreamMetricCallback() {
+  static folly::Indestructible<NoopStreamMetricCallback>
+      kNoopStreamMetricCallback;
+  return *kNoopStreamMetricCallback;
+}
+} // namespace
+
 RocketStreamClientCallback* FOLLY_NULLABLE
 RocketServerConnection::createStreamClientCallback(
     StreamId streamId,
@@ -125,7 +136,12 @@ RocketServerConnection::createStreamClientCallback(
     return nullptr;
   }
   auto cb = std::make_unique<RocketStreamClientCallback>(
-      streamId, connection, initialRequestN);
+      streamId,
+      connection,
+      initialRequestN,
+      THRIFT_FLAG(thrift_enable_stream_counters)
+          ? streamMetricCallback_
+          : getNoopStreamMetricCallback());
   auto cbPtr = cb.get();
   it->second = std::move(cb);
   return cbPtr;
@@ -231,6 +247,19 @@ void RocketServerConnection::send(
       std::move(data), std::move(cb), streamId, std::move(fds));
 }
 
+void RocketServerConnection::sendErrorAfterDrain(
+    StreamId streamId, RocketException&& rex) {
+  evb_.dcheckIsInEventBaseThread();
+  DCHECK(
+      state_ == ConnectionState::CLOSING || state_ == ConnectionState::CLOSED);
+
+  writeBatcher_.enqueueWrite(
+      ErrorFrame(streamId, std::move(rex)).serialize(),
+      nullptr,
+      streamId,
+      folly::SocketFds{});
+}
+
 RocketServerConnection::~RocketServerConnection() {
   DCHECK(inflightRequests_ == 0);
   DCHECK(inflightWritesQueue_.empty());
@@ -255,6 +284,19 @@ RocketServerConnection::~RocketServerConnection() {
   }
 }
 
+namespace {
+StreamRpcError getStreamConnectionClosingError() {
+  StreamRpcError streamRpcError;
+  streamRpcError.code_ref() = StreamRpcErrorCode::SERVER_CLOSING_CONNECTION;
+  streamRpcError.name_utf8_ref() =
+      apache::thrift::TEnumTraits<StreamRpcErrorCode>::findName(
+          StreamRpcErrorCode::SERVER_CLOSING_CONNECTION);
+  streamRpcError.what_utf8_ref() =
+      "Server closing connection, cancelling stream";
+  return streamRpcError;
+}
+} // namespace
+
 void RocketServerConnection::closeIfNeeded() {
   if (state_ == ConnectionState::DRAINING && inflightRequests_ == 0 &&
       inflightSinkFinalResponses_ == 0) {
@@ -306,7 +348,14 @@ void RocketServerConnection::closeIfNeeded() {
     // Calling application callback may trigger rehashing.
     folly::variant_match(
         callback,
-        [](const std::unique_ptr<RocketStreamClientCallback>& callback) {
+        [&](const std::unique_ptr<RocketStreamClientCallback>& callback) {
+          if (THRIFT_FLAG(enable_stream_graceful_shutdown)) {
+            sendErrorAfterDrain(
+                callback->getStreamId(),
+                RocketException(
+                    ErrorCode::CANCELED,
+                    packCompact(getStreamConnectionClosingError())));
+          }
           callback->onStreamCancel();
         },
         [](const std::unique_ptr<RocketSinkClientCallback>& callback) {
@@ -1082,7 +1131,7 @@ void RocketServerConnection::applyQosMarking(
   } catch (...) {
     FB_LOG_EVERY_MS(WARNING, 60 * 1000)
         << "Failed to apply DSCP to socket: "
-        << folly::exceptionStr(std::current_exception());
+        << folly::exceptionStr(folly::current_exception());
   }
 }
 
