@@ -37,7 +37,6 @@
 #include <thrift/lib/cpp2/server/LoggingEvent.h>
 #include <thrift/lib/cpp2/server/LoggingEventHelper.h>
 #include <thrift/lib/cpp2/server/MonitoringMethodNames.h>
-#include <thrift/lib/cpp2/server/metrics/MetricCollector.h>
 #include <thrift/lib/cpp2/transport/core/RpcMetadataUtil.h>
 #include <thrift/lib/cpp2/transport/core/ThriftRequest.h>
 #include <thrift/lib/cpp2/transport/rocket/FdSocket.h>
@@ -100,28 +99,57 @@ ThriftRocketServerHandler::ThriftRocketServerHandler(
           nullptr, /* eventBaseManager */
           nullptr, /* x509PeerCert */
           worker_->getServer()->getClientIdentityHook(),
-          worker_.get()),
+          worker_.get(),
+          apache::thrift::detail::getServiceInterceptorsIfServerIsSetUp(
+              *worker_->getServer())
+              .size()),
       setupFrameHandlers_(handlers),
       version_(static_cast<int32_t>(std::min(
           kRocketServerMaxVersion, THRIFT_FLAG(rocket_server_max_version)))),
       maxResponseWriteTime_(worker_->getServer()
                                 ->getThriftServerConfig()
                                 .getMaxResponseWriteTime()
-                                .get()),
-      metricCollector_{worker_->getServer()->getMetricCollector()} {
+                                .get()) {
   connContext_.setTransportType(Cpp2ConnContext::TransportType::ROCKET);
   for (const auto& handler : worker_->getServer()->getEventHandlersUnsafe()) {
     handler->newConnection(&connContext_);
   }
+#if FOLLY_HAS_COROUTINES
+  const auto& serviceInterceptorsInfo =
+      apache::thrift::detail::getServiceInterceptorsIfServerIsSetUp(
+          *worker_->getServer());
+  for (std::size_t i = 0; i < serviceInterceptorsInfo.size(); ++i) {
+    ServiceInterceptorBase::ConnectionInfo connectionInfo{
+        &connContext_,
+        connContext_.getStorageForServiceInterceptorOnConnectionByIndex(i)};
+    serviceInterceptorsInfo[i].interceptor->internal_onConnection(
+        connectionInfo);
+  }
+#endif // FOLLY_HAS_COROUTINES
 }
 
 ThriftRocketServerHandler::~ThriftRocketServerHandler() {
+#if FOLLY_HAS_COROUTINES
+  const auto& serviceInterceptorsInfo =
+      apache::thrift::detail::getServiceInterceptorsIfServerIsSetUp(
+          *worker_->getServer());
+  for (std::size_t i = 0; i < serviceInterceptorsInfo.size(); ++i) {
+    ServiceInterceptorBase::ConnectionInfo connectionInfo{
+        &connContext_,
+        connContext_.getStorageForServiceInterceptorOnConnectionByIndex(i)};
+    serviceInterceptorsInfo[i].interceptor->internal_onConnectionClosed(
+        connectionInfo);
+  }
+#endif // FOLLY_HAS_COROUTINES
+
   for (const auto& handler : worker_->getServer()->getEventHandlersUnsafe()) {
     handler->connectionDestroyed(&connContext_);
   }
   // Ensure each connAccepted() call has a matching connClosed()
   if (auto* observer = worker_->getServer()->getObserver()) {
-    observer->connClosed();
+    observer->connClosed(server::TServerObserver::ConnectionInfo(
+        reinterpret_cast<uint64_t>(transport_),
+        transport_->getSecurityProtocol()));
   }
 }
 
@@ -450,7 +478,6 @@ void ThriftRocketServerHandler::handleRequestCommon(
       if (auto* observer = serverConfigs_->getObserver()) {
         observer->taskKilled();
       }
-      metricCollector_.requestRejected({});
       handleRequestWithFdsExtractionFailure(
           makeActiveRequest(
               std::move(metadata),
@@ -508,7 +535,7 @@ void ThriftRocketServerHandler::handleRequestCommon(
               std::move(metadata),
               rocket::Payload{},
               createDefaultRequestContext()),
-          folly::exceptionStr(std::current_exception()).toStdString());
+          folly::exceptionStr(folly::current_exception()).toStdString());
       return;
     }
   }
@@ -684,7 +711,6 @@ void ThriftRocketServerHandler::handleRequestWithBadMetadata(
   if (auto* observer = serverConfigs_->getObserver()) {
     observer->taskKilled();
   }
-  metricCollector_.requestRejected({});
 
   request->sendErrorWrapped(
       folly::make_exception_wrapper<TApplicationException>(
@@ -698,7 +724,6 @@ void ThriftRocketServerHandler::handleRequestWithBadChecksum(
   if (auto* observer = serverConfigs_->getObserver()) {
     observer->taskKilled();
   }
-  metricCollector_.requestRejected({});
 
   request->sendErrorWrapped(
       folly::make_exception_wrapper<TApplicationException>(
@@ -711,7 +736,6 @@ void ThriftRocketServerHandler::handleDecompressionFailure(
   if (auto* observer = serverConfigs_->getObserver()) {
     observer->taskKilled();
   }
-  metricCollector_.requestRejected({});
 
   request->sendErrorWrapped(
       folly::make_exception_wrapper<TApplicationException>(
@@ -723,10 +747,8 @@ void ThriftRocketServerHandler::handleDecompressionFailure(
 void ThriftRocketServerHandler::handleRequestOverloadedServer(
     ThriftRequestCoreUniquePtr request, OverloadResult&& overloadResult) {
   if (auto* observer = serverConfigs_->getObserver()) {
-    observer->serverOverloaded();
+    observer->serverOverloaded(overloadResult.loadShedder);
   }
-  metricCollector_.requestRejected(
-      {RequestRejectedScope::ServerOverloaded{overloadResult.loadShedder}});
 
   request->sendErrorWrapped(
       folly::make_exception_wrapper<TApplicationException>(
@@ -745,7 +767,6 @@ void ThriftRocketServerHandler::handleQuotaExceededException(
   if (auto* observer = serverConfigs_->getObserver()) {
     observer->taskKilled();
   }
-  metricCollector_.requestRejected({});
 
   request->sendErrorWrapped(
       folly::make_exception_wrapper<TApplicationException>(
@@ -759,7 +780,6 @@ void ThriftRocketServerHandler::handleAppError(
   if (auto* observer = serverConfigs_->getObserver()) {
     observer->taskKilled();
   }
-  metricCollector_.requestRejected({});
 
   folly::variant_match(
       appErrorResult,
@@ -777,7 +797,6 @@ void ThriftRocketServerHandler::handleRequestWithFdsExtractionFailure(
   if (auto* observer = serverConfigs_->getObserver()) {
     observer->taskKilled();
   }
-  metricCollector_.requestRejected({});
 
   request->sendErrorWrapped(
       folly::make_exception_wrapper<TApplicationException>(
@@ -791,7 +810,6 @@ void ThriftRocketServerHandler::handleServerNotReady(
   if (auto* observer = serverConfigs_->getObserver()) {
     observer->taskKilled();
   }
-  metricCollector_.requestRejected({});
 
   request->sendErrorWrapped(
       folly::make_exception_wrapper<TApplicationException>(
@@ -804,7 +822,6 @@ void ThriftRocketServerHandler::handleServerShutdown(
   if (auto* observer = serverConfigs_->getObserver()) {
     observer->taskKilled();
   }
-  metricCollector_.requestRejected({});
 
   request->sendErrorWrapped(
       folly::make_exception_wrapper<TApplicationException>(
@@ -819,7 +836,6 @@ void ThriftRocketServerHandler::handleInjectedFault(
       if (auto* observer = serverConfigs_->getObserver()) {
         observer->taskKilled();
       }
-      metricCollector_.requestRejected({});
 
       request->sendErrorWrapped(
           folly::make_exception_wrapper<TApplicationException>(
