@@ -40,7 +40,6 @@ namespace thrift {
 
 class AsyncProcessor;
 class Cpp2ConnContext;
-class MetricCollector;
 class RequestRpcMetadata;
 class ThriftRequestCore;
 using ThriftRequestCoreUniquePtr =
@@ -128,8 +127,6 @@ class ThriftRocketServerHandler : public RocketServerHandler {
 
   folly::once_flag setupLoggingFlag_;
 
-  const MetricCollector& metricCollector_;
-
   template <class F>
   void handleRequestCommon(
       Payload&& payload, F&& makeRequest, RpcKind expectedKind);
