@@ -124,14 +124,15 @@ void testServerOneInput(const uint8_t* Data, size_t Size) {
   std::vector<std::unique_ptr<apache::thrift::rocket::SetupFrameHandler>> v;
   folly::SocketAddress address;
   MemoryTracker memoryTracker;
+  NoopStreamMetricCallback streamMetricCallback;
 
   auto connection = new apache::thrift::rocket::RocketServerConnection(
       std::move(sock),
       std::make_unique<apache::thrift::rocket::ThriftRocketServerHandler>(
           worker, address, sockPtr, v),
       memoryTracker, // (ingress)
-      memoryTracker // (egress)
-  );
+      memoryTracker, // (egress)
+      streamMetricCallback);
 
   folly::DelayedDestruction::DestructorGuard dg(connection);
   apache::thrift::rocket::Parser<apache::thrift::rocket::RocketServerConnection>
