@@ -27,37 +27,47 @@ import (
 const BinaryVersionMask uint32 = 0xffff0000
 const BinaryVersion1 uint32 = 0x80010000
 
-type BinaryProtocol struct {
-	trans         RichTransport
-	origTransport Transport
-	reader        io.Reader
-	writer        io.Writer
-	strictRead    bool
-	strictWrite   bool
-	buffer        [64]byte
+type binaryProtocol struct {
+	binaryEncoder
+	binaryDecoder
+	io.Closer
 }
 
-func NewBinaryProtocolTransport(t Transport) *BinaryProtocol {
+// NewBinaryProtocolTransport creates a new protocol handler using a buffer
+func NewBinaryProtocolTransport(t io.ReadWriteCloser) Format {
 	return NewBinaryProtocol(t, false, true)
 }
 
-func NewBinaryProtocol(t Transport, strictRead, strictWrite bool) *BinaryProtocol {
-	p := &BinaryProtocol{origTransport: t, strictRead: strictRead, strictWrite: strictWrite}
-	if et, ok := t.(RichTransport); ok {
-		p.trans = et
-	} else {
-		p.trans = NewRichTransport(t)
-	}
-	p.reader = p.trans
-	p.writer = p.trans
+// NewBinaryProtocol creates a new protocol handler using a buffer
+func NewBinaryProtocol(t io.ReadWriteCloser, strictRead, strictWrite bool) Format {
+	p := &binaryProtocol{}
+	p.binaryDecoder = binaryDecoder{reader: t, strictRead: strictRead}
+	p.binaryEncoder = binaryEncoder{writer: t, strictWrite: strictWrite}
+	p.Closer = t
 	return p
 }
 
+func newBinaryEncoder(writer io.Writer) Encoder {
+	// This default for binaryEncoder's strictWrite matches cpp implementation
+	return &binaryEncoder{writer: writer, strictWrite: true}
+}
+
+func newBinaryDecoder(reader io.Reader) Decoder {
+	// This default for binaryEncoder's strictRead matches cpp implementation
+	return &binaryDecoder{reader: reader, strictRead: false}
+}
+
 /**
  * Writing Methods
  */
 
-func (p *BinaryProtocol) WriteMessageBegin(name string, typeID MessageType, seqID int32) error {
+type binaryEncoder struct {
+	writer      io.Writer
+	strictWrite bool
+	buffer      [64]byte
+}
+
+func (p *binaryEncoder) WriteMessageBegin(name string, typeID MessageType, seqID int32) error {
 	if p.strictWrite {
 		version := uint32(VERSION_1) | uint32(typeID)
 		e := p.WriteI32(int32(version))
@@ -84,19 +94,19 @@ func (p *BinaryProtocol) WriteMessageBegin(name string, typeID MessageType, seqI
 	}
 }
 
-func (p *BinaryProtocol) WriteMessageEnd() error {
+func (p *binaryEncoder) WriteMessageEnd() error {
 	return nil
 }
 
-func (p *BinaryProtocol) WriteStructBegin(name string) error {
+func (p *binaryEncoder) WriteStructBegin(name string) error {
 	return nil
 }
 
-func (p *BinaryProtocol) WriteStructEnd() error {
+func (p *binaryEncoder) WriteStructEnd() error {
 	return nil
 }
 
-func (p *BinaryProtocol) WriteFieldBegin(name string, typeID Type, id int16) error {
+func (p *binaryEncoder) WriteFieldBegin(name string, typeID Type, id int16) error {
 	e := p.WriteByte(byte(typeID))
 	if e != nil {
 		return e
@@ -105,16 +115,16 @@ func (p *BinaryProtocol) WriteFieldBegin(name string, typeID Type, id int16) err
 	return e
 }
 
-func (p *BinaryProtocol) WriteFieldEnd() error {
+func (p *binaryEncoder) WriteFieldEnd() error {
 	return nil
 }
 
-func (p *BinaryProtocol) WriteFieldStop() error {
+func (p *binaryEncoder) WriteFieldStop() error {
 	e := p.WriteByte(STOP)
 	return e
 }
 
-func (p *BinaryProtocol) WriteMapBegin(keyType Type, valueType Type, size int) error {
+func (p *binaryEncoder) WriteMapBegin(keyType Type, valueType Type, size int) error {
 	e := p.WriteByte(byte(keyType))
 	if e != nil {
 		return e
@@ -127,11 +137,11 @@ func (p *BinaryProtocol) WriteMapBegin(keyType Type, valueType Type, size int) e
 	return e
 }
 
-func (p *BinaryProtocol) WriteMapEnd() error {
+func (p *binaryEncoder) WriteMapEnd() error {
 	return nil
 }
 
-func (p *BinaryProtocol) WriteListBegin(elemType Type, size int) error {
+func (p *binaryEncoder) WriteListBegin(elemType Type, size int) error {
 	e := p.WriteByte(byte(elemType))
 	if e != nil {
 		return e
@@ -140,11 +150,11 @@ func (p *BinaryProtocol) WriteListBegin(elemType Type, size int) error {
 	return e
 }
 
-func (p *BinaryProtocol) WriteListEnd() error {
+func (p *binaryEncoder) WriteListEnd() error {
 	return nil
 }
 
-func (p *BinaryProtocol) WriteSetBegin(elemType Type, size int) error {
+func (p *binaryEncoder) WriteSetBegin(elemType Type, size int) error {
 	e := p.WriteByte(byte(elemType))
 	if e != nil {
 		return e
@@ -153,61 +163,61 @@ func (p *BinaryProtocol) WriteSetBegin(elemType Type, size int) error {
 	return e
 }
 
-func (p *BinaryProtocol) WriteSetEnd() error {
+func (p *binaryEncoder) WriteSetEnd() error {
 	return nil
 }
 
-func (p *BinaryProtocol) WriteBool(value bool) error {
+func (p *binaryEncoder) WriteBool(value bool) error {
 	if value {
 		return p.WriteByte(1)
 	}
 	return p.WriteByte(0)
 }
 
-func (p *BinaryProtocol) WriteByte(value byte) error {
-	e := p.trans.WriteByte(value)
+func (p *binaryEncoder) WriteByte(value byte) error {
+	e := writeByte(p.writer, value)
 	return NewProtocolException(e)
 }
 
-func (p *BinaryProtocol) WriteI16(value int16) error {
+func (p *binaryEncoder) WriteI16(value int16) error {
 	v := p.buffer[0:2]
 	binary.BigEndian.PutUint16(v, uint16(value))
 	_, e := p.writer.Write(v)
 	return NewProtocolException(e)
 }
 
-func (p *BinaryProtocol) WriteI32(value int32) error {
+func (p *binaryEncoder) WriteI32(value int32) error {
 	v := p.buffer[0:4]
 	binary.BigEndian.PutUint32(v, uint32(value))
 	_, e := p.writer.Write(v)
 	return NewProtocolException(e)
 }
 
-func (p *BinaryProtocol) WriteI64(value int64) error {
+func (p *binaryEncoder) WriteI64(value int64) error {
 	v := p.buffer[0:8]
 	binary.BigEndian.PutUint64(v, uint64(value))
 	_, err := p.writer.Write(v)
 	return NewProtocolException(err)
 }
 
-func (p *BinaryProtocol) WriteDouble(value float64) error {
+func (p *binaryEncoder) WriteDouble(value float64) error {
 	return p.WriteI64(int64(math.Float64bits(value)))
 }
 
-func (p *BinaryProtocol) WriteFloat(value float32) error {
+func (p *binaryEncoder) WriteFloat(value float32) error {
 	return p.WriteI32(int32(math.Float32bits(value)))
 }
 
-func (p *BinaryProtocol) WriteString(value string) error {
+func (p *binaryEncoder) WriteString(value string) error {
 	e := p.WriteI32(int32(len(value)))
 	if e != nil {
 		return e
 	}
-	_, err := p.trans.WriteString(value)
+	_, err := p.writer.Write([]byte(value))
 	return NewProtocolException(err)
 }
 
-func (p *BinaryProtocol) WriteBinary(value []byte) error {
+func (p *binaryEncoder) WriteBinary(value []byte) error {
 	err := p.WriteI32(int32(len(value)))
 	if err != nil {
 		return err
@@ -218,11 +228,25 @@ func (p *BinaryProtocol) WriteBinary(value []byte) error {
 	return NewProtocolException(err)
 }
 
+func (p *binaryEncoder) Flush() (err error) {
+	flusher, ok := p.writer.(Flusher)
+	if !ok {
+		return nil
+	}
+	return NewProtocolException(flusher.Flush())
+}
+
 /**
  * Reading methods
  */
 
-func (p *BinaryProtocol) ReadMessageBegin() (name string, typeID MessageType, seqID int32, err error) {
+type binaryDecoder struct {
+	reader     io.Reader
+	strictRead bool
+	buffer     [64]byte
+}
+
+func (p *binaryDecoder) ReadMessageBegin() (name string, typeID MessageType, seqID int32, err error) {
 	size, e := p.ReadI32()
 	if e != nil {
 		return "", typeID, 0, NewProtocolException(e)
@@ -262,19 +286,19 @@ func (p *BinaryProtocol) ReadMessageBegin() (name string, typeID MessageType, se
 	return name, typeID, seqID, nil
 }
 
-func (p *BinaryProtocol) ReadMessageEnd() error {
+func (p *binaryDecoder) ReadMessageEnd() error {
 	return nil
 }
 
-func (p *BinaryProtocol) ReadStructBegin() (name string, err error) {
+func (p *binaryDecoder) ReadStructBegin() (name string, err error) {
 	return
 }
 
-func (p *BinaryProtocol) ReadStructEnd() error {
+func (p *binaryDecoder) ReadStructEnd() error {
 	return nil
 }
 
-func (p *BinaryProtocol) ReadFieldBegin() (name string, typeID Type, seqID int16, err error) {
+func (p *binaryDecoder) ReadFieldBegin() (name string, typeID Type, seqID int16, err error) {
 	t, err := p.ReadByte()
 	typeID = Type(t)
 	if err != nil {
@@ -286,13 +310,13 @@ func (p *BinaryProtocol) ReadFieldBegin() (name string, typeID Type, seqID int16
 	return name, typeID, seqID, err
 }
 
-func (p *BinaryProtocol) ReadFieldEnd() error {
+func (p *binaryDecoder) ReadFieldEnd() error {
 	return nil
 }
 
 var invalidDataLength = NewProtocolExceptionWithType(INVALID_DATA, errors.New("Invalid data length"))
 
-func (p *BinaryProtocol) ReadMapBegin() (kType, vType Type, size int, err error) {
+func (p *binaryDecoder) ReadMapBegin() (kType, vType Type, size int, err error) {
 	k, e := p.ReadByte()
 	if e != nil {
 		err = NewProtocolException(e)
@@ -314,7 +338,8 @@ func (p *BinaryProtocol) ReadMapBegin() (kType, vType Type, size int, err error)
 		err = invalidDataLength
 		return
 	}
-	if uint64(size32*2) > p.trans.RemainingBytes() || p.trans.RemainingBytes() == UnknownRemaining {
+	remainingBytes := remainingBytes(p.reader)
+	if uint64(size32*2) > remainingBytes || remainingBytes == UnknownRemaining {
 		err = invalidDataLength
 		return
 	}
@@ -322,11 +347,11 @@ func (p *BinaryProtocol) ReadMapBegin() (kType, vType Type, size int, err error)
 	return kType, vType, size, nil
 }
 
-func (p *BinaryProtocol) ReadMapEnd() error {
+func (p *binaryDecoder) ReadMapEnd() error {
 	return nil
 }
 
-func (p *BinaryProtocol) ReadListBegin() (elemType Type, size int, err error) {
+func (p *binaryDecoder) ReadListBegin() (elemType Type, size int, err error) {
 	b, e := p.ReadByte()
 	if e != nil {
 		err = NewProtocolException(e)
@@ -342,7 +367,8 @@ func (p *BinaryProtocol) ReadListBegin() (elemType Type, size int, err error) {
 		err = invalidDataLength
 		return
 	}
-	if uint64(size32) > p.trans.RemainingBytes() || p.trans.RemainingBytes() == UnknownRemaining {
+	remainingBytes := remainingBytes(p.reader)
+	if uint64(size32) > remainingBytes || remainingBytes == UnknownRemaining {
 		err = invalidDataLength
 		return
 	}
@@ -351,11 +377,11 @@ func (p *BinaryProtocol) ReadListBegin() (elemType Type, size int, err error) {
 	return
 }
 
-func (p *BinaryProtocol) ReadListEnd() error {
+func (p *binaryDecoder) ReadListEnd() error {
 	return nil
 }
 
-func (p *BinaryProtocol) ReadSetBegin() (elemType Type, size int, err error) {
+func (p *binaryDecoder) ReadSetBegin() (elemType Type, size int, err error) {
 	b, e := p.ReadByte()
 	if e != nil {
 		err = NewProtocolException(e)
@@ -371,7 +397,8 @@ func (p *BinaryProtocol) ReadSetBegin() (elemType Type, size int, err error) {
 		err = invalidDataLength
 		return
 	}
-	if uint64(size32) > p.trans.RemainingBytes() || p.trans.RemainingBytes() == UnknownRemaining {
+	remainingBytes := remainingBytes(p.reader)
+	if uint64(size32) > remainingBytes || remainingBytes == UnknownRemaining {
 		err = invalidDataLength
 		return
 	}
@@ -379,11 +406,11 @@ func (p *BinaryProtocol) ReadSetBegin() (elemType Type, size int, err error) {
 	return elemType, size, nil
 }
 
-func (p *BinaryProtocol) ReadSetEnd() error {
+func (p *binaryDecoder) ReadSetEnd() error {
 	return nil
 }
 
-func (p *BinaryProtocol) ReadBool() (bool, error) {
+func (p *binaryDecoder) ReadBool() (bool, error) {
 	b, e := p.ReadByte()
 	v := true
 	if b != 1 {
@@ -392,47 +419,46 @@ func (p *BinaryProtocol) ReadBool() (bool, error) {
 	return v, e
 }
 
-func (p *BinaryProtocol) ReadByte() (byte, error) {
-	v, err := p.trans.ReadByte()
-	return byte(v), err
+func (p *binaryDecoder) ReadByte() (byte, error) {
+	return readByte(p.reader)
 }
 
-func (p *BinaryProtocol) ReadI16() (value int16, err error) {
+func (p *binaryDecoder) ReadI16() (value int16, err error) {
 	buf := p.buffer[0:2]
 	err = p.readAll(buf)
 	value = int16(binary.BigEndian.Uint16(buf))
 	return value, err
 }
 
-func (p *BinaryProtocol) ReadI32() (value int32, err error) {
+func (p *binaryDecoder) ReadI32() (value int32, err error) {
 	buf := p.buffer[0:4]
 	err = p.readAll(buf)
 	value = int32(binary.BigEndian.Uint32(buf))
 	return value, err
 }
 
-func (p *BinaryProtocol) ReadI64() (value int64, err error) {
+func (p *binaryDecoder) ReadI64() (value int64, err error) {
 	buf := p.buffer[0:8]
 	err = p.readAll(buf)
 	value = int64(binary.BigEndian.Uint64(buf))
 	return value, err
 }
 
-func (p *BinaryProtocol) ReadDouble() (value float64, err error) {
+func (p *binaryDecoder) ReadDouble() (value float64, err error) {
 	buf := p.buffer[0:8]
 	err = p.readAll(buf)
 	value = math.Float64frombits(binary.BigEndian.Uint64(buf))
 	return value, err
 }
 
-func (p *BinaryProtocol) ReadFloat() (value float32, err error) {
+func (p *binaryDecoder) ReadFloat() (value float32, err error) {
 	buf := p.buffer[0:4]
 	err = p.readAll(buf)
 	value = math.Float32frombits(binary.BigEndian.Uint32(buf))
 	return value, err
 }
 
-func (p *BinaryProtocol) ReadString() (value string, err error) {
+func (p *binaryDecoder) ReadString() (value string, err error) {
 	size, e := p.ReadI32()
 	if e != nil {
 		return "", e
@@ -445,7 +471,7 @@ func (p *BinaryProtocol) ReadString() (value string, err error) {
 	return p.readStringBody(size)
 }
 
-func (p *BinaryProtocol) ReadBinary() ([]byte, error) {
+func (p *binaryDecoder) ReadBinary() ([]byte, error) {
 	size, e := p.ReadI32()
 	if e != nil {
 		return nil, e
@@ -453,38 +479,32 @@ func (p *BinaryProtocol) ReadBinary() ([]byte, error) {
 	if size < 0 {
 		return nil, invalidDataLength
 	}
-	if uint64(size) > p.trans.RemainingBytes() || p.trans.RemainingBytes() == UnknownRemaining {
+	remainingBytes := remainingBytes(p.reader)
+	if uint64(size) > remainingBytes || remainingBytes == UnknownRemaining {
 		return nil, invalidDataLength
 	}
 
 	isize := int(size)
 	buf := make([]byte, isize)
-	_, err := io.ReadFull(p.trans, buf)
+	_, err := io.ReadFull(p.reader, buf)
 	return buf, NewProtocolException(err)
 }
 
-func (p *BinaryProtocol) Flush() (err error) {
-	return NewProtocolException(p.trans.Flush())
-}
-
-func (p *BinaryProtocol) Skip(fieldType Type) (err error) {
+func (p *binaryDecoder) Skip(fieldType Type) (err error) {
 	return SkipDefaultDepth(p, fieldType)
 }
 
-func (p *BinaryProtocol) Close() error {
-	return p.origTransport.Close()
-}
-
-func (p *BinaryProtocol) readAll(buf []byte) error {
+func (p *binaryDecoder) readAll(buf []byte) error {
 	_, err := io.ReadFull(p.reader, buf)
 	return NewProtocolException(err)
 }
 
-func (p *BinaryProtocol) readStringBody(size int32) (value string, err error) {
+func (p *binaryDecoder) readStringBody(size int32) (value string, err error) {
 	if size < 0 {
 		return "", nil
 	}
-	if uint64(size) > p.trans.RemainingBytes() || p.trans.RemainingBytes() == UnknownRemaining {
+	remainingBytes := remainingBytes(p.reader)
+	if uint64(size) > remainingBytes || remainingBytes == UnknownRemaining {
 		return "", invalidDataLength
 	}
 	var buf []byte
@@ -493,6 +513,6 @@ func (p *BinaryProtocol) readStringBody(size int32) (value string, err error) {
 	} else {
 		buf = make([]byte, size)
 	}
-	_, e := io.ReadFull(p.trans, buf)
+	_, e := io.ReadFull(p.reader, buf)
 	return string(buf), NewProtocolException(e)
 }
