@@ -18,6 +18,8 @@ package thrift
 
 import (
 	"fmt"
+	"net"
+	"time"
 )
 
 type headerProtocol struct {
@@ -27,25 +29,31 @@ type headerProtocol struct {
 	protoID ProtocolID
 }
 
-func NewHeaderProtocol(trans Transport) Protocol {
-	p := &headerProtocol{
-		protoID: ProtocolIDCompact,
-	}
-	p.trans = newHeaderTransport(trans)
-
-	// Effectively an invariant violation.
-	if err := p.ResetProtocol(); err != nil {
-		panic(err)
-	}
-	return p
+// NewHeaderProtocol creates a new header protocol.
+func NewHeaderProtocol(conn net.Conn) (Protocol, error) {
+	return newHeaderProtocol(conn, ProtocolIDCompact, 0, nil)
 }
 
-func (p *headerProtocol) ResetProtocol() error {
-	if p.Format != nil && p.protoID == p.trans.ProtocolID() {
+func newHeaderProtocol(conn net.Conn, protoID ProtocolID, timeout time.Duration, persistentHeaders map[string]string) (Protocol, error) {
+	p := &headerProtocol{protoID: protoID}
+	p.trans = newHeaderTransport(conn, protoID)
+	p.trans.conn.readTimeout = timeout
+	p.trans.conn.writeTimeout = timeout
+	if err := p.resetProtocol(); err != nil {
+		return nil, err
+	}
+	for name, value := range persistentHeaders {
+		p.SetPersistentHeader(name, value)
+	}
+	return p, nil
+}
+
+func (p *headerProtocol) resetProtocol() error {
+	if p.Format != nil && p.protoID == p.trans.protoID {
 		return nil
 	}
 
-	p.protoID = p.trans.ProtocolID()
+	p.protoID = p.trans.protoID
 	switch p.protoID {
 	case ProtocolIDBinary:
 		// These defaults match cpp implementation
@@ -63,8 +71,9 @@ func (p *headerProtocol) ResetProtocol() error {
 //
 
 func (p *headerProtocol) WriteMessageBegin(name string, typeId MessageType, seqid int32) error {
-	p.ResetProtocol()
-
+	if err := p.resetProtocol(); err != nil {
+		return err
+	}
 	// The conditions here only match on the Go client side.
 	// If we are a client, set header seq id same as msg id
 	if typeId == CALL || typeId == ONEWAY {
@@ -83,12 +92,10 @@ func (p *headerProtocol) ReadMessageBegin() (name string, typeId MessageType, se
 			return name, EXCEPTION, seqid, err
 		}
 	}
-
-	err = p.ResetProtocol()
+	err = p.resetProtocol()
 	if err != nil {
 		return name, EXCEPTION, seqid, err
 	}
-
 	// see https://github.com/apache/thrift/blob/master/doc/specs/SequenceNumbers.md
 	// TODO:  This is a bug. if we are speaking header protocol, we should be using
 	// seq id from the header. However, doing it here creates a non-backwards
@@ -120,18 +127,6 @@ func (p *headerProtocol) GetSeqID() uint32 {
 
 // Control underlying header transport
 
-func (p *headerProtocol) SetIdentity(identity string) {
-	p.trans.SetIdentity(identity)
-}
-
-func (p *headerProtocol) Identity() string {
-	return p.trans.Identity()
-}
-
-func (p *headerProtocol) peerIdentity() string {
-	return p.trans.peerIdentity()
-}
-
 func (p *headerProtocol) SetPersistentHeader(key, value string) {
 	p.trans.SetPersistentHeader(key, value)
 }
@@ -140,46 +135,13 @@ func (p *headerProtocol) GetPersistentHeader(key string) (string, bool) {
 	return p.trans.GetPersistentHeader(key)
 }
 
-func (p *headerProtocol) GetPersistentHeaders() map[string]string {
-	return p.trans.GetPersistentHeaders()
-}
-
-func (p *headerProtocol) ClearPersistentHeaders() {
-	p.trans.ClearPersistentHeaders()
-}
-
-// GetRequestHeader returns a request header if the key exists, otherwise false
-func (p *headerProtocol) GetRequestHeader(key string) (string, bool) {
-	return p.trans.GetRequestHeader(key)
-}
-
-// Deprecated SetHeader is deprecated, rather use SetRequestHeader
-func (p *headerProtocol) SetHeader(key, value string) {
-	p.trans.SetRequestHeader(key, value)
-}
-
-// Deprecated Header is deprecated, rather use GetRequestHeader
-func (p *headerProtocol) Header(key string) (string, bool) {
-	return p.trans.GetRequestHeader(key)
-}
-
-// Deprecated Headers is deprecated, rather use GetRequestHeaders
-func (p *headerProtocol) Headers() map[string]string {
-	return p.trans.GetRequestHeaders()
-}
-
 // Deprecated: SetRequestHeader is deprecated and will eventually be private.
 func (p *headerProtocol) SetRequestHeader(key, value string) {
 	p.trans.SetRequestHeader(key, value)
 }
 
-// Deprecated: GetRequestHeader is deprecated and will eventually be private.
 func (p *headerProtocol) GetRequestHeaders() map[string]string {
-	return p.trans.GetRequestHeaders()
-}
-
-func (p *headerProtocol) GetResponseHeader(key string) (string, bool) {
-	return p.trans.GetResponseHeader(key)
+	return p.trans.getRequestHeaders()
 }
 
 func (p *headerProtocol) GetResponseHeaders() map[string]string {
@@ -190,13 +152,6 @@ func (p *headerProtocol) ProtocolID() ProtocolID {
 	return p.protoID
 }
 
-func (p *headerProtocol) SetProtocolID(protoID ProtocolID) error {
-	if err := p.trans.SetProtocolID(protoID); err != nil {
-		return err
-	}
-	return p.ResetProtocol()
-}
-
 // Deprecated: GetFlags() is a deprecated method.
 func (t *headerProtocol) GetFlags() HeaderFlags {
 	return t.trans.GetFlags()
@@ -211,15 +166,6 @@ func (p *headerProtocol) AddTransform(trans TransformID) error {
 	return p.trans.AddTransform(trans)
 }
 
-// Deprecated: HeaderProtocolIdentity is a deprecated type, temporarily introduced to ease transition to new API.
-type HeaderProtocolIdentity interface {
-	SetIdentity(identity string)
-	Identity() string
-}
-
-// Compile time interface enforcer
-var _ HeaderProtocolIdentity = (*headerProtocol)(nil)
-
 // Deprecated: HeaderProtocolSeqID is a deprecated type, temporarily introduced to ease transition to new API.
 type HeaderProtocolSeqID interface {
 	GetSeqID() uint32
@@ -237,12 +183,3 @@ type HeaderProtocolFlags interface {
 
 // Compile time interface enforcer
 var _ HeaderProtocolFlags = (*headerProtocol)(nil)
-
-// Deprecated: HeaderProtocolProtocolID is a deprecated type, temporarily introduced to ease transition to new API.
-type HeaderProtocolProtocolID interface {
-	ProtocolID() ProtocolID
-	SetProtocolID(protoID ProtocolID) error
-}
-
-// Compile time interface enforcer
-var _ HeaderProtocolProtocolID = (*headerProtocol)(nil)
