@@ -23,6 +23,7 @@ import (
 	"encoding/binary"
 	"fmt"
 	"io"
+	"net"
 )
 
 const (
@@ -31,7 +32,7 @@ const (
 )
 
 type headerTransport struct {
-	transport Transport
+	conn *connTimeout
 
 	// Used on read
 	rbuf       *bufio.Reader
@@ -42,7 +43,6 @@ type headerTransport struct {
 
 	// Used on write
 	wbuf                       *bytes.Buffer
-	identity                   string
 	writeInfoHeaders           map[string]string
 	persistentWriteInfoHeaders map[string]string
 
@@ -58,10 +58,11 @@ type headerTransport struct {
 }
 
 // newHeaderTransport creates a new transport with defaults.
-func newHeaderTransport(transport Transport) *headerTransport {
+func newHeaderTransport(c net.Conn, protoID ProtocolID) *headerTransport {
+	conn := &connTimeout{Conn: c}
 	return &headerTransport{
-		transport: transport,
-		rbuf:      bufio.NewReader(transport),
+		conn:      conn,
+		rbuf:      bufio.NewReader(conn),
 		framebuf:  newLimitedByteReader(bytes.NewReader(nil), 0),
 		frameSize: 0,
 
@@ -69,7 +70,7 @@ func newHeaderTransport(transport Transport) *headerTransport {
 		writeInfoHeaders:           map[string]string{},
 		persistentWriteInfoHeaders: map[string]string{},
 
-		protoID:         DefaulprotoID,
+		protoID:         protoID,
 		flags:           0,
 		clientType:      DefaultClientType,
 		writeTransforms: []TransformID{},
@@ -86,23 +87,6 @@ func (t *headerTransport) SeqID() uint32 {
 	return t.readSeqID
 }
 
-func (t *headerTransport) Identity() string {
-	return t.identity
-}
-
-func (t *headerTransport) SetIdentity(identity string) {
-	t.identity = identity
-}
-
-func (t *headerTransport) peerIdentity() string {
-	v, ok := t.GetResponseHeader(IdentityHeader)
-	vers, versok := t.GetResponseHeader(IDVersionHeader)
-	if ok && versok && vers == IDVersion {
-		return v
-	}
-	return ""
-}
-
 func (t *headerTransport) SetPersistentHeader(key, value string) {
 	t.persistentWriteInfoHeaders[key] = value
 }
@@ -112,20 +96,6 @@ func (t *headerTransport) GetPersistentHeader(key string) (string, bool) {
 	return v, ok
 }
 
-func (t *headerTransport) GetPersistentHeaders() map[string]string {
-	res := map[string]string{}
-	for k, v := range t.persistentWriteInfoHeaders {
-		res[k] = v
-	}
-	return res
-}
-
-func (t *headerTransport) ClearPersistentHeaders() {
-	if len(t.persistentWriteInfoHeaders) != 0 {
-		t.persistentWriteInfoHeaders = map[string]string{}
-	}
-}
-
 // SetRequestHeader sets a request header
 func (t *headerTransport) SetRequestHeader(key, value string) {
 	t.writeInfoHeaders[key] = value
@@ -136,29 +106,13 @@ func (t *headerTransport) SetHeader(key, value string) {
 	t.writeInfoHeaders[key] = value
 }
 
-// GetRequestHeader returns a request header if the key exists, otherwise false
-func (t *headerTransport) GetRequestHeader(key string) (string, bool) {
-	v, ok := t.writeInfoHeaders[key]
-	return v, ok
-}
-
 // Deprecated Header is deprecated rather use GetRequestHeader
 func (t *headerTransport) Header(key string) (string, bool) {
 	v, ok := t.writeInfoHeaders[key]
 	return v, ok
 }
 
-// GetRequestHeaders returns all the request headers
-func (t *headerTransport) GetRequestHeaders() map[string]string {
-	res := map[string]string{}
-	for k, v := range t.writeInfoHeaders {
-		res[k] = v
-	}
-	return res
-}
-
-// Deprecated Headers is deprecated rather use GetRequestHeaders
-func (t *headerTransport) Headers() map[string]string {
+func (t *headerTransport) getRequestHeaders() map[string]string {
 	res := map[string]string{}
 	for k, v := range t.writeInfoHeaders {
 		res[k] = v
@@ -172,18 +126,6 @@ func (t *headerTransport) clearRequestHeaders() {
 	}
 }
 
-func (t *headerTransport) GetResponseHeader(key string) (string, bool) {
-	if t.readHeader == nil {
-		return "", false
-	}
-	// per the C++ implementation, prefer persistent headers
-	if v, ok := t.readHeader.pHeaders[key]; ok {
-		return v, ok
-	}
-	v, ok := t.readHeader.headers[key]
-	return v, ok
-}
-
 func (t *headerTransport) GetResponseHeaders() map[string]string {
 	res := map[string]string{}
 	if t.readHeader == nil {
@@ -198,20 +140,6 @@ func (t *headerTransport) GetResponseHeaders() map[string]string {
 	return res
 }
 
-func (t *headerTransport) ProtocolID() ProtocolID {
-	return t.protoID
-}
-
-func (t *headerTransport) SetProtocolID(protoID ProtocolID) error {
-	if !(protoID == ProtocolIDBinary || protoID == ProtocolIDCompact) {
-		return NewTransportException(
-			NOT_IMPLEMENTED, fmt.Sprintf("unimplemented proto ID: %s (%#x)", protoID.String(), int64(protoID)),
-		)
-	}
-	t.protoID = protoID
-	return nil
-}
-
 func (t *headerTransport) AddTransform(trans TransformID) error {
 	if sup, ok := supportedTransforms[trans]; !ok || !sup {
 		return NewTransportException(
@@ -305,7 +233,7 @@ func (t *headerTransport) ResetProtocol() error {
 
 // Close closes the internal transport
 func (t *headerTransport) Close() error {
-	return t.transport.Close()
+	return t.conn.Close()
 }
 
 // Read reads from the current framebuffer. EOF if the frame is done.
@@ -411,11 +339,6 @@ func (t *headerTransport) flushHeader() error {
 	hdr.clientType = t.clientType
 	hdr.flags = t.flags
 
-	if t.identity != "" {
-		hdr.headers[IdentityHeader] = t.identity
-		hdr.headers[IDVersionHeader] = IDVersion
-	}
-
 	outbuf, err := applyTransforms(t.wbuf, t.writeTransforms)
 	if err != nil {
 		return NewTransportExceptionFromError(err)
@@ -428,7 +351,7 @@ func (t *headerTransport) flushHeader() error {
 		return NewTransportExceptionFromError(err)
 	}
 
-	err = hdr.Write(t.transport)
+	err = hdr.Write(t.conn)
 	return NewTransportExceptionFromError(err)
 }
 
@@ -442,7 +365,7 @@ func (t *headerTransport) flushFramed() error {
 		)
 	}
 
-	err := binary.Write(t.transport, binary.BigEndian, framesize)
+	err := binary.Write(t.conn, binary.BigEndian, framesize)
 	return NewTransportExceptionFromError(err)
 }
 
@@ -471,7 +394,7 @@ func (t *headerTransport) Flush() error {
 
 	// Writeout the payload
 	if t.wbuf.Len() > 0 {
-		_, err = t.wbuf.WriteTo(t.transport)
+		_, err = t.wbuf.WriteTo(t.conn)
 		if err != nil {
 			t.wbuf.Reset() // reset on return
 			return NewTransportExceptionFromError(err)
@@ -481,8 +404,6 @@ func (t *headerTransport) Flush() error {
 	// Remove the non-persistent headers on flush
 	t.clearRequestHeaders()
 
-	err = t.transport.Flush()
-
 	t.wbuf.Reset() // reset incase wbuf pointer changes in xform
 	return NewTransportExceptionFromError(err)
 }
