@@ -24,7 +24,7 @@ import (
 )
 
 func TestHeaderTransport(t *testing.T) {
-	trans := newHeaderTransport(newMockSocket())
+	trans := newHeaderTransport(newMockSocket(), ProtocolIDCompact)
 	TransportTest(t, trans, trans)
 }
 
@@ -103,8 +103,8 @@ func TestHeaderFramedBinary(t *testing.T) {
 	tmb := newMockSocket()
 	testHeaderToProto(
 		t, FramedDeprecated, tmb,
-		NewBinaryProtocol(NewFramedTransport(tmb), true, true),
-		newHeaderTransport(tmb),
+		NewBinaryProtocol(newFramedTransport(tmb), true, true),
+		newHeaderTransport(tmb, ProtocolIDCompact),
 	)
 }
 
@@ -112,70 +112,36 @@ func TestHeaderFramedCompact(t *testing.T) {
 	tmb := newMockSocket()
 	testHeaderToProto(
 		t, FramedCompact, tmb,
-		NewCompactProtocol(NewFramedTransport(tmb)),
-		newHeaderTransport(tmb),
+		NewCompactProtocol(newFramedTransport(tmb)),
+		newHeaderTransport(tmb, ProtocolIDCompact),
 	)
 }
 
-func TestHeaderProtoID(t *testing.T) {
-	n := 1
-	tmb := newMockSocket()
-	// write transport
-	trans1 := newHeaderTransport(tmb)
-	// read transport
-	trans2 := newHeaderTransport(tmb)
-	targetID := ProtocolIDBinary
-
-	assertEq(t, DefaulprotoID, trans1.ProtocolID())
-
-	err := trans1.SetProtocolID(targetID)
-	if err != nil {
-		t.Fatalf("failed to set binary protocol")
-	}
-
-	assertEq(t, targetID, trans1.ProtocolID())
-
-	_, err = trans1.Write([]byte("ASDF"))
-	if err != nil {
-		t.Fatalf("failed to write frame %d: %s", n, err)
-	}
-	err = trans1.Flush()
-	if err != nil {
-		t.Fatalf("failed to xmit frame %d: %s", n, err)
-	}
-
-	assertEq(t, DefaulprotoID, trans2.ProtocolID())
-	err = trans2.ResetProtocol()
-	if err != nil {
-		t.Fatalf("failed to reset proto for frame %d: %s", n, err)
-	}
-	// Make sure the protocol gets changed after recving the frame
-	assertEq(t, targetID, trans1.ProtocolID())
-}
-
 func TestHeaderHeaders(t *testing.T) {
 	n := 1
 	tmb := newMockSocket()
 	// write transport
-	trans1 := newHeaderTransport(tmb)
+	trans1 := newHeaderTransport(tmb, ProtocolIDCompact)
 	// read transport
-	trans2 := newHeaderTransport(tmb)
+	trans2 := newHeaderTransport(tmb, ProtocolIDCompact)
 
 	// make sure we don't barf reading header with no frame
-	_, ok := trans1.GetResponseHeader("something")
+	_, ok := trans1.GetResponseHeaders()["something"]
 	assertEq(t, false, ok)
 	assertEq(t, 0, len(trans1.GetResponseHeaders()))
 
-	trans1.SetIdentity("localhost")
+	trans1.SetPersistentHeader(IDVersionHeader, IDVersion)
+	trans1.SetPersistentHeader(IdentityHeader, "localhost")
 	trans1.SetRequestHeader("thrift_protocol", "compact")
 	trans1.SetRequestHeader("thrift_transport", "header")
 	trans1.SetRequestHeader("preferred_cheese", "cheddar")
 	trans1.SetPersistentHeader("preferred_cheese", "gouda")
 
-	assertEq(t, 3, len(trans1.Headers()))
-	assertEq(t, 1, len(trans1.GetPersistentHeaders()))
+	assertEq(t, 3, len(trans1.getRequestHeaders()))
+	// 1 for persistent header and 2 more for identity headers
+	assertEq(t, 3, len(trans1.persistentWriteInfoHeaders))
 
-	headerval, _ := trans1.GetRequestHeader("preferred_cheese")
+	headerval, _ := trans1.getRequestHeaders()["preferred_cheese"]
 	assertEq(t, "cheddar", headerval)
 	headerval, _ = trans1.GetPersistentHeader("preferred_cheese")
 	assertEq(t, "gouda", headerval)
@@ -190,36 +156,42 @@ func TestHeaderHeaders(t *testing.T) {
 	}
 
 	// Make sure we zero the headers
-	assertEq(t, 0, len(trans1.Headers()))
+	assertEq(t, 0, len(trans1.getRequestHeaders()))
 	// But not the persistent ones
-	assertEq(t, 1, len(trans1.GetPersistentHeaders()))
+	assertEq(t, 3, len(trans1.persistentWriteInfoHeaders))
 
 	err = trans2.ResetProtocol()
 	if err != nil {
 		t.Fatalf("failed to reset proto for frame %d: %s", n, err)
 	}
 
-	headerval, _ = trans2.GetResponseHeader("thrift_protocol")
+	headerval, _ = trans2.GetResponseHeaders()["thrift_protocol"]
 	assertEq(t, "compact", headerval)
-	headerval, _ = trans2.GetResponseHeader("thrift_transport")
+	headerval, _ = trans2.GetResponseHeaders()["thrift_transport"]
 	assertEq(t, "header", headerval)
 	// make sure we prefer persistent headers
-	headerval, _ = trans2.GetResponseHeader("preferred_cheese")
+	headerval, _ = trans2.GetResponseHeaders()["preferred_cheese"]
 	assertEq(t, "gouda", headerval)
-	assertEq(t, "localhost", trans2.peerIdentity())
+	assertEq(t, "localhost", peerIdentity(trans2))
 	assertEq(t, 5, len(trans2.GetResponseHeaders()))
 
-	trans2.readHeader.headers[IDVersionHeader] = "invalid"
-	assertEq(t, "", trans2.peerIdentity())
+	trans2.readHeader.pHeaders[IDVersionHeader] = "invalid"
+	assertEq(t, "", peerIdentity(trans2))
+}
 
-	trans1.ClearPersistentHeaders()
-	assertEq(t, 0, len(trans1.GetPersistentHeaders()))
+func peerIdentity(t *headerTransport) string {
+	v, ok := t.GetResponseHeaders()[IdentityHeader]
+	vers, versok := t.GetResponseHeaders()[IDVersionHeader]
+	if ok && versok && vers == IDVersion {
+		return v
+	}
+	return ""
 }
 
 func TestHeaderRWSmall(t *testing.T) {
 	n := 1
 	tmb := newMockSocket()
-	trans := newHeaderTransport(tmb)
+	trans := newHeaderTransport(tmb, ProtocolIDCompact)
 	data := []byte("ASDFASDFASDF")
 
 	_, err := trans.Write(data)
@@ -282,7 +254,7 @@ func TestHeaderRWSmall(t *testing.T) {
 func TestHeaderZlib(t *testing.T) {
 	n := 1
 	tmb := newMockSocket()
-	trans := newHeaderTransport(tmb)
+	trans := newHeaderTransport(tmb, ProtocolIDCompact)
 	data := []byte("ASDFASDFASDFASDFASDFASDFASDFASDFASDFASDFASDFASDFASDFASDFASDF")
 	uncompressedlen := 30
 
@@ -353,7 +325,7 @@ func testRWOnce(t *testing.T, n int, data []byte, trans *headerTransport) {
 
 func TestHeaderTransportRWMultiple(t *testing.T) {
 	tmb := newMockSocket()
-	trans := newHeaderTransport(tmb)
+	trans := newHeaderTransport(tmb, ProtocolIDCompact)
 
 	// Test Junk Data
 	testRWOnce(t, 1, []byte("ASDF"), trans)
@@ -367,9 +339,10 @@ func BenchmarkHeaderFlush(b *testing.B) {
 
 	for n := 0; n < b.N; n++ {
 		tmb := newMockSocket()
-		trans1 := newHeaderTransport(tmb)
+		trans1 := newHeaderTransport(tmb, ProtocolIDCompact)
 
-		trans1.SetIdentity("localhost")
+		trans1.SetPersistentHeader(IDVersionHeader, IDVersion)
+		trans1.SetPersistentHeader(IdentityHeader, "localhost")
 		trans1.SetRequestHeader("thrift_protocol", "compact")
 		trans1.SetRequestHeader("thrift_transport", "header")
 		trans1.SetRequestHeader("preferred_cheese", "cheddar")
