@@ -23,24 +23,24 @@ import (
 )
 
 func TestHeaderProtocolSomePersistentHeaders(t *testing.T) {
-	protocol := NewHeaderProtocol(newMockSocket())
-	protocol.SetPersistentHeader("key", "value")
+	protocol, err := newHeaderProtocol(newMockSocket(), ProtocolIDCompact, 0, map[string]string{"key": "value"})
+	assert.NoError(t, err)
 	v, ok := protocol.GetPersistentHeader("key")
 	assert.True(t, ok)
 	assert.Equal(t, "value", v)
 }
 
 func TestRocketProtocolSomePersistentHeaders(t *testing.T) {
-	protocol := NewRocketProtocol(newMockSocket())
-	protocol.SetPersistentHeader("key", "value")
+	protocol, err := newRocketClient(newMockSocket(), ProtocolIDCompact, 0, map[string]string{"key": "value"})
+	assert.NoError(t, err)
 	v, ok := protocol.GetPersistentHeader("key")
 	assert.True(t, ok)
 	assert.Equal(t, "value", v)
 }
 
 func TestUpgradeToRocketProtocolSomePersistentHeaders(t *testing.T) {
-	protocol := NewUpgradeToRocketProtocol(newMockSocket())
-	protocol.SetPersistentHeader("key", "value")
+	protocol, err := newUpgradeToRocketClient(newMockSocket(), ProtocolIDCompact, 0, map[string]string{"key": "value"})
+	assert.NoError(t, err)
 	v, ok := protocol.GetPersistentHeader("key")
 	assert.True(t, ok)
 	assert.Equal(t, "value", v)
