@@ -22,80 +22,32 @@ import (
 	"strings"
 )
 
-// Processor exposes access to processor functions which
+// ProcessorContext exposes access to processor functions which
 // manage I/O and processing of a input message for a specific
 // server function
-type Processor interface {
-	// GetProcessorFunction is given the name of a thrift function and
+type ProcessorContext interface {
+	// GetProcessorFunctionContext is given the name of a thrift function and
 	// the type of the inbound thrift message.  It is expected to return
-	// a non-nil ProcessorFunction when the function can be successfully
+	// a non-nil GetProcessorFunctionContext when the function can be successfully
 	// found.
 	//
-	// If an error is returned, it will be wrapped in an application level
-	// thrift exception and returned.
-	//
-	// If ProcessorFunction and error are both nil, a generic error will be
+	// If ProcessorFunctionContext is nil, a generic error will be
 	// sent which explains that no processor function exists with the specified
 	// name on this server.
-	GetProcessorFunction(name string) (ProcessorFunction, error)
+	GetProcessorFunctionContext(name string) ProcessorFunctionContext
 }
 
-// ProcessorFunction is the interface that must be implemented in
+// ProcessorFunctionContext is the interface that must be implemented in
 // order to perform io and message processing
-type ProcessorFunction interface {
+type ProcessorFunctionContext interface {
 	// Read a serializable message from the protocol.
 	Read(prot Format) (Struct, Exception)
-	// Process a message handing it to the client handler.
-	Run(args Struct) (WritableStruct, ApplicationException)
-	// Write a serializable responsne
-	Write(seqID int32, result WritableStruct, prot Format) Exception
-}
-
-// ProcessorContext is a Processor that supports contexts.
-type ProcessorContext interface {
-	GetProcessorFunctionContext(name string) (ProcessorFunctionContext, error)
-}
-
-// NewProcessorContextAdapter creates a ProcessorContext from a regular Processor.
-func NewProcessorContextAdapter(p Processor) ProcessorContext {
-	return &ctxProcessorAdapter{p}
-}
-
-type ctxProcessorAdapter struct {
-	Processor
-}
-
-func (p ctxProcessorAdapter) GetProcessorFunctionContext(name string) (ProcessorFunctionContext, error) {
-	f, err := p.Processor.GetProcessorFunction(name)
-	if err != nil {
-		return nil, err
-	}
-	if f == nil {
-		return nil, nil
-	}
-	return NewProcessorFunctionContextAdapter(f), nil
-}
-
-// ProcessorFunctionContext is a ProcessorFunction that supports contexts.
-type ProcessorFunctionContext interface {
-	Read(prot Format) (Struct, Exception)
+	// RunContext processes a message handing it to the client handler.
 	RunContext(ctx context.Context, args Struct) (WritableStruct, ApplicationException)
+	// Write a serializable response
 	Write(seqID int32, result WritableStruct, prot Format) Exception
 }
 
-// NewProcessorFunctionContextAdapter creates a ProcessorFunctionContext from a regular ProcessorFunction.
-func NewProcessorFunctionContextAdapter(p ProcessorFunction) ProcessorFunctionContext {
-	return &ctxProcessorFunctionAdapter{p}
-}
-
-type ctxProcessorFunctionAdapter struct {
-	ProcessorFunction
-}
-
-func (p ctxProcessorFunctionAdapter) RunContext(ctx context.Context, args Struct) (WritableStruct, ApplicationException) {
-	return p.ProcessorFunction.Run(args)
-}
-
 func errorType(err error) string {
 	// get type name without package or pointer information
 	fqet := strings.Replace(fmt.Sprintf("%T", err), "*", "", -1)
@@ -107,15 +59,12 @@ func errorType(err error) string {
 // and fully process a message. It understands the thrift protocol.
 // A framework could be written outside of the thrift library but would need to
 // duplicate this logic.
-func processContext(ctx context.Context, processor ProcessorContext, prot Protocol) (keepOpen bool, ext Exception) {
+func processContext(ctx context.Context, processor ProcessorContext, prot Protocol) (ext Exception) {
 	name, messageType, seqID, rerr := prot.ReadMessageBegin()
 	if rerr != nil {
-		if err, ok := rerr.(TransportException); ok && err.TypeID() == END_OF_FILE {
-			// connection terminated because client closed connection
-			return false, nil
-		}
-		return false, rerr
+		return rerr
 	}
+	ctx = WithHeaders(ctx, prot.GetResponseHeaders())
 	var err ApplicationException
 	var pfunc ProcessorFunctionContext
 	if messageType != CALL && messageType != ONEWAY {
@@ -124,13 +73,9 @@ func processContext(ctx context.Context, processor ProcessorContext, prot Protoc
 		// error should be sent, connection should stay open if successful
 	}
 	if err == nil {
-		pf, e2 := processor.GetProcessorFunctionContext(name)
+		pf := processor.GetProcessorFunctionContext(name)
 		if pf == nil {
-			if e2 == nil {
-				err = NewApplicationException(UNKNOWN_METHOD, fmt.Sprintf("no such function: %q", name))
-			} else {
-				err = NewApplicationException(UNKNOWN_METHOD, e2.Error())
-			}
+			err = NewApplicationException(UNKNOWN_METHOD, fmt.Sprintf("no such function: %q", name))
 		} else {
 			pfunc = pf
 		}
@@ -140,17 +85,17 @@ func processContext(ctx context.Context, processor ProcessorContext, prot Protoc
 	// rest of the invalid message but keep the connection open.
 	if err != nil {
 		if e2 := prot.Skip(STRUCT); e2 != nil {
-			return false, e2
+			return e2
 		} else if e2 := prot.ReadMessageEnd(); e2 != nil {
-			return false, e2
+			return e2
 		}
 		// for ONEWAY, we have no way to report that the processing failed.
 		if messageType != ONEWAY {
 			if e2 := sendException(prot, name, seqID, err); e2 != nil {
-				return false, e2
+				return e2
 			}
 		}
-		return true, nil
+		return nil
 	}
 
 	if pfunc == nil {
@@ -160,7 +105,7 @@ func processContext(ctx context.Context, processor ProcessorContext, prot Protoc
 	argStruct, e2 := pfunc.Read(prot)
 	if e2 != nil {
 		// close connection on read failure
-		return false, e2
+		return e2
 	}
 	var result WritableStruct
 	result, err = pfunc.RunContext(ctx, argStruct)
@@ -188,7 +133,7 @@ func processContext(ctx context.Context, processor ProcessorContext, prot Protoc
 		if result != nil {
 			if e2 := pfunc.Write(seqID, result, prot); e2 != nil {
 				// close connection on write failure
-				return false, err
+				return err
 			}
 		}
 	}
@@ -196,5 +141,5 @@ func processContext(ctx context.Context, processor ProcessorContext, prot Protoc
 	// keep the connection open and ignore errors
 	// if type was CALL, error has already been serialized to client
 	// if type was ONEWAY, no exception is to be thrown
-	return true, nil
+	return nil
 }
