@@ -164,7 +164,9 @@ func tcpStreamSetupForTest(t *testing.T) (io.Reader, io.Writer) {
 	return rConn, wConn
 }
 
-type protocolTest func(t testing.TB, p Format, trans Transport)
+type protocolTest func(t testing.TB, p Format, trans io.ReadWriter)
+type protocolReaderTest func(t testing.TB, p Format, trans io.Reader)
+type protocolWriterTest func(t testing.TB, p Format, trans io.Writer)
 
 // ReadWriteProtocolParallelTest tests that a given protocol is safe to read
 // from and write to in different goroutines. This requires both a protocol
@@ -173,21 +175,21 @@ type protocolTest func(t testing.TB, p Format, trans Transport)
 // It also should only be used with an underlying Transport that is capable of
 // blocking reads and writes (socket, stream), since other golang Transport
 // implementations require that the data exists to be read when they are called (like bytes.Buffer)
-func ReadWriteProtocolParallelTest(t *testing.T, newFormat func(Transport) Format) {
+func ReadWriteProtocolParallelTest(t *testing.T, newFormat func(io.ReadWriteCloser) Format) {
 	rConn, wConn := tcpStreamSetupForTest(t)
 	rdr, writer := io.Pipe()
 
-	transports := []func() Transport{
-		func() Transport {
-			return NewFramedTransportMaxLength(NewStreamTransport(rdr, writer), DEFAULT_MAX_LENGTH)
+	transports := []func() io.ReadWriteCloser{
+		func() io.ReadWriteCloser {
+			return newFramedTransportMaxLength(newStreamTransport(rdr, writer), DEFAULT_MAX_LENGTH)
 		}, // framed over pipe
-		func() Transport {
-			return NewFramedTransportMaxLength(NewStreamTransport(rConn, wConn), DEFAULT_MAX_LENGTH)
+		func() io.ReadWriteCloser {
+			return newFramedTransportMaxLength(newStreamTransport(rConn, wConn), DEFAULT_MAX_LENGTH)
 		}, // framed over tcp
 	}
 	const iterations = 100
 
-	doForAllTransportsParallel := func(read, write protocolTest) {
+	doForAllTransportsParallel := func(read protocolReaderTest, write protocolWriterTest) {
 		for _, tf := range transports {
 			trans := tf()
 			p := newFormat(trans)
@@ -219,7 +221,7 @@ func ReadWriteProtocolParallelTest(t *testing.T, newFormat func(Transport) Forma
 	doForAllTransportsParallel(ReadStruct, WriteStruct)
 
 	// perform set of many sequenced sets of reads and writes
-	doForAllTransportsParallel(func(t testing.TB, p Format, trans Transport) {
+	doForAllTransportsParallel(func(t testing.TB, p Format, trans io.Reader) {
 		ReadBool(t, p, trans)
 		ReadByte(t, p, trans)
 		ReadI16(t, p, trans)
@@ -230,7 +232,7 @@ func ReadWriteProtocolParallelTest(t *testing.T, newFormat func(Transport) Forma
 		ReadString(t, p, trans)
 		ReadBinary(t, p, trans)
 		ReadStruct(t, p, trans)
-	}, func(t testing.TB, p Format, trans Transport) {
+	}, func(t testing.TB, p Format, trans io.Writer) {
 		WriteBool(t, p, trans)
 		WriteByte(t, p, trans)
 		WriteI16(t, p, trans)
@@ -244,15 +246,17 @@ func ReadWriteProtocolParallelTest(t *testing.T, newFormat func(Transport) Forma
 	})
 }
 
-func ReadWriteProtocolTest(t *testing.T, newFormat func(Transport) Format) {
+func ReadWriteProtocolTest(t *testing.T, newFormat func(io.ReadWriteCloser) Format) {
 	l := HTTPClientSetupForTest(t)
 	defer l.Close()
 
-	transports := []func() Transport{
-		func() Transport { return NewMemoryBufferLen(1024) },
-		func() Transport { return NewFramedTransportMaxLength(NewMemoryBufferLen(1024), DEFAULT_MAX_LENGTH) },
-		func() Transport {
-			http, err := NewHTTPPostClient("http://" + l.Addr().String())
+	transports := []func() io.ReadWriteCloser{
+		func() io.ReadWriteCloser { return NewMemoryBufferLen(1024) },
+		func() io.ReadWriteCloser {
+			return newFramedTransportMaxLength(NewMemoryBufferLen(1024), DEFAULT_MAX_LENGTH)
+		},
+		func() io.ReadWriteCloser {
+			http, err := newHTTPPostClient("http://" + l.Addr().String())
 			if err != nil {
 				panic(err)
 			}
@@ -281,7 +285,7 @@ func ReadWriteProtocolTest(t *testing.T, newFormat func(Transport) Format) {
 	doForAllTransports(ReadWriteStruct)
 
 	// perform set of many sequenced reads and writes
-	doForAllTransports(func(t testing.TB, p Format, trans Transport) {
+	doForAllTransports(func(t testing.TB, p Format, trans io.ReadWriter) {
 		ReadWriteI64(t, p, trans)
 		ReadWriteDouble(t, p, trans)
 		ReadWriteFloat(t, p, trans)
@@ -291,14 +295,14 @@ func ReadWriteProtocolTest(t *testing.T, newFormat func(Transport) Format) {
 	})
 }
 
-func ReadBool(t testing.TB, p Format, trans Transport) {
+func ReadBool(t testing.TB, p Format, trans io.Reader) {
 	thetype := Type(BOOL)
 	thelen := len(boolValues)
 	thetype2, thelen2, err := p.ReadListBegin()
 	if err != nil {
 		t.Fatalf("%s: %T %T %q Error reading list: %v", "ReadBool", p, trans, err, boolValues)
 	}
-	_, ok := p.(*SimpleJSONProtocol)
+	_, ok := p.(*simpleJSONProtocol)
 	if !ok {
 		if thetype != thetype2 {
 			t.Fatalf("%s: %T %T type %s != type %s", "ReadBool", p, trans, thetype, thetype2)
@@ -322,7 +326,7 @@ func ReadBool(t testing.TB, p Format, trans Transport) {
 	}
 }
 
-func WriteBool(t testing.TB, p Format, trans Transport) {
+func WriteBool(t testing.TB, p Format, trans io.Writer) {
 	thetype := Type(BOOL)
 	thelen := len(boolValues)
 	err := p.WriteListBegin(thetype, thelen)
@@ -345,12 +349,12 @@ func WriteBool(t testing.TB, p Format, trans Transport) {
 	}
 }
 
-func ReadWriteBool(t testing.TB, p Format, trans Transport) {
+func ReadWriteBool(t testing.TB, p Format, trans io.ReadWriter) {
 	WriteBool(t, p, trans)
 	ReadBool(t, p, trans)
 }
 
-func WriteByte(t testing.TB, p Format, trans Transport) {
+func WriteByte(t testing.TB, p Format, trans io.Writer) {
 	thetype := Type(BYTE)
 	thelen := len(byteValues)
 	err := p.WriteListBegin(thetype, thelen)
@@ -373,14 +377,14 @@ func WriteByte(t testing.TB, p Format, trans Transport) {
 	}
 }
 
-func ReadByte(t testing.TB, p Format, trans Transport) {
+func ReadByte(t testing.TB, p Format, trans io.Reader) {
 	thetype := Type(BYTE)
 	thelen := len(byteValues)
 	thetype2, thelen2, err := p.ReadListBegin()
 	if err != nil {
 		t.Fatalf("%s: %T %T %q Error reading list: %q", "ReadByte", p, trans, err, byteValues)
 	}
-	_, ok := p.(*SimpleJSONProtocol)
+	_, ok := p.(*simpleJSONProtocol)
 	if !ok {
 		if thetype != thetype2 {
 			t.Fatalf("%s: %T %T type %s != type %s", "ReadByte", p, trans, thetype, thetype2)
@@ -404,12 +408,12 @@ func ReadByte(t testing.TB, p Format, trans Transport) {
 	}
 }
 
-func ReadWriteByte(t testing.TB, p Format, trans Transport) {
+func ReadWriteByte(t testing.TB, p Format, trans io.ReadWriter) {
 	WriteByte(t, p, trans)
 	ReadByte(t, p, trans)
 }
 
-func WriteI16(t testing.TB, p Format, trans Transport) {
+func WriteI16(t testing.TB, p Format, trans io.Writer) {
 	thetype := Type(I16)
 	thelen := len(int16Values)
 	p.WriteListBegin(thetype, thelen)
@@ -420,14 +424,14 @@ func WriteI16(t testing.TB, p Format, trans Transport) {
 	p.Flush()
 }
 
-func ReadI16(t testing.TB, p Format, trans Transport) {
+func ReadI16(t testing.TB, p Format, trans io.Reader) {
 	thetype := Type(I16)
 	thelen := len(int16Values)
 	thetype2, thelen2, err := p.ReadListBegin()
 	if err != nil {
 		t.Fatalf("%s: %T %T %q Error reading list: %q", "ReadI16", p, trans, err, int16Values)
 	}
-	_, ok := p.(*SimpleJSONProtocol)
+	_, ok := p.(*simpleJSONProtocol)
 	if !ok {
 		if thetype != thetype2 {
 			t.Fatalf("%s: %T %T type %s != type %s", "ReadI16", p, trans, thetype, thetype2)
@@ -451,12 +455,12 @@ func ReadI16(t testing.TB, p Format, trans Transport) {
 	}
 }
 
-func ReadWriteI16(t testing.TB, p Format, trans Transport) {
+func ReadWriteI16(t testing.TB, p Format, trans io.ReadWriter) {
 	WriteI16(t, p, trans)
 	ReadI16(t, p, trans)
 }
 
-func WriteI32(t testing.TB, p Format, trans Transport) {
+func WriteI32(t testing.TB, p Format, trans io.Writer) {
 	thetype := Type(I32)
 	thelen := len(int32Values)
 	p.WriteListBegin(thetype, thelen)
@@ -467,14 +471,14 @@ func WriteI32(t testing.TB, p Format, trans Transport) {
 	p.Flush()
 }
 
-func ReadI32(t testing.TB, p Format, trans Transport) {
+func ReadI32(t testing.TB, p Format, trans io.Reader) {
 	thetype := Type(I32)
 	thelen := len(int32Values)
 	thetype2, thelen2, err := p.ReadListBegin()
 	if err != nil {
 		t.Fatalf("%s: %T %T %q Error reading list: %q", "ReadI32", p, trans, err, int32Values)
 	}
-	_, ok := p.(*SimpleJSONProtocol)
+	_, ok := p.(*simpleJSONProtocol)
 	if !ok {
 		if thetype != thetype2 {
 			t.Fatalf("%s: %T %T type %s != type %s", "ReadI32", p, trans, thetype, thetype2)
@@ -497,12 +501,12 @@ func ReadI32(t testing.TB, p Format, trans Transport) {
 	}
 }
 
-func ReadWriteI32(t testing.TB, p Format, trans Transport) {
+func ReadWriteI32(t testing.TB, p Format, trans io.ReadWriter) {
 	WriteI32(t, p, trans)
 	ReadI32(t, p, trans)
 }
 
-func WriteI64(t testing.TB, p Format, trans Transport) {
+func WriteI64(t testing.TB, p Format, trans io.Writer) {
 	thetype := Type(I64)
 	thelen := len(int64Values)
 	p.WriteListBegin(thetype, thelen)
@@ -513,14 +517,14 @@ func WriteI64(t testing.TB, p Format, trans Transport) {
 	p.Flush()
 }
 
-func ReadI64(t testing.TB, p Format, trans Transport) {
+func ReadI64(t testing.TB, p Format, trans io.Reader) {
 	thetype := Type(I64)
 	thelen := len(int64Values)
 	thetype2, thelen2, err := p.ReadListBegin()
 	if err != nil {
 		t.Fatalf("%s: %T %T %q Error reading list: %q", "ReadI64", p, trans, err, int64Values)
 	}
-	_, ok := p.(*SimpleJSONProtocol)
+	_, ok := p.(*simpleJSONProtocol)
 	if !ok {
 		if thetype != thetype2 {
 			t.Fatalf("%s: %T %T type %s != type %s", "ReadI64", p, trans, thetype, thetype2)
@@ -543,12 +547,12 @@ func ReadI64(t testing.TB, p Format, trans Transport) {
 	}
 }
 
-func ReadWriteI64(t testing.TB, p Format, trans Transport) {
+func ReadWriteI64(t testing.TB, p Format, trans io.ReadWriter) {
 	WriteI64(t, p, trans)
 	ReadI64(t, p, trans)
 }
 
-func WriteDouble(t testing.TB, p Format, trans Transport) {
+func WriteDouble(t testing.TB, p Format, trans io.Writer) {
 	doubleValues = []float64{459.3, 0.0, -1.0, 1.0, 0.5, 0.3333, 3.14159, 1.537e-38, 1.673e25, 6.02214179e23, -6.02214179e23, INFINITY.Float64(), NEGATIVE_INFINITY.Float64(), NAN.Float64()}
 	thetype := Type(DOUBLE)
 	thelen := len(doubleValues)
@@ -561,7 +565,7 @@ func WriteDouble(t testing.TB, p Format, trans Transport) {
 
 }
 
-func ReadDouble(t testing.TB, p Format, trans Transport) {
+func ReadDouble(t testing.TB, p Format, trans io.Reader) {
 	doubleValues = []float64{459.3, 0.0, -1.0, 1.0, 0.5, 0.3333, 3.14159, 1.537e-38, 1.673e25, 6.02214179e23, -6.02214179e23, INFINITY.Float64(), NEGATIVE_INFINITY.Float64(), NAN.Float64()}
 	thetype := Type(DOUBLE)
 	thelen := len(doubleValues)
@@ -594,12 +598,12 @@ func ReadDouble(t testing.TB, p Format, trans Transport) {
 	}
 }
 
-func ReadWriteDouble(t testing.TB, p Format, trans Transport) {
+func ReadWriteDouble(t testing.TB, p Format, trans io.ReadWriter) {
 	WriteDouble(t, p, trans)
 	ReadDouble(t, p, trans)
 }
 
-func WriteFloat(t testing.TB, p Format, trans Transport) {
+func WriteFloat(t testing.TB, p Format, trans io.Writer) {
 	floatValues = []float32{459.3, 0.0, -1.0, 1.0, 0.5, 0.3333, 3.14159, 1.537e-38, 1.673e25, 6.02214179e23, -6.02214179e23, INFINITY.Float32(), NEGATIVE_INFINITY.Float32(), NAN.Float32()}
 
 	thetype := Type(FLOAT)
@@ -613,7 +617,7 @@ func WriteFloat(t testing.TB, p Format, trans Transport) {
 
 }
 
-func ReadFloat(t testing.TB, p Format, trans Transport) {
+func ReadFloat(t testing.TB, p Format, trans io.Reader) {
 	floatValues = []float32{459.3, 0.0, -1.0, 1.0, 0.5, 0.3333, 3.14159, 1.537e-38, 1.673e25, 6.02214179e23, -6.02214179e23, INFINITY.Float32(), NEGATIVE_INFINITY.Float32(), NAN.Float32()}
 
 	thetype := Type(FLOAT)
@@ -649,12 +653,12 @@ func ReadFloat(t testing.TB, p Format, trans Transport) {
 
 }
 
-func ReadWriteFloat(t testing.TB, p Format, trans Transport) {
+func ReadWriteFloat(t testing.TB, p Format, trans io.ReadWriter) {
 	WriteFloat(t, p, trans)
 	ReadFloat(t, p, trans)
 }
 
-func WriteString(t testing.TB, p Format, trans Transport) {
+func WriteString(t testing.TB, p Format, trans io.Writer) {
 	thetype := Type(STRING)
 	thelen := len(stringValues)
 	p.WriteListBegin(thetype, thelen)
@@ -665,7 +669,7 @@ func WriteString(t testing.TB, p Format, trans Transport) {
 	p.Flush()
 }
 
-func ReadString(t testing.TB, p Format, trans Transport) {
+func ReadString(t testing.TB, p Format, trans io.Reader) {
 	thetype := Type(STRING)
 	thelen := len(stringValues)
 
@@ -673,7 +677,7 @@ func ReadString(t testing.TB, p Format, trans Transport) {
 	if err != nil {
 		t.Fatalf("%s: %T %T %q Error reading list: %q", "ReadString", p, trans, err, stringValues)
 	}
-	_, ok := p.(*SimpleJSONProtocol)
+	_, ok := p.(*simpleJSONProtocol)
 	if !ok {
 		if thetype != thetype2 {
 			t.Fatalf("%s: %T %T type %s != type %s", "ReadString", p, trans, thetype, thetype2)
@@ -696,18 +700,18 @@ func ReadString(t testing.TB, p Format, trans Transport) {
 	}
 }
 
-func ReadWriteString(t testing.TB, p Format, trans Transport) {
+func ReadWriteString(t testing.TB, p Format, trans io.ReadWriter) {
 	WriteString(t, p, trans)
 	ReadString(t, p, trans)
 }
 
-func WriteBinary(t testing.TB, p Format, trans Transport) {
+func WriteBinary(t testing.TB, p Format, trans io.Writer) {
 	v := protocolBdata
 	p.WriteBinary(v)
 	p.Flush()
 }
 
-func ReadBinary(t testing.TB, p Format, trans Transport) {
+func ReadBinary(t testing.TB, p Format, trans io.Reader) {
 	v := protocolBdata
 	value, err := p.ReadBinary()
 	if err != nil {
@@ -725,12 +729,12 @@ func ReadBinary(t testing.TB, p Format, trans Transport) {
 
 }
 
-func ReadWriteBinary(t testing.TB, p Format, trans Transport) {
+func ReadWriteBinary(t testing.TB, p Format, trans io.ReadWriter) {
 	WriteBinary(t, p, trans)
 	ReadBinary(t, p, trans)
 }
 
-func WriteStruct(t testing.TB, p Format, trans Transport) {
+func WriteStruct(t testing.TB, p Format, trans io.Writer) {
 	v := structTestData
 	p.WriteStructBegin(v.name)
 	p.WriteFieldBegin(v.fields[0].name, v.fields[0].typ, v.fields[0].id)
@@ -752,7 +756,7 @@ func WriteStruct(t testing.TB, p Format, trans Transport) {
 	}
 }
 
-func ReadStruct(t testing.TB, p Format, trans Transport) {
+func ReadStruct(t testing.TB, p Format, trans io.Reader) {
 	v := structTestData
 	_, err := p.ReadStructBegin()
 	if err != nil {
@@ -812,7 +816,7 @@ func ReadStruct(t testing.TB, p Format, trans Transport) {
 	}
 }
 
-func ReadWriteStruct(t testing.TB, p Format, trans Transport) {
-	WriteStruct(t, p, trans)
-	ReadStruct(t, p, trans)
+func ReadWriteStruct(t testing.TB, p Format, buffer io.ReadWriter) {
+	WriteStruct(t, p, buffer)
+	ReadStruct(t, p, buffer)
 }
