@@ -16,16 +16,12 @@
 
 package thrift
 
-import "context"
+import (
+	"context"
+)
 
 // Server is a thrift server
 type Server interface {
-	ServerTransport() ServerTransport
-	// Serve starts the server
-	Serve() error
 	// ServeContext starts the server, and stops it when the context is cancelled
 	ServeContext(ctx context.Context) error
-	// Stop stops the server. This is optional on a per-implementation basis. Not
-	// all servers are required to be cleanly stoppable.
-	Stop() error
 }
