@@ -16,26 +16,12 @@
 
 package thrift
 
-import (
-	"log"
-	"os"
-)
-
-// ServerOptions is options needed to run a thrift server
+// Deprecated: use WrapInterceptorContext
 type ServerOptions struct {
-	quit        chan struct{}
-	log         *log.Logger
 	interceptor Interceptor
 }
 
-// Logger sets the logger used for the server
-func Logger(log *log.Logger) func(*ServerOptions) {
-	return func(server *ServerOptions) {
-		server.log = log
-	}
-}
-
-// WithInterceptor sets the interceptor for the server
+// Deprecated: use WrapInterceptorContext
 func WithInterceptor(interceptor Interceptor) func(*ServerOptions) {
 	return func(server *ServerOptions) {
 		server.interceptor = interceptor
@@ -43,8 +29,13 @@ func WithInterceptor(interceptor Interceptor) func(*ServerOptions) {
 }
 
 func defaultServerOptions() *ServerOptions {
-	return &ServerOptions{
-		quit: make(chan struct{}, 1),
-		log:  log.New(os.Stderr, "", log.LstdFlags),
+	return &ServerOptions{}
+}
+
+func simpleServerOptions(options ...func(*ServerOptions)) *ServerOptions {
+	opts := defaultServerOptions()
+	for _, option := range options {
+		option(opts)
 	}
+	return opts
 }
