@@ -31,7 +31,7 @@ const (
 	TransportIDUnknown TransportID = 0
 	// TransportIDHeader is the header transport
 	TransportIDHeader TransportID = 1
-	// TransportIDRocket is the rocket transport
+	// Deprecated: use TransportIDUpgradeToRocket, this is only used for testing purposes.
 	TransportIDRocket TransportID = 2
 	// TransportIDUpgradeToRocket is the transport that upgrades header to rocket
 	TransportIDUpgradeToRocket TransportID = 3
@@ -47,13 +47,6 @@ type ReadSizeProvider interface {
 	RemainingBytes() (numBytes uint64)
 }
 
-// Transport is an encapsulation of the I/O layer
-type Transport interface {
-	io.ReadWriteCloser
-	Flusher
-	ReadSizeProvider
-}
-
 type stringWriter interface {
 	WriteString(s string) (n int, err error)
 }
@@ -63,7 +56,7 @@ type stringWriter interface {
 // Notably, Socket does not implement this interface, and it is always a mistake to use
 // Socket directly in protocol.
 type RichTransport interface {
-	io.ReadWriter
+	io.ReadWriteCloser
 	io.ByteReader
 	io.ByteWriter
 	stringWriter
