@@ -80,7 +80,7 @@ public class ByteBufTJSONProtocol extends ByteBufTProtocol {
 
   private static final TStruct ANONYMOUS_STRUCT = new TStruct();
 
-  private static final int EMPTY_STRUCT_SIZE = 2;
+  private static final int EMPTY_STRUCT_SIZE = 3;
 
   private static final byte[] getTypeNameForTypeID(byte typeID) throws TException {
     switch (typeID) {
@@ -116,7 +116,7 @@ public class ByteBufTJSONProtocol extends ByteBufTProtocol {
   private static final byte getTypeIDForTypeName(ByteBuf name) throws TException {
     try {
       byte result = TType.STOP;
-      if (name.readByte() > 1) {
+      if (name.readableBytes() > 0) {
         byte n1 = name.readByte();
         byte n2 = name.readByte();
         switch (n1) {
@@ -270,6 +270,10 @@ public class ByteBufTJSONProtocol extends ByteBufTProtocol {
       hasData_ = true;
       return data_;
     }
+
+    protected boolean bytesAvailable() {
+      return getByteBuf().readableBytes() > 0;
+    }
   }
 
   // Stack of nested contexts that we may be in
@@ -306,7 +310,7 @@ public class ByteBufTJSONProtocol extends ByteBufTProtocol {
   // Marked protected to avoid synthetic accessor in JSONListContext.read
   // and JSONPairContext.read
   protected void readJSONSyntaxChar(byte[] b) throws TException {
-    byte ch = getByteBuf().readByte();
+    byte ch = reader_.read();
     if (ch != b[0]) {
       throw new TProtocolException(
           TProtocolException.INVALID_DATA, "Unexpected character:" + (char) ch);
@@ -448,24 +452,24 @@ public class ByteBufTJSONProtocol extends ByteBufTProtocol {
 
   private void writeJSONObjectStart() throws TException {
     context_.write();
-    trans_.write(LBRACE);
+    getByteBuf().writeBytes(LBRACE);
     pushContext(new JSONPairContext());
   }
 
   private void writeJSONObjectEnd() throws TException {
     popContext();
-    trans_.write(RBRACE);
+    getByteBuf().writeBytes(RBRACE);
   }
 
   private void writeJSONArrayStart() throws TException {
     context_.write();
-    trans_.write(LBRACKET);
+    getByteBuf().writeBytes(LBRACKET);
     pushContext(new JSONListContext());
   }
 
   private void writeJSONArrayEnd() throws TException {
     popContext();
-    trans_.write(RBRACKET);
+    getByteBuf().writeBytes(RBRACKET);
   }
 
   @Override
@@ -665,6 +669,9 @@ public class ByteBufTJSONProtocol extends ByteBufTProtocol {
   private String readJSONNumericChars() throws TException {
     StringBuilder strbld = new StringBuilder();
     while (true) {
+      if (!reader_.bytesAvailable()) {
+        break;
+      }
       byte ch = reader_.peek();
       if (!isJSONNumeric(ch)) {
         break;
