@@ -174,7 +174,9 @@ class CppServerObserver : public TServerObserver {
   explicit CppServerObserver(object serverObserver)
       : observer_(serverObserver) {}
 
-  void connAccepted(const wangle::TransportInfo& /* info */) override {
+  void connAccepted(
+      const wangle::TransportInfo& /* info */,
+      const TServerObserver::ConnectionInfo& /* connInfo */) override {
     this->call("connAccepted");
   }
   void connDropped() override { this->call("connDropped"); }
@@ -185,7 +187,9 @@ class CppServerObserver : public TServerObserver {
   void tlsResumption() override { this->call("tlsResumption"); }
   void taskKilled() override { this->call("taskKilled"); }
   void taskTimeout() override { this->call("taskTimeout"); }
-  void serverOverloaded() override { this->call("serverOverloaded"); }
+  void serverOverloaded(apache::thrift::LoadShedder /*loadShedder*/) override {
+    this->call("serverOverloaded");
+  }
   void receivedRequest(const std::string* /*method*/) override {
     this->call("receivedRequest");
   }
