@@ -24,7 +24,17 @@ from thrift.py3.converter import to_py3_struct
 from thrift.py3.types import BadEnum, Struct
 
 
+# Reimplementing brokenInAutoMigrate using convertible because including testing.thrift and
+# convertible.thrift in the same binary creates linker errors
+def brokenInAutoMigrate():  # pyre-ignore[3] unittest isn't very well typed
+    # kinda janky way of testing if we're in auto migrate mode or not
+    if py3_types.Simple == python_types.Simple:
+        return unittest.expectedFailure
+    return lambda func: func
+
+
 class PyDeprecatedToPy3ConverterTest(unittest.TestCase):
+    @brokenInAutoMigrate()
     def test_simple(self) -> None:
         simple = py_deprecated_types.Simple(
             intField=42,
@@ -43,6 +53,7 @@ class PyDeprecatedToPy3ConverterTest(unittest.TestCase):
         self.assertEqual(simple.color, py3_types.Color.GREEN)
         self.assertEqual(simple.name_, "myname")
 
+    @brokenInAutoMigrate()
     def test_nested(self) -> None:
         nested = py_deprecated_types.Nested(
             simpleField=py_deprecated_types.Simple(
@@ -93,21 +104,25 @@ class PyDeprecatedToPy3ConverterTest(unittest.TestCase):
             nested.colorToSimpleMap[py3_types.Color.BLUE].color, py3_types.Color.BLUE
         )
 
+    @brokenInAutoMigrate()
     def test_simple_union(self) -> None:
         simple_union = py_deprecated_types.Union(intField=42)._to_py3()
         self.assertEqual(simple_union.type, py3_types.Union.Type.intField)
         self.assertEqual(simple_union.value, 42)
 
+    @brokenInAutoMigrate()
     def test_union_with_py3_name_annotation(self) -> None:
         simple_union = py_deprecated_types.Union(name="myname")._to_py3()
         self.assertEqual(simple_union.type, py3_types.Union.Type.name_)
         self.assertEqual(simple_union.value, "myname")
 
+    @brokenInAutoMigrate()
     def test_union_with_containers(self) -> None:
         union_with_list = py_deprecated_types.Union(intList=[1, 2, 3])._to_py3()
         self.assertEqual(union_with_list.type, py3_types.Union.Type.intList)
         self.assertEqual(union_with_list.value, [1, 2, 3])
 
+    @brokenInAutoMigrate()
     def test_complex_union(self) -> None:
         complex_union = py_deprecated_types.Union(
             simpleField=py_deprecated_types.Simple(
@@ -122,6 +137,7 @@ class PyDeprecatedToPy3ConverterTest(unittest.TestCase):
         self.assertEqual(complex_union.type, py3_types.Union.Type.simple_)
         self.assertEqual(complex_union.simple_.intField, 42)
 
+    @brokenInAutoMigrate()
     def test_optional_defaults(self) -> None:
         converted = py_deprecated_types.OptionalDefaultsStruct()._to_py3()
         # pyre-fixme[6]: Expected `HasIsSet[Variable[thrift.py3.py3_types._T]]` for 1st
@@ -170,6 +186,7 @@ class PyDeprecatedToPy3ConverterTest(unittest.TestCase):
         self.assertEqual(to.type, py3_types.Potahto.Type.to)
         self.assertEqual(to.value, True)
 
+    @brokenInAutoMigrate()
     def test_py_bad_enum(self) -> None:
         simple = py_deprecated_types.Simple(
             intField=42,
@@ -212,9 +229,11 @@ class PythonToPy3ConverterTest(unittest.TestCase):
         self.assertIsInstance(simple.empty, BadEnum)
         self.assertEqual(int(simple.empty), 0)
 
+    @brokenInAutoMigrate()
     def test_simple(self) -> None:
         self.assert_simple(self.make_simple_python()._to_py3())
 
+    @brokenInAutoMigrate()
     def test_simple_capi(self) -> None:
         self.assert_simple(py3_types.Simple.from_python(self.make_simple_python()))
 
@@ -263,6 +282,7 @@ class PythonToPy3ConverterTest(unittest.TestCase):
             nested.colorToSimpleMap[py3_types.Color.BLUE].color, py3_types.Color.BLUE
         )
 
+    @brokenInAutoMigrate()
     def test_nested_capi(self) -> None:
         self.assertEqual(
             self.make_nested_python()._to_py3(),
@@ -274,6 +294,7 @@ class PythonToPy3ConverterTest(unittest.TestCase):
         self.assertEqual(simple_union.type, py3_types.Union.Type.intField)
         self.assertEqual(simple_union.value, 42)
 
+    @brokenInAutoMigrate()
     def test_simple_union_capi(self) -> None:
         simple_union = py3_types.Union.from_python(python_types.Union(intField=42))
         self.assertEqual(simple_union.type, py3_types.Union.Type.intField)
@@ -284,6 +305,7 @@ class PythonToPy3ConverterTest(unittest.TestCase):
         self.assertEqual(simple_union.type, py3_types.Union.Type.name_)
         self.assertEqual(simple_union.value, "myname")
 
+    @brokenInAutoMigrate()
     def test_union_with_py3_name_annotation_capi(self) -> None:
         simple_union = py3_types.Union.from_python(python_types.Union(name_="myname"))
         self.assertEqual(simple_union.type, py3_types.Union.Type.name_)
@@ -294,6 +316,7 @@ class PythonToPy3ConverterTest(unittest.TestCase):
         self.assertEqual(union_with_list.type, py3_types.Union.Type.intList)
         self.assertEqual(union_with_list.value, [1, 2, 3])
 
+    @brokenInAutoMigrate()
     def test_union_with_containers_capi(self) -> None:
         union_with_list = py3_types.Union.from_python(
             python_types.Union(intList=[1, 2, 3])
@@ -308,6 +331,7 @@ class PythonToPy3ConverterTest(unittest.TestCase):
         self.assertEqual(complex_union.type, py3_types.Union.Type.simple_)
         self.assertEqual(complex_union.simple_.intField, 42)
 
+    @brokenInAutoMigrate()
     def test_complex_union_capi(self) -> None:
         complex_union = py3_types.Union.from_python(
             python_types.Union(
@@ -317,6 +341,7 @@ class PythonToPy3ConverterTest(unittest.TestCase):
         self.assertEqual(complex_union.type, py3_types.Union.Type.simple_)
         self.assertEqual(complex_union.simple_.intField, 42)
 
+    @brokenInAutoMigrate()
     def test_optional_defaults(self) -> None:
         converted = python_types.OptionalDefaultsStruct()._to_py3()
         # pyre-fixme[6]: Expected `HasIsSet[Variable[thrift.py3.py3_types._T]]` for 1st
