@@ -22,6 +22,10 @@ import typing
 
 from unittest import IsolatedAsyncioTestCase
 
+from thrift.lib.python.client.test.client_event_handler.helper import (
+    TestHelper as ClientEventHandlerTestHelper,
+)
+
 from thrift.lib.python.client.test.event_handler_helper import (
     client_handler_that_throws,
 )
@@ -168,7 +172,7 @@ class AsyncClientTests(IsolatedAsyncioTestCase):
                 self.assertEqual(3, sum)
 
     async def test_transport_error(self) -> None:
-        async with get_client(TestService, path="/no/where") as client:
+        async with get_client(TestService, host="localhost", port=1) as client:
             with self.assertRaises(TransportError) as ex:
                 await client.add(1, 2)
             self.assertEqual(TransportErrorType.UNKNOWN, ex.exception.type)
@@ -306,7 +310,7 @@ class AsyncClientTests(IsolatedAsyncioTestCase):
         self,
     ) -> None:
         with HijackTestHelper():
-            async with get_client(TestService, path="/no/where") as client:
+            async with get_client(TestService, host="localhost", port=1) as client:
                 with self.assertRaises(HijackTestException) as context:
                     options = RpcOptions()
                     options.timeout = 12.5
@@ -317,7 +321,7 @@ class AsyncClientTests(IsolatedAsyncioTestCase):
         self,
     ) -> None:
         with HijackTestHelper():
-            async with get_client(TestService, path="/no/where") as client:
+            async with get_client(TestService, host="localhost", port=1) as client:
                 with self.assertRaises(HijackTestException) as context:
                     await client.add(1, 2)
                 self.assertEqual(context.exception.timeout, 0.0)
@@ -344,6 +348,16 @@ class AsyncClientTests(IsolatedAsyncioTestCase):
         self.assertTrue(cb1.triggered)
         self.assertTrue(cb2.triggered)
 
+    async def test_client_event_handler(self) -> None:
+        test_helper: ClientEventHandlerTestHelper = ClientEventHandlerTestHelper()
+        self.assertFalse(test_helper.is_handler_called())
+        async with test_helper.get_async_client(TestService, port=1) as cli:
+            try:
+                await cli.noop()
+            except TransportError:
+                pass
+            self.assertTrue(test_helper.is_handler_called())
+
     async def test_exception_in_client_event_handler(self) -> None:
         async with server_in_event_loop() as addr:
             with self.assertRaises(RuntimeError):
