@@ -259,7 +259,7 @@ class OmniClientTest : public ::testing::Test {
   template <class S, typename T>
   void testContains(folly::IOBuf buf, const T& expected) {
     std::string expectedStr = S::template serialize<std::string>(expected);
-    std::string result = buf.moveToFbString().toStdString();
+    std::string result = buf.to<std::string>();
     // Contains instead of equals because of the envelope around the response.
     EXPECT_THAT(result, testing::HasSubstr(expectedStr));
   }
