@@ -20,6 +20,10 @@ from __future__ import annotations
 import time
 import unittest
 
+from thrift.lib.python.client.test.client_event_handler.helper import (
+    TestHelper as ClientEventHandlerTestHelper,
+)
+
 from thrift.lib.python.client.test.event_handler_helper import (
     addEventHandler,
     client_handler_that_throws,
@@ -116,7 +120,7 @@ class SyncClientTests(unittest.TestCase):
                 self.assertEqual(3, client.add(1, 2))
 
     def test_transport_error(self) -> None:
-        with get_sync_client(TestService, path="/no/where") as client:
+        with get_sync_client(TestService, host="localhost", port=1) as client:
             with self.assertRaises(TransportError) as ex:
                 client.add(1, 2)
             self.assertEqual(TransportErrorType.UNKNOWN, ex.exception.type)
@@ -142,7 +146,7 @@ class SyncClientTests(unittest.TestCase):
         self,
     ) -> None:
         with HijackTestHelper():
-            with get_sync_client(TestService, path="/no/where") as client:
+            with get_sync_client(TestService, host="localhost", port=1) as client:
                 with self.assertRaises(HijackTestException) as context:
                     options = RpcOptions()
                     options.timeout = 12.5
@@ -153,7 +157,7 @@ class SyncClientTests(unittest.TestCase):
         self,
     ) -> None:
         with HijackTestHelper():
-            with get_sync_client(TestService, path="/no/where") as client:
+            with get_sync_client(TestService, host="localhost", port=1) as client:
                 with self.assertRaises(HijackTestException) as context:
                     client.add(1, 2)
                 self.assertEqual(context.exception.timeout, 0.0)
@@ -177,6 +181,16 @@ class SyncClientTests(unittest.TestCase):
         self.assertTrue(cb1.triggered)
         self.assertTrue(cb2.triggered)
 
+    async def test_client_event_handler(self) -> None:
+        test_helper: ClientEventHandlerTestHelper = ClientEventHandlerTestHelper()
+        self.assertFalse(test_helper.is_handler_called())
+        with test_helper.get_sync_client(TestService, port=1) as cli:
+            try:
+                cli.noop()
+            except TransportError:
+                pass
+            self.assertTrue(test_helper.is_handler_called())
+
     def test_exception_in_client_event_handler(self) -> None:
         with self.assertRaises(RuntimeError):
             with client_handler_that_throws():
