@@ -83,6 +83,15 @@ class StructTests(unittest.TestCase):
         dif_int = copy.copy(x.an_int)
         self.assertEqual(x.an_int, dif_int)
 
+    def test_no_dict(self) -> None:
+        with self.assertRaises(AttributeError):
+            easy().__dict__
+        # TODO: make it a tuple instead of a list
+        # self.assertIsInstance(easy().__class__.__slots__, tuple)
+        # pyre-ignore[16]
+        self.assertIs(easy().__class__.__slots__, easy().__slots__)
+        self.assertIs(easy(val=5).__slots__, easy(name="yo").__slots__)
+
     def test_hashability(self) -> None:
         hash(easy())
         hash(EmptyStruct())
