@@ -70,6 +70,17 @@ inline T convInt(V v) {
   LOG(FATAL) << "int out of range";
 }
 
+/**
+ * Creates a new "union tuple" initialized for an empty union.
+ *
+ * The returned tuple has 2 items:
+ *  1. The enum value of the field that is currently set for this union, or
+ *     the special "empty" (with value 0) if the union is empty.
+ *  2. The value of the current field for this enum, or Py_None if the union
+ *     is empty.
+ *
+ * The tuple returned by this function always has values `(0, Py_None)`.
+ */
 PyObject* createUnionTuple();
 
 /***
@@ -399,7 +410,7 @@ inline void* setList(void* objectPtr) {
   return objectPtr;
 }
 
-// Sets the Python object pointed to by `object` to an empty Python `set`
+// Sets the Python object pointed to by `objectPtr` to an empty Python `set`
 inline void* setMutableSet(void* objectPtr) {
   if (!setPyObject(objectPtr, UniquePyObjectPtr{PySet_New(nullptr)})) {
     THRIFT_PY3_CHECK_ERROR();
@@ -407,6 +418,15 @@ inline void* setMutableSet(void* objectPtr) {
   return objectPtr;
 }
 
+// Sets the Python object pointed to by `objectPtr` to an empty Python
+// `dictionary`
+inline void* setMutableMap(void* objectPtr) {
+  if (!setPyObject(objectPtr, UniquePyObjectPtr{PyDict_New()})) {
+    THRIFT_PY3_CHECK_ERROR();
+  }
+  return objectPtr;
+}
+
 class ListTypeInfo {
  public:
   static std::uint32_t size(const void* object) {
@@ -717,6 +737,99 @@ class MapTypeInfo {
   const detail::TypeInfo typeinfo_;
 };
 
+class MutableMapTypeInfo {
+ public:
+  static std::uint32_t size(const void* object) {
+    return folly::to<std::uint32_t>(
+        PyDict_Size(const_cast<PyObject*>(toPyObject(object))));
+  }
+
+  static void clear(void* objectPtr) { setMutableMap(objectPtr); }
+
+  /**
+   * Deserializes a dict (with `mapSize` key/value pairs) into `objectPtr`.
+   *
+   * The function pointer arguments (`keyReader` and `valueReader`) are expected
+   * to take two (type-erased, i.e. `void*`) arguments:
+   *   1. The given `context`
+   *   2. A pointer to a `PyObject*`, that should be set by the function to the
+   *      (next) key and value, respectively.
+   *
+   * @param context Will be passed to every call to `keyReader` and
+   *        `valueReader`.
+   * @param objectPtr Pointer to a `PyObject*`, that will be set to a new
+   *        `PyDict` instance containing `mapSize` elements whose keys and
+   *        values are obtained by calling the given `*Reader` functions.
+   */
+  static void read(
+      const void* context,
+      void* objectPtr,
+      std::uint32_t mapSize,
+      void (*keyReader)(const void* context, void* key),
+      void (*valueReader)(const void* context, void* val));
+
+  /**
+   * Serializes the dict given in `object`.
+   *
+   * The `writer` function will be called for every item in the dict pointed by
+   * the given `object`. It is expected to take the arguments described below,
+   * and return the number of bytes written:
+   *   1. The given `context`
+   *   2 & 3. `PyObject**` pointing to the next key and value from dict,
+   * respectively.
+   *
+   * @param context Passed to `writer` on every call.
+   * @param object Input `PyObject*`, holds the `PyDict` to serialize.
+   *
+   * @return Total number of bytes written.
+   */
+  static size_t write(
+      const void* context,
+      const void* object,
+      bool protocolSortKeys,
+      size_t (*writer)(
+          const void* context, const void* keyElem, const void* valueElem));
+
+  /**
+   * Deserializes a single key, value pair (using the given function pointers)
+   * and adds it to the given dict.
+   *
+   * See `read()` for the expectations on `keyReader` and `valueReader`.
+   *
+   * @param context Passed to `keyReader` and `valueReader` on every call.
+   * @param objectPtr Pointer to a `PyObject*` that holds a dict, to which the
+   *        new key/value pair will be added.
+   */
+  static void consumeElem(
+      const void* context,
+      void* object,
+      void (*keyReader)(const void* /*context*/, void* /*val*/),
+      void (*valueReader)(const void* /*context*/, void* /*val*/));
+
+  explicit MutableMapTypeInfo(
+      const detail::TypeInfo* keyInfo, const detail::TypeInfo* valInfo)
+      : tableBasedSerializerMapFieldExt_{
+            /* .keyInfo */ keyInfo,
+            /* .valInfo */ valInfo,
+            /* .size */ size,
+            /* .clear */ clear,
+            /* .consumeElem */ consumeElem,
+            /* .readSet */ read,
+            /* .writeSet */ write,
+        },
+        tableBasedSerializerTypeinfo_{
+            protocol::TType::T_MAP,
+            getStruct,
+            reinterpret_cast<detail::VoidFuncPtr>(setMutableMap),
+            &tableBasedSerializerMapFieldExt_,
+        } {}
+  const detail::TypeInfo* get() const { return &tableBasedSerializerTypeinfo_; }
+
+ private:
+  const detail::MapFieldExt tableBasedSerializerMapFieldExt_;
+  const detail::TypeInfo tableBasedSerializerTypeinfo_;
+};
+
 using FieldValueMap = std::unordered_map<int16_t, PyObject*>;
 
 /**
