@@ -25,15 +25,15 @@ class TestTMemoryBuffer(unittest.TestCase):
         buf = TMemoryBuffer(b"hello")
         data = buf.read(5)
         buf.write(b"world")
-        self.assertEquals(data, b"hello")
-        self.assertEquals(buf.getvalue(), b"world")
+        self.assertEqual(data, b"hello")
+        self.assertEqual(buf.getvalue(), b"world")
 
     def testNoInitialValue(self):
         buf = TMemoryBuffer()
         data = buf.read(5)
         buf.write(b"world")
-        self.assertEquals(data, b"")
-        self.assertEquals(buf.getvalue(), b"world")
+        self.assertEqual(data, b"")
+        self.assertEqual(buf.getvalue(), b"world")
 
     def testClose(self):
         buf = TMemoryBuffer(b"hello")
