@@ -80,11 +80,7 @@ module.exports = {
         type: 'doc',
         id: 'idl/index',
       },
-      items: [
-        'idl/field-qualifiers',
-        'idl/annotations',
-        'idl/mixins',
-      ],
+      items: ['idl/field-qualifiers', 'idl/annotations', 'idl/mixins'],
     },
     {
       type: 'category',
@@ -106,7 +102,10 @@ module.exports = {
             type: 'doc',
             id: 'features/serialization/index',
           },
-          items: ['features/serialization/protocols'],
+          items: [
+            'features/serialization/protocols',
+            'features/serialization/cursor',
+          ],
         },
         'features/operators',
         'features/universal-name',
@@ -312,6 +311,7 @@ module.exports = {
         items: [
           'fb/server/howto',
           'fb/server/server-lifecycle',
+          'fb/server/server-modules',
           'fb/server/background-tasks',
           'fb/server/flavors-of-main',
           'fb/server/components',
@@ -323,7 +323,10 @@ module.exports = {
               type: 'doc',
               id: 'fb/server/interface/index',
             },
-            items: ['fb/server/interface/rocket'],
+            items: [
+              'fb/server/interface/rocket',
+              'fb/server/interface/server-transport-exception-contract',
+            ],
           },
           'fb/server/channels',
           'fb/server/threading-models',
@@ -368,8 +371,7 @@ module.exports = {
                   type: 'doc',
                   id: 'fb/server/overload-protection/dls/index',
                 },
-                items: [
-                ],
+                items: [],
               },
               // 'fb/server/overload-protection/dls/index',
               'fb/server/overload-protection/adaptive-concurrency',
@@ -385,6 +387,16 @@ module.exports = {
             },
             items: ['fb/server/resource-pools/api'],
           },
+          {
+            type: 'category',
+            label: 'Quota Management',
+            link: {
+              type: 'doc',
+              id: 'fb/server/quota-management/index',
+            },
+            items: [],
+          },
+          'fb/server/stream-graceful-shutdown',
         ],
       },
       {
@@ -453,7 +465,18 @@ module.exports = {
               'fb/troubleshooting/exception-handling/error-classification',
             ],
           },
-          'fb/troubleshooting/debugging-tools',
+          {
+            type: 'category',
+            label: 'Debugging Tools',
+            link: {
+              type: 'doc',
+              id: 'fb/troubleshooting/debugging-tools',
+            },
+            items: [
+              'fb/troubleshooting/debuggingtools/thriftdbg-info',
+              'fb/troubleshooting/debuggingtools/thriftdbg-sendRequest',
+            ],
+          },
           'fb/troubleshooting/fuzzer',
           {
             type: 'category',
@@ -590,9 +613,7 @@ module.exports = {
         type: 'doc',
         id: 'glossary/index',
       },
-      items: [
-        'glossary/kinds-of-types',
-      ],
+      items: ['glossary/kinds-of-types'],
     },
   ],
 };
